<?xml version="1.0" encoding="utf-8"?>
<!--
    
    ****************************************************************************************
    
    European Union Public Licence - EUPL v.1.1
    
    Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl
    
    Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
    European Commission - subsequent versions of the EUPL (the "Licence");
    You may not use this work except in compliance with the Licence.
    You may obtain a copy of the Licence at:
    
    http://ec.europa.eu/idabc/eupl
    
    Unless required by applicable law or agreed to in writing, software distributed
    under the Licence is distributed on an "AS IS" basis, 
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the Licence for the specific language governing permissions and limitations
    under the Licence.
    
    ****************************************************************************************
    
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
    xmlns:functx="http://www.functx.com"
    xmlns:tns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
    xmlns:xlink="http://www.w3.org/1999/xlink"
    xsi:schemaLocation="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd                  http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias" version="2.0"
    exclude-result-prefixes="xsl xsi functx tns xlink xs axsl">

    <xsl:strip-space elements="*"/>

    <xsl:output method="xml" encoding="utf-8" indent="yes"/>

    <!-- 
        ************************************************
        This XSLT generates the msg file to be used by CPA CreationToolkit version 3.1
        By: Ernst Jan van Nigtevecht
        November 2008
        
        Specifiy the desired output. There are three options:
        all                     : The full msg specification is generated. This is the default.
        services          : only the services section is generated.
        parameters    : only the parameter values are generated (to be used by the participant)
        ************************************************
    -->
    <xsl:param name="options">services</xsl:param>

    <xsl:function name="functx:getBusinessTransactionAndDocumentID" as="xs:string?"
        xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:variable name="subStringBT">
            <xsl:sequence select="substring-after($arg,'_BT')"/>
        </xsl:variable>
        <xsl:variable name="subStringBD">
            <xsl:sequence select="substring-after($subStringBT,'_BD')"/>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="($subStringBT!='') and ($subStringBD!='')">
                <xsl:value-of select="concat('BT',$subStringBT)"/>
            </xsl:when>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="functx:capitalize-first" as="xs:string?"
        xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:sequence select="concat(upper-case(substring($arg,1,1)),substring($arg,2))"/>
    </xsl:function>

    <xsl:function name="functx:createID" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:sequence
            select="string-join((for $word in tokenize($arg,'[\s:./\-\(\)]+') return functx:capitalize-first($word)),'')"/>
    </xsl:function>

    <xsl:template match="/">
                <specification>
                    <xsl:comment>This file is generated from existing CPA's.</xsl:comment>
                            <services>
                                <xsl:apply-templates select="//tns:PartyInfo[1]/tns:CollaborationRole"
                                    mode="services"/>
                            </services>
                </specification>
    </xsl:template>

    <!-- 
        ************************************************
        Generate the services section
        ************************************************
    -->
    <xsl:template match="tns:PartyInfo/tns:CollaborationRole" mode="services">
        <service
            serviceId="sid_{functx:createID(tns:ServiceBinding/tns:Service/descendant-or-self::text())}"
            process="{tns:ProcessSpecification/@tns:name}"
            service="{tns:ServiceBinding/tns:Service/descendant-or-self::text()}">
            <xsl:choose>
                <xsl:when test="tns:ServiceBinding/tns:Service/@tns:type">
                    <xsl:attribute name="servicetype">
                        <xsl:value-of select="tns:ServiceBinding/tns:Service/@tns:type"/>
                    </xsl:attribute>
                </xsl:when>
            </xsl:choose>
            <xsl:apply-templates select="tns:ServiceBinding/tns:CanSend">
                <xsl:with-param name="fromRole" select="tns:Role/@tns:name"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="tns:ServiceBinding/tns:CanReceive">
                <xsl:with-param name="fromRole" select="tns:Role/@tns:name"/>
            </xsl:apply-templates>
        </service>
    </xsl:template>

    <xsl:template match="tns:ServiceBinding/tns:CanSend">
        <xsl:param name="fromRole"/>
        <xsl:variable name="otherPartyActionBindingId"
            select="tns:OtherPartyActionBinding/descendant-or-self::text()"/>
        <xsl:variable name="toRole"
            select="//tns:PartyInfo[2]/tns:CollaborationRole/tns:ServiceBinding/tns:CanReceive/tns:ThisPartyActionBinding[@tns:id=$otherPartyActionBindingId]/parent::node()/parent::node()/parent::node()/tns:Role/@tns:name"/>
        <message fromPartyname="{//tns:PartyInfo[1]/@tns:partyName}" toPartyname="{//tns:PartyInfo[2]/@tns:partyName}" fromRole="{$fromRole}" toRole="{$toRole}">
            <xsl:call-template name="BTandBD">
                <xsl:with-param name="thisPartyActionBindingId"
                    select="tns:ThisPartyActionBinding/@tns:id"/>
            </xsl:call-template>
            <xsl:call-template name="determineProfile">
                <xsl:with-param name="channelId"
                    select="tns:ThisPartyActionBinding/tns:ChannelId/descendant-or-self::text()"/>
            </xsl:call-template>
            <xsl:value-of select="tns:ThisPartyActionBinding/@tns:action"/>
        </message>
    </xsl:template>

    <xsl:template match="tns:ServiceBinding/tns:CanReceive">
        <xsl:param name="fromRole"/>
        <xsl:variable name="otherPartyActionBindingId"
            select="tns:OtherPartyActionBinding/descendant-or-self::text()"/>
        <xsl:variable name="toRole"
            select="//tns:PartyInfo[2]/tns:CollaborationRole/tns:ServiceBinding/tns:CanSend/tns:ThisPartyActionBinding[@tns:id=$otherPartyActionBindingId]/parent::node()/parent::node()/parent::node()/tns:Role/@tns:name"/>
        <message fromPartyname="{//tns:PartyInfo[2]/@tns:partyName}" toPartyname="{//tns:PartyInfo[1]/@tns:partyName}" fromRole="{$toRole}" toRole="{$fromRole}">
            <xsl:call-template name="BTandBD">
                <xsl:with-param name="thisPartyActionBindingId"
                    select="tns:ThisPartyActionBinding/@tns:id"/>
            </xsl:call-template>
            <xsl:call-template name="determineProfile">
                <xsl:with-param name="channelId"
                    select="tns:ThisPartyActionBinding/tns:ChannelId/descendant-or-self::text()"/>
            </xsl:call-template>
            <xsl:value-of select="tns:ThisPartyActionBinding/@tns:action"/>
        </message>
    </xsl:template>

    <!-- 
        ************************************************
        Generate the rolebindings section
        ************************************************
    -->
    <xsl:template match="//tns:PartyInfo" mode="rolebindings">
        <binding name="{functx:createID(@tns:partyName)}"
            parameterId="pid_{functx:createID(@tns:partyName)}">
            <xsl:apply-templates select="tns:CollaborationRole" mode="roles"/>
        </binding>
    </xsl:template>

    <xsl:template match="tns:CollaborationRole" mode="roles">
        <role
            serviceId="sid_{functx:createID(tns:ServiceBinding/tns:Service/descendant-or-self::text())}">
            <xsl:value-of select="tns:Role/@tns:name"/>
        </role>
    </xsl:template>

    <!-- 
        ************************************************
        Generate the parameters section
        ************************************************
    -->
    <xsl:template match="//tns:PartyInfo" mode="parameters">
        <xsl:choose>
            <xsl:when test="$options='all'">
                <xsl:element name="group">
                    <xsl:attribute name="parameterId" select="concat('pid_',functx:createID(@tns:partyName))"/>
                    <xsl:call-template name="createParameterList"/>
                </xsl:element>
            </xsl:when>
            <xsl:when test="$options='parameters'">
                <xsl:element name="parameters">
                    <xsl:attribute name="name" select="functx:createID(@tns:partyName)"/>
                    <xsl:call-template name="createParameterList"/>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise><xsl:comment>You did not specify a valid option. Valid options are: all, service, parameters</xsl:comment></xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="createParameterList">
        <parameter name="CPAID">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:value-of select="parent::node()/@tns:cpaid"/>
        </parameter>
        <parameter name="PartyName">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:value-of select="@tns:partyName"/>
        </parameter>
        <parameter name="PartyRef">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:value-of select="tns:PartyRef/@xlink:href"/>
        </parameter>
        <parameter name="PartyIdList">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:apply-templates select="tns:PartyId"/>
        </parameter>
        <parameter name="PartyIdType">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:value-of select="tns:PartyId[1]/@tns:type"/>
        </parameter>
        <parameter name="ConditionalEndpointUri">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'true'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="$options='parameters'">
                    <xsl:comment>Always check the values below! </xsl:comment>
                    <xsl:comment>The following EndpointUri was used by this partner (included here for verification purposes only):
                        <xsl:value-of select="tns:Transport/tns:TransportReceiver/tns:Endpoint/@tns:uri"/>
                    </xsl:comment>
                    <condition name="local" switch="false">http://dit-staat-niet-in-de-cpa.nl/zelf-invullen</condition>
                    <condition name="default" switch="true">
                        <xsl:value-of select="tns:Transport/tns:TransportReceiver/tns:Endpoint/@tns:uri"/>
                    </condition>
                </xsl:when>
            </xsl:choose>
        </parameter>
        <xsl:apply-templates select="(tns:DocExchange/tns:ebXMLSenderBinding/tns:ReliableMessaging)[1]"/>
        <xsl:apply-templates select="tns:Certificate"/>
    </xsl:template>
    <!-- 
        ************************************************
        Helper templates
        ************************************************
    -->
    <xsl:template name="BTandBD">
        <xsl:param name="thisPartyActionBindingId"/>
        <xsl:variable name="subString"
            select="functx:getBusinessTransactionAndDocumentID($thisPartyActionBindingId)"/>
        <xsl:choose>
            <xsl:when test="$subString">
                <xsl:attribute name="id" select="$subString"/>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="determineProfile">
        <xsl:param name="channelId"/>
        <xsl:choose>
            <xsl:when test="contains($channelId,'ProfileBestEffort')">
                <xsl:attribute name="profile" select="'jab-besteffort'"/>
            </xsl:when>
            <xsl:when test="contains($channelId,'ProfileReliableMessaging')">
                <xsl:attribute name="profile" select="'jab-basis'"/>
            </xsl:when>
            <xsl:when test="contains($channelId,'ProfileHoogReliableMessaging')">
                <xsl:attribute name="profile" select="'jab-hoog'"/>
            </xsl:when>
            <xsl:when test="contains($channelId,'ProfileHoogBestEffort')">
                <xsl:attribute name="profile" select="'jab-besteffort'"/>
                <xsl:attribute name="security" select="'payload'"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:attribute name="profile" select="$channelId"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="tns:PartyId">
        <value>
            <xsl:value-of select="descendant-or-self::text()"></xsl:value-of>
        </value>
    </xsl:template>
    
    <xsl:template match="tns:Certificate">
        <xsl:variable name="certType" select="substring-after(@tns:certId,'_')"></xsl:variable>
        <parameter>
            <xsl:attribute name="name" select="$certType"/>
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:copy-of copy-namespaces="no" select="child::node()"/>
        </parameter>
    </xsl:template>

    <xsl:template match="tns:DocExchange/tns:ebXMLSenderBinding/tns:ReliableMessaging">
        <xsl:variable name="retries" select="tns:Retries/descendant-or-self::text()"></xsl:variable>
        <xsl:variable name="retryInterval" select="tns:RetryInterval/descendant-or-self::text()"></xsl:variable>
        <xsl:variable name="persistduration" select="parent::node()/tns:PersistDuration/descendant-or-self::text()"></xsl:variable>
        <xsl:choose>
            <xsl:when test="$retries!='8'">
                <parameter name="Retries">
                    <xsl:choose>
                        <xsl:when test="$options='all'">
                            <xsl:attribute name="required" select="'false'"/>
                        </xsl:when>
                    </xsl:choose>
                    <xsl:value-of select="$retries"/>
                </parameter>
            </xsl:when>
        </xsl:choose>
        <xsl:choose>
            <xsl:when test="$retryInterval!='PT3H'">
                <parameter name="RetryInterval">
                    <xsl:choose>
                        <xsl:when test="$options='all'">
                            <xsl:attribute name="required" select="'false'"/>
                        </xsl:when>
                    </xsl:choose>
                    <xsl:value-of select="$retryInterval"/>
                </parameter>
            </xsl:when>
        </xsl:choose>
        <xsl:choose>
            <xsl:when test="$persistduration!='P1D'">
                <parameter name="PersistDuration">
                    <xsl:choose>
                        <xsl:when test="$options='all'">
                            <xsl:attribute name="required" select="'false'"/>
                        </xsl:when>
                    </xsl:choose>
                    <xsl:value-of select="$persistduration"/>
                </parameter>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>