<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
    xmlns:functx="http://www.functx.com"
    xmlns:tns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
    xmlns:xlink="http://www.w3.org/1999/xlink"
    xmlns:msgpar="http://www.sonnenglanz.net/2010/07/09/smf.services.2.0.xsd"
    xsi:schemaLocation="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd                  http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias" version="2.0"
    exclude-result-prefixes="xsl xsi functx tns xlink xs axsl msgpar">

    <xsl:strip-space elements="*"/>

    <xsl:output method="xml" encoding="utf-8" indent="yes"/>

    <!-- 
        ************************************************
        This XSLT generates the msg file to be used by CPA CreationToolkit version 3.1
        By: Ernst Jan van Nigtevecht
        November 2008
        
        Specifiy the desired output. There are three options:
        parameters    : only the parameter values are generated (to be used by the participant)
        ************************************************
    -->
    <xsl:param name="options">parameters</xsl:param>

    <xsl:function name="functx:getBusinessTransactionAndDocumentID" as="xs:string?"
        xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:variable name="subStringBT">
            <xsl:sequence select="substring-after($arg,'_BT')"/>
        </xsl:variable>
        <xsl:variable name="subStringBD">
            <xsl:sequence select="substring-after($subStringBT,'_BD')"/>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="($subStringBT!='') and ($subStringBD!='')">
                <xsl:value-of select="concat('BT',$subStringBT)"/>
            </xsl:when>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="functx:capitalize-first" as="xs:string?"
        xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:sequence select="concat(upper-case(substring($arg,1,1)),substring($arg,2))"/>
    </xsl:function>

    <xsl:function name="functx:createID" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:sequence
            select="string-join((for $word in tokenize($arg,'[\s:./\-\(\)]+') return functx:capitalize-first($word)),'')"/>
    </xsl:function>

    <xsl:template match="/">
        <xsl:choose>
            <xsl:when test="$options='parameters'">
                <parameters-collection>
                    <xsl:comment>This file is generated from existing CPA's.</xsl:comment>
                    <xsl:apply-templates select="//tns:PartyInfo[1]" mode="parameters"/>
                    <xsl:apply-templates select="//tns:PartyInfo[2]" mode="parameters"/>
                </parameters-collection>
            </xsl:when>
            <xsl:otherwise><xsl:comment>You did not specify a valid option. Valid options are: all, service, parameters</xsl:comment></xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- 
        ************************************************
        Generate the parameters section
        ************************************************
    -->

    <xsl:template match="//tns:PartyInfo" mode="parameters">
        <msgpar:parameters xmlns:msgpar="http://www.sonnenglanz.net/2010/07/09/smf.parameters.2.0.xsd">
            <xsl:call-template name="createParameterList"/>
        </msgpar:parameters>
    </xsl:template>
    
    <xsl:template name="createParameterList">
        <msgpar:parameter name="CPAID">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:value-of select="parent::node()/@tns:cpaid"/>
        </msgpar:parameter>
        <msgpar:parameter name="PartyName">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:value-of select="@tns:partyName"/>
        </msgpar:parameter>
        <msgpar:parameter name="PartyRef">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:value-of select="tns:PartyRef/@xlink:href"/>
        </msgpar:parameter>
        <msgpar:parameter name="PartyIdList">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:apply-templates select="tns:PartyId"/>
        </msgpar:parameter>
        <msgpar:parameter name="PartyIdType">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:value-of select="tns:PartyId[1]/@tns:type"/>
        </msgpar:parameter>
        <msgpar:parameter name="ConditionalEndpointUri">
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'true'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="$options='parameters'">
                    <xsl:comment>Always check the values below! </xsl:comment>
                    <xsl:comment>The following EndpointUri was used by this partner (included here for verification purposes only):
                        <xsl:value-of select="tns:Transport/tns:TransportReceiver/tns:Endpoint/@tns:uri"/>
                    </xsl:comment>
                    <msgpar:condition name="local" switch="false">http://dit-staat-niet-in-de-cpa.nl/zelf-invullen</msgpar:condition>
                    <msgpar:condition name="default" switch="true">
                        <xsl:value-of select="tns:Transport/tns:TransportReceiver/tns:Endpoint/@tns:uri"/>
                    </msgpar:condition>
                </xsl:when>
            </xsl:choose>
            </msgpar:parameter>
        <xsl:apply-templates select="(tns:DocExchange/tns:ebXMLSenderBinding/tns:ReliableMessaging)[1]"/>
        <xsl:apply-templates select="tns:Certificate"/>
    </xsl:template>
    <!-- 
        ************************************************
        Helper templates
        ************************************************
    -->
    <xsl:template name="BTandBD">
        <xsl:param name="thisPartyActionBindingId"/>
        <xsl:variable name="subString"
            select="functx:getBusinessTransactionAndDocumentID($thisPartyActionBindingId)"/>
        <xsl:choose>
            <xsl:when test="$subString">
                <xsl:attribute name="id" select="$subString"/>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="determineProfile">
        <xsl:param name="channelId"/>
        <xsl:choose>
            <xsl:when test="contains($channelId,'ProfileBestEffort')">
                <xsl:attribute name="transport" select="'besteffort'"/>
            </xsl:when>
            <xsl:when test="contains($channelId,'ProfileReliableMessaging')">
                <xsl:attribute name="transport" select="'reliable'"/>
            </xsl:when>
            <xsl:when test="contains($channelId,'ProfileHoogReliableMessaging')">
                <xsl:attribute name="transport" select="'reliable'"/>
                <xsl:attribute name="security" select="'payload'"/>
            </xsl:when>
            <xsl:when test="contains($channelId,'ProfileHoogBestEffort')">
                <xsl:attribute name="transport" select="'besteffort'"/>
                <xsl:attribute name="security" select="'payload'"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:attribute name="transport" select="$channelId"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="tns:PartyId">
        <msgpar:value>
            <xsl:value-of select="descendant-or-self::text()"></xsl:value-of>
        </msgpar:value>
    </xsl:template>
    
    <xsl:template match="tns:Certificate">
        <xsl:variable name="certType" select="substring-after(@tns:certId,'_')"></xsl:variable>
        <msgpar:parameter>
            <xsl:attribute name="name" select="$certType"/>
            <xsl:choose>
                <xsl:when test="$options='all'">
                    <xsl:attribute name="required" select="'false'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:copy-of copy-namespaces="no" select="child::node()"/>
        </msgpar:parameter>
    </xsl:template>

    <xsl:template match="tns:DocExchange/tns:ebXMLSenderBinding/tns:ReliableMessaging">
        <xsl:variable name="retries" select="tns:Retries/descendant-or-self::text()"></xsl:variable>
        <xsl:variable name="retryInterval" select="tns:RetryInterval/descendant-or-self::text()"></xsl:variable>
        <xsl:variable name="persistduration" select="parent::node()/tns:PersistDuration/descendant-or-self::text()"></xsl:variable>
        <xsl:choose>
            <xsl:when test="$retries!='8'">
                <msgpar:parameter name="Retries">
                    <xsl:choose>
                        <xsl:when test="$options='all'">
                            <xsl:attribute name="required" select="'false'"/>
                        </xsl:when>
                    </xsl:choose>
                    <xsl:value-of select="$retries"/>
                </msgpar:parameter>
            </xsl:when>
        </xsl:choose>
        <xsl:choose>
            <xsl:when test="$retryInterval!='PT3H'">
                <msgpar:parameter name="RetryInterval">
                    <xsl:choose>
                        <xsl:when test="$options='all'">
                            <xsl:attribute name="required" select="'false'"/>
                        </xsl:when>
                    </xsl:choose>
                    <xsl:value-of select="$retryInterval"/>
                </msgpar:parameter>
            </xsl:when>
        </xsl:choose>
        <xsl:choose>
            <xsl:when test="$persistduration!='P1D'">
                <msgpar:parameter name="PersistDuration">
                    <xsl:choose>
                        <xsl:when test="$options='all'">
                            <xsl:attribute name="required" select="'false'"/>
                        </xsl:when>
                    </xsl:choose>
                    <xsl:value-of select="$persistduration"/>
                </msgpar:parameter>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>