<?xml version="1.0" encoding="UTF-8"?>
<!--
    
    ****************************************************************************************
    This XSLT generates the service specification files based on version 2.0
    for ebBP 2.0.
    
    July 2009

    European Union Public Licence - EUPL v.1.1
    
    Copyright (C) 2009  Sonnenglanz Consulting BV, www.Sonnenglanz.net
    
    Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
    European Commission - subsequent versions of the EUPL (the "Licence");
    You may not use this work except in compliance with the Licence.
    You may obtain a copy of the Licence at:
    
    http://ec.europa.eu/idabc/eupl
    
    Unless required by applicable law or agreed to in writing, software distributed
    under the Licence is distributed on an "AS IS" basis, 
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the Licence for the specific language governing permissions and limitations
    under the Licence.
    
    ****************************************************************************************
    
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:ebbp="http://docs.oasis-open.org/ebxml-bp/ebbp-2.0"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:functx="http://www.functx.com"
    xmlns="http://www.sonnenglanz.net/2010/07/09/smf.services.2.0.xsd"
    exclude-result-prefixes="xsl xsi ebbp functx xs">

    <xsl:output method="xml" indent="yes" encoding="utf-8"/>

    <xsl:param name="serviceIdBase">0</xsl:param>

    <xsl:include href="functions2.xsl"/>

    <!--
        ****************************************************************************************
        The transformation template: ebBP(to msg (version 4.0). 
        ****************************************************************************************
    -->
    <xsl:template match="ebbp:ProcessSpecification">
        <specification xmlns="http://www.sonnenglanz.net/2010/07/09/smf.services.2.0.xsd ">
            <services>
                <xsl:apply-templates select="ebbp:BusinessCollaboration"/>
            </services>

            <!--
                The rest will be filled in by a template. 
                First: get the default Rolebindings

            <xi:include xmlns:xi="http://www.w3.org/2001/XInclude"
                href="ebbp2smf-bindings.template.xml" xpointer="element(/1/1)" parse="xml"
                xsl:exclude-result-prefixes="#all">
                <xi:fallback>Could not include the rest (part 1) of the specification (filename:
                    ebbp.ebiz-tcf.template.xml).</xi:fallback>
            </xi:include>
            -->
            <!-- Rolebinding: default.  -->
            <rolebindings>
                <xsl:apply-templates select="ebbp:BusinessCollaboration/descendant::ebbp:Role"/>
            </rolebindings>
            
            <!--
                The rest will be filled in by a template. 
                Second: get the default Parameters

            <xi:include xmlns:xi="http://www.w3.org/2001/XInclude"
                href="ebbp2smf-bindings.template.xml" xpointer="element(/1/2)" parse="xml"
                xsl:exclude-result-prefixes="#all">
                <xi:fallback>Could not include the rest (part 2)of the specification (filename:
                    ebbp.ebiz-tcf.template.xml).</xi:fallback>
            </xi:include>
            -->
            <!-- Parameters: default.  -->
            <parameters>
                <group parameterId="p1">
                    <parameter name="ServiceType" required="false">http://www.moda-ml.net/ebiz-retail/repository/ebbp/v2008-1/en/</parameter>
                    <parameter name="PartyIdType" required="false">http:/www.iso.int/schemas/eanucc/gln</parameter>
                    <parameter name="PartyRef" required="false">http://spring.bologna.enea.it/ebiz</parameter>
                    <parameter name="PartyName" required="true"/>
                    <parameter name="PartyId" required="true"/>
                    <parameter name="EndpointUri" required="true"/>
                    <parameter name="ClientCert" required="true"/>
                    <parameter name="ServerCert" required="true"/>
                </group>
            </parameters>
            

        </specification>
    </xsl:template>

    <xsl:template match="ebbp:Role">
        <binding name="{concat('PartnerBindingName_', @name)}" parameterId="p1">
            <role serviceId="s1" name="{@name}"/>
        </binding>
    </xsl:template>

    <xsl:template match="ebbp:BusinessCollaboration">
        <xsl:variable name="counter" select="$serviceIdBase+position()"/>
        <service serviceId="s{$counter}" process="{@name}"
            processUuid="{parent::ebbp:ProcessSpecification/@uuid}"
            service="{parent::ebbp:ProcessSpecification/@uuid}" serviceType="ServiceTypePlaceholder">
            <xsl:variable name="P1" select="descendant::ebbp:Role[position()=1]"/>
            <xsl:variable name="P1id" select="$P1/@nameID"/>
            <xsl:variable name="P1name" select="$P1/@name"/>
            <xsl:variable name="P2" select="descendant::ebbp:Role[position()=2]"/>
            <xsl:variable name="P2id" select="$P2/@nameID"/>
            <xsl:variable name="P2name" select="$P2/@name"/>

            <xsl:apply-templates select="ebbp:BusinessTransactionActivity" mode="from">
                <xsl:with-param name="P1id" select="$P1id"/>
                <xsl:with-param name="P1name" select="$P1name"/>
                <xsl:with-param name="P2id" select="$P2id"/>
                <xsl:with-param name="P2name" select="$P2name"/>
            </xsl:apply-templates>

            <xsl:apply-templates select="ebbp:BusinessTransactionActivity" mode="to">
                <xsl:with-param name="P1id" select="$P1id"/>
                <xsl:with-param name="P1name" select="$P1name"/>
                <xsl:with-param name="P2id" select="$P2id"/>
                <xsl:with-param name="P2name" select="$P2name"/>
            </xsl:apply-templates>


        </service>

    </xsl:template>

    <xsl:template match="ebbp:BusinessTransactionActivity" mode="from">
        <xsl:param name="P1id"/>
        <xsl:param name="P1name"/>
        <xsl:param name="P2id"/>
        <xsl:param name="P2name"/>
        <xsl:variable name="BTP1id" select="ebbp:Performs[@currentRoleRef=$P1id]/@performsRoleRef"/>
        <xsl:variable name="myBusinessTransactionRef" select="@businessTransactionRef"/>
        <xsl:variable name="myNameID" select="@nameID"/>
        <xsl:variable name="firstNameID"
            select="//ebbp:BusinessTransactionActivity[@businessTransactionRef=$myBusinessTransactionRef][1]/@nameID"/>
        <xsl:if test="$myNameID=$firstNameID">
            <xsl:apply-templates select="//node()[ebbp:RequestingRole/@nameID=$BTP1id]">
                <xsl:with-param name="from" select="$P1name"/>
                <xsl:with-param name="to" select="$P2name"/>
            </xsl:apply-templates>
        </xsl:if>
    </xsl:template>

    <xsl:template match="ebbp:BusinessTransactionActivity" mode="to">
        <xsl:param name="P1id"/>
        <xsl:param name="P1name"/>
        <xsl:param name="P2id"/>
        <xsl:param name="P2name"/>
        <xsl:variable name="BTP1id" select="ebbp:Performs[@currentRoleRef=$P1id]/@performsRoleRef"/>
        <xsl:variable name="myBusinessTransactionRef" select="@businessTransactionRef"/>
        <xsl:variable name="myNameID" select="@nameID"/>
        <xsl:variable name="firstNameID"
            select="//ebbp:BusinessTransactionActivity[@businessTransactionRef=$myBusinessTransactionRef][1]/@nameID"/>
        <xsl:if test="$myNameID=$firstNameID">
            <xsl:apply-templates select="//node()[ebbp:RespondingRole/@nameID=$BTP1id]">
                <xsl:with-param name="from" select="$P2name"/>
                <xsl:with-param name="to" select="$P1name"/>
            </xsl:apply-templates>
        </xsl:if>
    </xsl:template>

    <xsl:template match="ebbp:CommercialTransaction | ebbp:Notification">
        <xsl:param name="from"/>
        <xsl:param name="to"/>
        <xsl:variable name="BT" select="@nameID"/>
        <xsl:variable name="REQ-RBA" select="ebbp:RequestingBusinessActivity/@nameID"/>
        <xsl:variable name="REQ-BD"
            select="ebbp:RequestingBusinessActivity/ebbp:DocumentEnvelope/@ businessDocumentRef"/>
        <xsl:variable name="RES-RBA" select="ebbp:RespondingBusinessActivity/@nameID"/>
        <xsl:variable name="RES-BD"
            select="ebbp:RespondingBusinessActivity/ebbp:DocumentEnvelope/@ businessDocumentRef"/>
        <message 
            from="{$from}" 
            to="{$to}" 
            id="{functx:createID(concat($REQ-RBA,'__',$REQ-BD))}"
            bt="{$BT}"
            bd="{$REQ-BD}"
            transport="reliable" 
            security="transport">
            <xsl:value-of select="functx:words-to-camel-case(ebbp:RequestingBusinessActivity/@name)"
            />
        </message>
        <xsl:if test="ebbp:RespondingBusinessActivity/ebbp:DocumentEnvelope">
            <message 
                from="{$to}"
                to="{$from}" 
                id="{functx:createID(concat($RES-RBA,'__',$RES-BD))}"
                bt="{$BT}"
                bd="{$RES-BD}"
                transport="reliable" 
                security="transport">
                <xsl:value-of
                    select="functx:words-to-camel-case(ebbp:RespondingBusinessActivity/@name)"/>
            </message>
        </xsl:if>
    </xsl:template>
</xsl:stylesheet>
