<?xml version="1.0" encoding="UTF-8"?>
<!--
    
    ****************************************************************************************
    This XSLT  will remove certificate information from a CPA  depending on de ssl-offloading configuration.
    May 2009
    
    Parameter: partyName
    Specify the exact @partyName value that is used in the CPA to specify for which partner you want to change the CPA.
    
    Parameter: options
    Specify the direction of the ssl offloading.
    Allowed values:
    NONE (default)
    IN
    OUT
    IN-AND-OUT
    
    European Union Public Licence - EUPL v.1.1
    
    Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl
    
    Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
    European Commission - subsequent versions of the EUPL (the "Licence");
    You may not use this work except in compliance with the Licence.
    You may obtain a copy of the Licence at:
    
    http://ec.europa.eu/idabc/eupl
    
    Unless required by applicable law or agreed to in writing, software distributed
    under the Licence is distributed on an "AS IS" basis, 
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the Licence for the specific language governing permissions and limitations
    under the Licence.
    
    ****************************************************************************************
    
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:functx="http://www.functx.com"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:tns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
    xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:xlink="http://www.w3.org/1999/xlink"
    xsi:schemaLocation="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd                  http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
    version="2.0">
    
    <xsl:output method="xml" indent="yes" encoding="utf-8"/>

    <xsl:function name="functx:replace-string" as="xs:string?" xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:sequence select="concat(upper-case(substring($arg,1,1)),substring($arg,2))"/>
    </xsl:function>

    <xsl:param name="partner">00000003271987420000</xsl:param>
    <xsl:param name="options">infra-inout</xsl:param>
    <xsl:param name="otherPartnerEndpoint"></xsl:param>

    <xsl:variable name="partyName" >
        <xsl:for-each select="//tns:PartyInfo/tns:PartyId">
            <xsl:choose>
                <xsl:when test="descendant-or-self::text()=$partner">
                    <!-- There can be only one, so that one will be used to return the partyname. If there is no match, it will be 'empty'. -->
                    <xsl:value-of select="parent::node()/@tns:partyName"/>
                </xsl:when>
            </xsl:choose>            
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="role">
        <xsl:for-each select="//tns:PartyInfo/tns:PartyId">
            <xsl:choose>
                <xsl:when test="descendant-or-self::text()=$partner">
                    <!-- There can be only one, so that one will be used to return the partyname. If there is no match, it will be 'empty'. -->
                    <xsl:value-of select="following-sibling::node()//tns:Role/@tns:name"/>
                </xsl:when>
            </xsl:choose>            
        </xsl:for-each>
    </xsl:variable>

    <xsl:template match="//tns:PartyInfo[@tns:partyName!=$partyName]//tns:Endpoint">
        <xsl:choose>
            <xsl:when test="$otherPartnerEndpoint != ''">
                <xsl:comment xml:lang="en-EN">Changed the Endpoint of the other partner.</xsl:comment>
                <tns:Endpoint tns:uri="{$otherPartnerEndpoint}" tns:type="{@tns:type}"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="//tns:PartyInfo[@tns:partyName=$partyName]/tns:Certificate">
        <xsl:choose>
            <xsl:when test="$options='infra-in'">
                <xsl:choose>
                    <xsl:when test="@tns:certId=concat($role,'_ServerCert')">
                        <xsl:comment xml:lang="en-EN">Server Certificate removed: <xsl:value-of select="@tns:certId"/></xsl:comment>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy>
                            <xsl:apply-templates select="@*"/>
                            <xsl:apply-templates/>
                        </xsl:copy>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$options='infra-out'">
                <xsl:choose>
                    <xsl:when test="@tns:certId=concat($role,'_ClientCert')">
                        <xsl:comment xml:lang="en-EN">Client Certificate removed: <xsl:value-of select="@tns:certId"/></xsl:comment>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy>
                            <xsl:apply-templates select="@*"/>
                            <xsl:apply-templates/>
                        </xsl:copy>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$options='infra-inout'">
                <xsl:comment xml:lang="en-EN">Client and Server Certificate removed: <xsl:value-of select="@tns:certId"/></xsl:comment>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="//tns:PartyInfo[@tns:partyName=$partyName]/tns:SecurityDetails/tns:TrustAnchors/tns:AnchorCertificateRef">
        <xsl:choose>
            <xsl:when test="$options='infra-in'">
                <xsl:choose>
                    <xsl:when test="@tns:certId=concat($role,'_ServerCert')">
                        <xsl:comment xml:lang="en-EN">Server Anchor Certificate removed: <xsl:value-of select="@tns:certId"/></xsl:comment>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy>
                            <xsl:apply-templates select="@*"/>
                            <xsl:apply-templates/>
                        </xsl:copy>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$options='infra-out'">
                <xsl:choose>
                    <xsl:when test="@tns:certId=concat($role,'_ClientCert')">
                        <xsl:comment xml:lang="en-EN">Client Anchor Certificate removed: <xsl:value-of select="@tns:certId"/></xsl:comment>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy>
                            <xsl:apply-templates select="@*"/>
                            <xsl:apply-templates/>
                        </xsl:copy>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="//tns:PartyInfo[@tns:partyName=$partyName]/tns:SecurityDetails">
        <xsl:choose>
            <xsl:when test="$options='infra-inout'">
                <xsl:comment xml:lang="en-EN"> SecurityDetails removed: <xsl:value-of
                    select="@tns:securityId"/>
                </xsl:comment>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="//tns:PartyInfo[@tns:partyName=$partyName]//tns:TransportClientSecurity">
        <xsl:choose>
            <xsl:when test="$options='infra-inout' or $options='infra-out'">
                <xsl:comment xml:lang="en-EN"> TransportClientSecurity removed. </xsl:comment>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="//tns:PartyInfo[@tns:partyName=$partyName]//tns:TransportServerSecurity">
        <xsl:choose>
            <xsl:when test="$options='infra-inout' or $options='infra-in'">
                <xsl:comment xml:lang="en-EN"> TransportServerSecurity removed. </xsl:comment>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="//tns:PartyInfo[@tns:partyName=$partyName]//tns:Endpoint">
        <xsl:variable name="isHTTPS" select="substring-after(@tns:uri,'https://')"/>
        <xsl:variable name="uri" select="substring-after(@tns:uri,'https://')"/>
        <xsl:choose>
            <xsl:when test="($options='infra-inout' or $options='infra-in') and ($isHTTPS)">
                <xsl:comment xml:lang="en-EN"> Endpoint changed from https:// into http:// </xsl:comment>
                <tns:Endpoint tns:uri="{concat('http://',$uri)}" tns:type="{@tns:type}"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="@*|node()">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>
