<?xml version="1.0" encoding="utf-8"?>
<!--
   
   ****************************************************************************************
   This XSLT  will create CPA templates
   January, 19, 2010
   January 29, 2010: Added value of @processversion (SMF) to action @id (CPA)
   January 29, 2010: Corrections made regarding the checks for adding certificates details.
   
   European Union Public Licence - EUPL v.1.1
   
   Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl
   
   Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
   European Commission - subsequent versions of the EUPL (the "Licence");
   You may not use this work except in compliance with the Licence.
   You may obtain a copy of the Licence at:
   
   http://ec.europa.eu/idabc/eupl
   
   Unless required by applicable law or agreed to in writing, software distributed
   under the Licence is distributed on an "AS IS" basis, 
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the Licence for the specific language governing permissions and limitations
   under the Licence.
   
   ****************************************************************************************

-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
   xmlns:functx="http://www.functx.com"
   xmlns:xs="http://www.w3.org/2001/XMLSchema"
   xmlns:msg="http://www.sonnenglanz.net/2010/07/09/smf.services.2.0.xsd"
   xmlns:tns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
   xmlns:ds="http://www.w3.org/2000/09/xmldsig#" 
   xmlns:xlink="http://www.w3.org/1999/xlink"
   xsi:schemaLocation="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd                  http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
   version="2.0" xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias"
   exclude-result-prefixes="xsl msg functx">

   <xsl:output indent="yes" method="xml" encoding="utf-8"/>
   <xsl:strip-space elements="*"/>

   <xsl:param name="processSpecificationName" select="//msg:service[1]/@process"/>
   <xsl:param name="serviceName" select="//msg:service[1]/@service"/>
   <xsl:param name="serviceType"/>
   <xsl:param name="fromRole"/>
   <xsl:param name="toRole"/>
   <xsl:param name="startDate">2000-01-01T00:00:00Z</xsl:param>
   <xsl:param name="endDate">3000-01-01T00:00:00Z</xsl:param>
   <xsl:param name="partyIdType">PartyIdType_PH</xsl:param>
   <xsl:param name="defaultProcessHREFbaseUrl">HTTP://BASE-HREF-PH/</xsl:param>
   <xsl:param name="defaultProcessUUIDbaseUrn">URN:BASE:UUID:PH:</xsl:param>

   <xsl:include href="functions.xsl"/>

   <!--
      ****************************************************************************************
      Default roles in case there is no param input. 
      ****************************************************************************************
   -->
   <xsl:variable name="defaultFromRole" select="//msg:service[1]/msg:message[1]/@from"/>
   <xsl:variable name="defaultToRole" select="//msg:service[1]/msg:message[1]/@to"/>
   <xsl:variable name="defaultFromRoleBinding" select="//msg:rolebindings/msg:binding[1]/@name"/>
   <xsl:variable name="defaultToRoleBinding" select="//msg:rolebindings/msg:binding[2]/@name"/>

   <!--
      ****************************************************************************************
      What are the default characteristics for the transport and security?
      ****************************************************************************************
   -->
   <xsl:variable name="defaultMessage">
      <xsl:element name="default">
         <xsl:attribute name="transport">
            <xsl:choose>
               <xsl:when test="//msg:services/@defaultTransport"><xsl:value-of select="//msg:services/@defaultTransport"/></xsl:when>
               <xsl:otherwise>besteffort</xsl:otherwise><!-- The default document exchange characteristic is best effort (no acknowledgements). -->
            </xsl:choose>
         </xsl:attribute>
         <xsl:attribute name="security">
            <xsl:choose>
               <xsl:when test="//msg:services/@defaultSecurity"><xsl:value-of select="//msg:services/@defaultSecurity"/></xsl:when>
               <xsl:otherwise/><!-- The default security is none (no transport or payload security). -->
            </xsl:choose>
         </xsl:attribute>
         <xsl:attribute name="retryCount">
            <xsl:choose>
               <xsl:when test="//msg:services/@defaultRetryCount"><xsl:value-of select="//msg:services/@defaultRetryCount"/></xsl:when>
               <xsl:otherwise>8</xsl:otherwise><!-- The default Retry Count is 8 times (after the initial transmission). -->
            </xsl:choose>
         </xsl:attribute>
         <xsl:attribute name="retryInterval">
            <xsl:choose>
               <xsl:when test="//msg:services/@defaultRetryInterval"><xsl:value-of select="//msg:services/@defaultRetryInterval"/></xsl:when>
               <xsl:otherwise>PT3H</xsl:otherwise><!-- The default Retry Interval is 3 hours. -->
            </xsl:choose>
         </xsl:attribute>
      </xsl:element>
   </xsl:variable>
   <!-- Shortcuts -->
   <xsl:variable name="defaultTransport" select="$defaultMessage/default/@transport"/>
   <xsl:variable name="defaultSecurity" select="$defaultMessage/default/@security"/>
   <xsl:variable name="defaultRetryInterval" select="$defaultMessage/default/@retryInterval"/>
   <xsl:variable name="defaultRetryCount" select="$defaultMessage/default/@retryCount"/>

   <!--
      ****************************************************************************************
      What are the characteristics of the default delivery channels?
      The characteristics are derived from the default attribute values, 
      as part of the services element.
      ****************************************************************************************
   -->
   <xsl:variable name="defaultDeliveryChannel"
      select="functx:createChannelID('default ', $defaultMessage/default)"/>
   <xsl:variable name="defaultDocExchange"
      select="functx:createDocExchangeID('default ', $defaultMessage/default)"/>

   <!--
      ****************************************************************************************
      Select the partnerRoleBinding based on the availability of the rolebindings element.
      If there is no rolebindingselement, just two roles are assumed: the first FROM and the
      first TO is selected (if there are more roles, each should be provided as a parameter).
      ****************************************************************************************
   -->
   <xsl:variable name="partnerRoleBinding">
      <xsl:choose>
         <xsl:when test="$fromRole!=''"><xsl:value-of select="$fromRole"/></xsl:when>
         <xsl:otherwise>
            <xsl:choose>
               <xsl:when test="$defaultFromRoleBinding!=''"><xsl:value-of select="$defaultFromRoleBinding"/></xsl:when>
               <xsl:otherwise><xsl:value-of select="$defaultFromRole"/></xsl:otherwise>
            </xsl:choose>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <xsl:variable name="otherPartnerRoleBinding">
      <xsl:choose>
         <xsl:when test="$toRole!=''"><xsl:value-of select="$toRole"/></xsl:when>
         <xsl:otherwise>
            <xsl:choose>
               <xsl:when test="$defaultToRoleBinding!=''"><xsl:value-of select="$defaultToRoleBinding"/></xsl:when>
               <xsl:otherwise><xsl:value-of select="$defaultToRole"/></xsl:otherwise>
            </xsl:choose>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>

   <!-- 
      ****************************************************************************************
      A CPA contains only those messages that are part of the 'current' two given partners.
      Therefore, create an "active" set of messages and profiles, filtered by:
      service id
      partnerRoleBinding 
      otherPartnerRoleBinding.
      ****************************************************************************************
   -->
   <xsl:variable name="selectedServiceIds" select="//msg:rolebindings/msg:binding[@name=$partnerRoleBinding]/msg:role[@serviceId=(//msg:rolebindings/msg:binding[@name=$otherPartnerRoleBinding]/msg:role/@serviceId)]/@serviceId"/>
   <xsl:variable name="partnerRole" select="//msg:rolebindings/msg:binding[@name=$partnerRoleBinding]/msg:role[@serviceId=$selectedServiceIds]/@name"/>
   <xsl:variable name="otherPartnerRole" select="//msg:rolebindings/msg:binding[@name=$otherPartnerRoleBinding]/msg:role[@serviceId=$selectedServiceIds]/@name"/>
   <xsl:variable name="allMyMessages" select="//msg:message[(parent::node()/@serviceId=$selectedServiceIds) and ((@from=$partnerRole and @to=$otherPartnerRole) or (@to=$partnerRole and @from=$otherPartnerRole))]"/>
   <xsl:variable name="allMyProfiles" select="//msg:profile[@name=$allMyMessages/@profile]"/>

   <!--
      ****************************************************************************************
      Select the unique Delivery Channel (DC) id's.
      Based on a filtered set of messages AND the default
      values (they can overrule the message values).
      ****************************************************************************************
   -->
   <xsl:key name="channelDs" match="//msg:message[(parent::node()/@serviceId=$selectedServiceIds) and ((@from=$partnerRole and @to=$otherPartnerRole) or (@to=$partnerRole and @from=$otherPartnerRole))]"
      use="functx:createChannelID(parent::node()/@serviceId, functx:inheritDefault(. , $defaultMessage/default, //msg:profiles))"/>
   <xsl:variable name="uniqueChannelDs"
      select="//msg:message[generate-id(.) = generate-id(key('channelDs',functx:createChannelID(parent::node()/@serviceId, functx:inheritDefault(. , $defaultMessage/default, //msg:profiles)))[1])]"/>
   
   <!--
      ****************************************************************************************
      Select the unique Document Exchange (DE) id's.
      Based on a filtered set of messages AND the default
      values (they can overrule the message values).
      ****************************************************************************************
   -->
   <xsl:key name="docExchangeIDs" match="//msg:message[(parent::node()/@serviceId=$selectedServiceIds) and ((@from=$partnerRole and @to=$otherPartnerRole) or (@to=$partnerRole and @from=$otherPartnerRole))]"
      use="functx:createDocExchangeID(parent::node()/@serviceId, functx:inheritDefault(. , $defaultMessage/default, //msg:profiles))"/>
   <xsl:variable name="uniqueDocExchangeIDs"
      select="//msg:message[generate-id(.) = generate-id(key('docExchangeIDs',functx:createDocExchangeID(parent::node()/@serviceId, functx:inheritDefault(. , $defaultMessage/default, //msg:profiles)))[1])]"/>

   <!--
      ****************************************************************************************
      Start template creation based on the input specfication. 
      ****************************************************************************************
   -->
   <xsl:template match="/">
      <!--
      <xsl:template name="createCpaTemplate">
      <xsl:param name="partnerRoleBinding"/>
      <xsl:param name="otherPartnerRoleBinding"/>
   -->
      <tns:CollaborationProtocolAgreement
         xsi:schemaLocation="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd                  http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
         tns:cpaid="{concat($partnerRoleBinding, '-', $otherPartnerRoleBinding)}" tns:version="1.0">

         <tns:Status tns:value="agreed"/>
         <tns:Start>
            <xsl:value-of select="$startDate"/>
         </tns:Start>
         <tns:End>
            <xsl:value-of select="$endDate"/>
         </tns:End>
         <tns:ConversationConstraints tns:concurrentConversations="0" tns:invocationLimit="0"/>

         <xsl:call-template name="PartyInfo">
            <xsl:with-param name="partner" select="$partnerRoleBinding"/>
            <xsl:with-param name="otherPartner" select="$otherPartnerRoleBinding"/>
         </xsl:call-template>

         <xsl:call-template name="PartyInfo">
            <xsl:with-param name="partner" select="$otherPartnerRoleBinding"/>
            <xsl:with-param name="otherPartner" select="$partnerRoleBinding"/>
         </xsl:call-template>

         <xsl:call-template name="SimpePart"/>

         <xsl:call-template name="Packaging"/>

         <xsl:call-template name="Comments"/>

      </tns:CollaborationProtocolAgreement>
   </xsl:template>

   <!--
      ****************************************************************************************
      Create PartyInfo element. 
      ****************************************************************************************
   -->
   <xsl:template name="PartyInfo">
      <xsl:param name="partner"/>
      <xsl:param name="otherPartner"/>
      <tns:PartyInfo tns:partyName="{concat($partner, '_PartyName_PH')}"
         tns:defaultMshChannelId="{concat(functx:createID($partner), $defaultDeliveryChannel)}"
         tns:defaultMshPackageId="defaultPackage_Profile">
         <tns:PartyId tns:type="{$partyIdType}">
            <xsl:value-of select="concat($partner, '_PartyId_PH')"/>
         </tns:PartyId>
         <tns:PartyRef xlink:href="{concat($partner, '_PartyRefUrl_PH')}" xlink:type="simple"/>

         <xsl:apply-templates select="//msg:rolebindings/msg:binding[@name=$partner]"
            mode="collaborationroles">
            <xsl:with-param name="partner" select="$partner"/>
            <xsl:with-param name="otherPartner" select="$otherPartner"/>
         </xsl:apply-templates>

         <xsl:call-template name="CertificatesAndSecurityDetails">
            <xsl:with-param name="partner" select="$partner"/>
         </xsl:call-template>

         <xsl:call-template name="DeliveryChannels">
            <xsl:with-param name="partner" select="$partner"/>
         </xsl:call-template>

         <xsl:call-template name="Transport">
            <xsl:with-param name="partner" select="$partner"/>
         </xsl:call-template>

         <xsl:call-template name="DocExchanges">
            <xsl:with-param name="partner" select="$partner"/>
         </xsl:call-template>

      </tns:PartyInfo>
   </xsl:template>

   <!--
      ****************************************************************************************
      Helper (intermediate) templates to find the correct roles.
      Note that the partner name can be different from the roles: the actual roles
      are to be found in the rolebindings section for that particular partner.
      ****************************************************************************************
   -->
   <xsl:template match="msg:rolebindings/msg:binding/msg:role" mode="collaborationroles">
      <xsl:param name="partner"/>
      <xsl:param name="otherPartner"/>
      <xsl:variable name="serviceId" select="@serviceId"/>
      <xsl:call-template name="CollaborationRole">
         <xsl:with-param name="partner" select="$partner"/>
         <xsl:with-param name="otherPartner" select="$otherPartner"/>
         <xsl:with-param name="serviceId" select="@serviceId"/>
         <xsl:with-param name="role" select="@name"/>
         <xsl:with-param name="otherRole" select="//msg:rolebindings/msg:binding[@name=$otherPartner]/msg:role[@serviceId=$serviceId]/@name"/>
      </xsl:call-template>
   </xsl:template>

   <!--
      ****************************************************************************************
      Create CollaborationRole element. 
      ****************************************************************************************
   -->
   <xsl:template name="CollaborationRole">
      <xsl:param name="partner"/>
      <xsl:param name="otherPartner"/>
      <xsl:param name="serviceId"/>
      <xsl:param name="role"/>
      <xsl:param name="otherRole"/>
      <xsl:if test="(//msg:service[@serviceId=$serviceId]/msg:message[@from=$role][@to=$otherRole]) or (//msg:service[@serviceId=$serviceId]/msg:message[@to=$role][@from=$otherRole])">
         <xsl:variable name="roleServiceName" select="//msg:service[@serviceId=$serviceId]/@service"/>
         <xsl:variable name="roleServiceType" select="//msg:service[@serviceId=$serviceId]/@serviceType"/>
         <xsl:variable name="roleProcessName" select="//msg:service[@serviceId=$serviceId]/@process"/>
         <xsl:variable name="processUUID" select="//msg:service[@serviceId=$serviceId]/@processUuid"/>
         <xsl:variable name="processHREF" select="//msg:service[@serviceId=$serviceId]/@processHref"/>
         <xsl:variable name="processVERSION">
            <xsl:choose>
               <xsl:when test="//msg:service[@serviceId=$serviceId]/@processversion"><xsl:value-of select="//msg:service[@serviceId=$serviceId]/@processversion"/></xsl:when>
               <xsl:otherwise><xsl:value-of>1.0</xsl:value-of></xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         <xsl:variable name="xlink_href">
            <xsl:choose>
               <xsl:when test="$processHREF!=''"><xsl:value-of select="$processHREF"/></xsl:when>
               <xsl:otherwise><xsl:value-of select="concat($defaultProcessHREFbaseUrl,functx:createID($processSpecificationName))"/></xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         <xsl:variable name="tns_uuid">
            <xsl:choose>
               <xsl:when test="$processUUID"><xsl:value-of select="$processUUID"/></xsl:when>
               <xsl:otherwise><xsl:value-of select="concat($defaultProcessUUIDbaseUrn,functx:createID($processSpecificationName),':',functx:createID($roleServiceName))"/></xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         <xsl:variable name="tns_name">
            <xsl:choose>
               <xsl:when test="$roleProcessName!=''"><xsl:value-of select="$roleProcessName"/></xsl:when>
               <xsl:otherwise><xsl:value-of select="$processSpecificationName"/></xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         <tns:CollaborationRole>
            <tns:ProcessSpecification tns:name="{$tns_name}" tns:version="{$processVERSION}"
               xlink:type="simple" xlink:href="{$xlink_href}" tns:uuid="{$tns_uuid}"/>
            <tns:Role tns:name="{$role}" xlink:href="{$xlink_href}"/>
            <tns:ServiceBinding>
               <xsl:choose>
                  <xsl:when test="$serviceType!='' or $roleServiceType!=''">
                     <xsl:choose>
                        <xsl:when test="$roleServiceType!=''"><tns:Service tns:type="{$roleServiceType}"><xsl:value-of select="$roleServiceName"/></tns:Service></xsl:when>
                        <xsl:otherwise>
                           <tns:Service tns:type="{$serviceType}">
                              <xsl:choose>
                                 <xsl:when test="$roleServiceName!=''"><xsl:value-of select="$roleServiceName"/></xsl:when>
                                 <xsl:otherwise><xsl:value-of select="$serviceName"/></xsl:otherwise>
                              </xsl:choose>
                           </tns:Service>
                        </xsl:otherwise>
                     </xsl:choose>
                  </xsl:when>
                  <xsl:otherwise>
                     <tns:Service>
                        <xsl:choose>
                           <xsl:when test="$roleServiceName!=''"><xsl:value-of select="$roleServiceName"/></xsl:when>
                           <xsl:otherwise><xsl:value-of select="$serviceName"/></xsl:otherwise>
                        </xsl:choose>
                     </tns:Service>
                  </xsl:otherwise>
               </xsl:choose>
               <xsl:apply-templates select="//msg:service[@serviceId=$serviceId]/msg:message[@from=$role][@to=$otherRole]" mode="send">
                  <xsl:with-param name="partner" select="$partner"/>
                  <xsl:with-param name="otherPartner" select="$otherPartner"/>
                  <xsl:with-param name="serviceId" select="$serviceId"/>
                  <xsl:with-param name="version" select="$processVERSION"/>
               </xsl:apply-templates>
               <xsl:apply-templates select="//msg:service[@serviceId=$serviceId]/msg:message[@to=$role][@from=$otherRole]" mode="receive">
                  <xsl:with-param name="partner" select="$partner"/>
                  <xsl:with-param name="otherPartner" select="$otherPartner"/>
                  <xsl:with-param name="serviceId" select="$serviceId"/>
                  <xsl:with-param name="version" select="$processVERSION"/>
               </xsl:apply-templates>
            </tns:ServiceBinding>
         </tns:CollaborationRole>
      </xsl:if>
   </xsl:template>

   <!-- 
      ************************************************************************************
      Create CanSend elements based on the profile
      ************************************************************************************
   -->
   <xsl:template match="msg:service/msg:message" mode="send">
      <xsl:param name="partner"/>
      <xsl:param name="otherPartner"/>
      <xsl:param name="serviceId"/>
      <xsl:param name="version"/>
      <xsl:variable name="inherited" select="functx:inheritDefault(. , $defaultMessage/default, //msg:profiles)"/>
      <xsl:variable name="pair" select="functx:createID(concat($serviceId,'_',@from,'_',@to))"/>
      <xsl:variable name="actionId">
         <xsl:choose>
            <xsl:when test="@id!=''"><xsl:value-of select="functx:createID(@id)"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="functx:createID(descendant-or-self::text())"/></xsl:otherwise>
         </xsl:choose>
      </xsl:variable>
      <tns:CanSend>
         <tns:ThisPartyActionBinding
            tns:id="{concat(functx:createID($partner),'_',$pair, '_S_', $actionId, '_PV_', translate($version, '.', '_'))}"
            tns:action="{descendant-or-self::text()}" tns:packageId="defaultPackage_Profile">
            <xsl:call-template name="declareBusinessTransactionCharacteristics">
               <xsl:with-param name="message" select="$inherited"/>
            </xsl:call-template>
            <tns:ChannelId><xsl:value-of select="concat(functx:createID($partner), functx:createChannelID(parent::node()/@serviceId, $inherited))"/></tns:ChannelId>
         </tns:ThisPartyActionBinding>
         <tns:OtherPartyActionBinding><xsl:value-of select="concat(functx:createID($otherPartner),'_',$pair, '_R_',  $actionId, '_PV_', translate($version, '.', '_'))"/></tns:OtherPartyActionBinding>
      </tns:CanSend>
   </xsl:template>

   <!-- 
      ************************************************************************************
      Create CanReceive elements based on the profile
      ************************************************************************************
   -->
   <xsl:template match="msg:service/msg:message" mode="receive">
      <xsl:param name="partner"/>
      <xsl:param name="otherPartner"/>
      <xsl:param name="serviceId"/>
      <xsl:param name="version"/>
      <xsl:variable name="inherited" select="functx:inheritDefault(. , $defaultMessage/default, //msg:profiles)"/>
      <xsl:variable name="pair" select="functx:createID(concat($serviceId,'_',@from,'_',@to))"/>
      <xsl:variable name="actionId">
         <xsl:choose>
            <xsl:when test="@id!=''"><xsl:value-of select="functx:createID(@id)"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="functx:createID(descendant-or-self::text())"/></xsl:otherwise>
         </xsl:choose>
      </xsl:variable>
      <tns:CanReceive>
         <tns:ThisPartyActionBinding
            tns:id="{concat(functx:createID($partner),'_',$pair, '_R_', $actionId, '_PV_', translate($version, '.', '_'))}"
            tns:action="{descendant-or-self::text()}" tns:packageId="defaultPackage_Profile">
            <xsl:call-template name="declareBusinessTransactionCharacteristics">
               <xsl:with-param name="message" select="$inherited"/>
            </xsl:call-template>
            <tns:ChannelId><xsl:value-of select="concat(functx:createID($partner), functx:createChannelID(parent::node()/@serviceId, $inherited))"/></tns:ChannelId>
         </tns:ThisPartyActionBinding>
         <tns:OtherPartyActionBinding><xsl:value-of select="concat(functx:createID($otherPartner),'_',$pair, '_S_',  $actionId, '_PV_', translate($version, '.', '_'))"/></tns:OtherPartyActionBinding>
      </tns:CanReceive>
   </xsl:template>

   <!-- 
      ****************************************************************************************
     Creation of the BusinessTransactionCharacteristics elements, belonging
     to the CanSend and CanReceive.
      ****************************************************************************************
   -->
   <xsl:template name="declareBusinessTransactionCharacteristics">
      <xsl:param name="message"/>
      <tns:BusinessTransactionCharacteristics 
         tns:isAuthenticated="{functx:isAuthenticated($message/@security)}"
         tns:isAuthorizationRequired="true" 
         tns:isIntelligibleCheckRequired="false"
         tns:isNonRepudiationReceiptRequired="{functx:isNonRepudiationRequired($message/@security)}"
         tns:isNonRepudiationRequired="{functx:isNonRepudiationRequired($message/@security)}"
         tns:isConfidential="{functx:isConfidential($message/@security)}"
         tns:isTamperProof="{functx:isAuthenticated($message/@security)}">
         <xsl:choose>
            <xsl:when test="$message/@transport = 'reliable'"><xsl:attribute name="tns:timeToPerform" select="functx:deriveSafeTimeToPerform($message/@retryCount, $message/@retryInterval)"/></xsl:when>
            <xsl:otherwise><xsl:attribute name="tns:timeToPerform">PT10S</xsl:attribute></xsl:otherwise>
         </xsl:choose>
      </tns:BusinessTransactionCharacteristics>
   </xsl:template>

   <!-- 
      ****************************************************************************************
      Creation of the Certificate elements and the SecurityDetails elements.
      Create only the security details that are related to MY CPA, which it based on the "current" partner in the rolebindings.
      The existence of an attribute in a message takes precedence over (the same attribute in) a profile attribute.
   ****************************************************************************************
   -->
   <xsl:template name="CertificatesAndSecurityDetails">
      <xsl:param name="partner"/>
      <xsl:variable name="partnerID" select="functx:createID($partner)"/>
      <xsl:variable name="signing" select="$allMyProfiles[contains(@security, 'sign') or contains(@security, 'payload')]"/>
      <xsl:variable name="encryption" select="$allMyProfiles[contains(@security, 'payload')]"/>
      <xsl:variable name="transport" select="$allMyProfiles[contains(@security, 'transport')]"/>
      <!-- 
         ****************************************************************************************
         Do we have to incorporate a signing certificate? 
         ****************************************************************************************
      -->
      <xsl:variable name="declareSignCert" select="($allMyMessages[(contains(@security, 'sign') or contains(@security, 'payload')) or (not (@security) and (@profile=$signing/@name))])
         or
         contains($defaultMessage/default/@security, 'sign') or contains($defaultMessage/default/@security, 'payload')"/>
      <!-- 
         ****************************************************************************************
         Do we have to incorporate an encryption certificate? 
         ****************************************************************************************
      -->
      <xsl:variable name="declareEncrCert" select="($allMyMessages[(contains(@security, 'payload')) or (not (@security) and (@profile=$encryption/@name))])
         or
         contains($defaultMessage/default/@security, 'payload')"/>
      <!-- 
         ****************************************************************************************
         Do we have to incorporate the client/server certificates? 
         (We always use the server AND client certificates for HTTPS.)
         ****************************************************************************************
      -->
      <xsl:variable name="declareClientServerCerts" select="($allMyMessages[(contains(@security, 'transport')) or (not (@security) and (@profile=$transport/@name))])
         or
         contains($defaultMessage/default/@security, 'transport')"/>
      <!-- 
         ****************************************************************************************
         Do we have to incorporate a trusted root certificate? 
         ****************************************************************************************
      -->
      <xsl:variable name="parameterId" select="//msg:binding[@name=$partner]/@parameterId"/>
      <xsl:variable name="declareTrustedRoot" select="//msg:parameters/msg:group[@parameterId=$parameterId]/msg:parameter[@name='RootCert']"/>
      <!--
         *****************************************************
         Creation the Certificate elements (if needed).
         *****************************************************
      -->
      <xsl:if test="$declareSignCert">
         <!-- PLACEHOLDER for Signing Certificate -->
         <tns:Certificate tns:certId="{concat($partnerID, '_SigningCert')}">
            <KeyInfo xmlns="http://www.w3.org/2000/09/xmldsig#">
               <xsl:value-of select="concat($partner, '_SigningCertKeyInfo_PH')"/>
            </KeyInfo>
         </tns:Certificate>
      </xsl:if>
      <xsl:if test="$declareEncrCert">
         <!-- PLACEHOLDER for Encryption Certificate -->
         <tns:Certificate tns:certId="{concat($partnerID, '_EncryptionCert')}">
            <KeyInfo xmlns="http://www.w3.org/2000/09/xmldsig#">
               <xsl:value-of select="concat($partner, '_EncryptionCertKeyInfo_PH')"/>
            </KeyInfo>
         </tns:Certificate>
      </xsl:if>
      <xsl:if test="$declareClientServerCerts">
         <!-- PLACEHOLDER for Server Certificate -->
         <tns:Certificate tns:certId="{concat($partnerID, '_ServerCert')}">
            <KeyInfo xmlns="http://www.w3.org/2000/09/xmldsig#">
               <xsl:value-of select="concat($partner, '_ServerCertKeyInfo_PH')"/>
            </KeyInfo>
         </tns:Certificate>
         <!-- PLACEHOLDER for Client Certificate -->
         <tns:Certificate tns:certId="{concat($partnerID, '_ClientCert')}">
            <KeyInfo xmlns="http://www.w3.org/2000/09/xmldsig#">
               <xsl:value-of select="concat($partner, '_ClientCertKeyInfo_PH')"/>
            </KeyInfo>
         </tns:Certificate>
      </xsl:if>
      <xsl:if test="$declareTrustedRoot">
         <!-- PLACEHOLDER for Root Certificate -->
         <tns:Certificate tns:certId="{concat($partnerID, '_RootCert')}">
            <KeyInfo xmlns="http://www.w3.org/2000/09/xmldsig#">
               <xsl:value-of select="concat($partner, '_RootCertKeyInfo_PH')"/>
            </KeyInfo>
         </tns:Certificate>
      </xsl:if>
   <!-- 
      ****************************************************************************************
      Creation of the SecurityDetails elements (if needed).
      ****************************************************************************************
   -->
      <xsl:if test="$declareSignCert or $declareEncrCert">
         <tns:SecurityDetails tns:securityId="{concat($partnerID, '_Security')}">
               <tns:TrustAnchors>
                  <xsl:if test="$declareTrustedRoot"><tns:AnchorCertificateRef tns:certId="{concat($partnerID, '_RootCert')}"/></xsl:if>
                  <xsl:if test="$declareSignCert"><tns:AnchorCertificateRef tns:certId="{concat($partnerID, '_SigningCert')}"/></xsl:if>
                  <xsl:if test="$declareEncrCert"><tns:AnchorCertificateRef tns:certId="{concat($partnerID, '_EncryptionCert')}"/></xsl:if>
               </tns:TrustAnchors>
            </tns:SecurityDetails>
      </xsl:if>
      <xsl:if test="$declareClientServerCerts">
         <tns:SecurityDetails tns:securityId="{concat($partnerID, '_TransportSecurity')}">
            <tns:TrustAnchors>
               <xsl:if test="$declareTrustedRoot"><tns:AnchorCertificateRef tns:certId="{concat($partnerID, '_RootCert')}"/></xsl:if>
               <tns:AnchorCertificateRef tns:certId="{concat($partnerID, '_ServerCert')}"/>
               <tns:AnchorCertificateRef tns:certId="{concat($partnerID, '_ClientCert')}"/>
            </tns:TrustAnchors>
         </tns:SecurityDetails>
      </xsl:if>
   </xsl:template>

   <!-- 
      ****************************************************************************************
      Creation of the DeliveryChannel elements.
      ****************************************************************************************
   -->
   <xsl:template name="DeliveryChannels">
      <xsl:param name="partner"/>
      <xsl:call-template name="declareDefaultDeliveryChannel">
         <!-- The Default Delivery Channel -->
         <xsl:with-param name="partner" select="$partner"/>
      </xsl:call-template>
      <xsl:for-each select="$uniqueChannelDs">
         <xsl:call-template name="declareDeliveryChannel">
            <!-- The Delivery Channels for the different Actions -->
            <xsl:with-param name="partner" select="$partner"/>
            <xsl:with-param name="message" select="current()"/>
         </xsl:call-template>
      </xsl:for-each>
   </xsl:template>

   <xsl:template name="declareDefaultDeliveryChannel">
      <xsl:param name="partner"/>
      <tns:DeliveryChannel
         tns:channelId="{concat(functx:createID($partner), $defaultDeliveryChannel)}"
         tns:docExchangeId="{concat(functx:createID($partner), $defaultDocExchange)}"
         tns:transportId="{concat(functx:createID($partner), '_T_', functx:transportSecurity($defaultSecurity))}">
         <tns:MessagingCharacteristics tns:syncReplyMode="none"
            tns:ackRequested="{functx:ackRequested($defaultTransport)}"
            tns:actor="urn:oasis:names:tc:ebxml-msg:actor:toPartyMSH"
            tns:ackSignatureRequested="{functx:ackSignatureRequested($defaultTransport, $defaultSecurity)}"
            tns:duplicateElimination="{functx:ackRequested($defaultTransport)}"/>
      </tns:DeliveryChannel>
   </xsl:template>

   <xsl:template name="declareDeliveryChannel">
      <xsl:param name="partner"/>
      <xsl:param name="message"/>
      <xsl:variable name="inherited" select="functx:inheritDefault($message, $defaultMessage/default, //msg:profiles)"/>
      <xsl:variable name="currentServiceId" select="$message/parent::node()/@serviceId"/>
      <xsl:variable name="deliveryChannelID" select="functx:createChannelID($currentServiceId, $inherited)"/>
      <xsl:if test="($defaultDeliveryChannel != $deliveryChannelID)">
         <tns:DeliveryChannel
            tns:channelId="{concat(functx:createID($partner), $deliveryChannelID)}"
            tns:docExchangeId="{concat(functx:createID($partner),  functx:createDocExchangeID($message/parent::node()/@serviceId, $inherited))}"
            tns:transportId="{concat(functx:createID($partner), '_T_', functx:transportSecurity($inherited/@security))}">
            <tns:MessagingCharacteristics tns:syncReplyMode="none"
               tns:ackRequested="{functx:ackRequested($inherited/@transport)}"
               tns:actor="urn:oasis:names:tc:ebxml-msg:actor:toPartyMSH"
               tns:ackSignatureRequested="{functx:ackSignatureRequested($inherited/@transport, $inherited/@security)}"
               tns:duplicateElimination="{functx:ackRequested($inherited/@transport)}"/>
         </tns:DeliveryChannel>
      </xsl:if>
   </xsl:template>

   <!-- 
      ****************************************************************************************
      Creation of the Transport elements.
      Create only the transports that are related to MY CPA, which it based on the value of the partner in the rolebindings.
      The existence of an attribute in a message takes precedence over (the same attribute in) a profile attribute.
      ****************************************************************************************
   -->
   <xsl:template name="Transport">
      <xsl:param name="partner"/>
      <xsl:variable name="transport" select="$allMyProfiles[contains(@security, 'transport')]"/>
      <!--
         *****************************************************
         Check the occurrence of a transport WITHOUT SSL.
         Please note that this can occur together with the occurrence of SSL!
         *****************************************************
      -->
      <!-- 
         If a message does not use transport security, and a referenced profile does not specify transport security (in that case, we fall-back to the default and should check the default), then declare HTTP transport. 
      -->
      <xsl:if test="($allMyMessages[(not (contains(@security, 'transport'))) and ((not (@security)) and (not (@profile=$transport/@name)) and (not (contains($defaultMessage/default/@security, 'transport'))))]) 
         or
         (not (contains($defaultMessage/default/@security, 'transport')))"><!-- In case the Default Delivery Channel is using HTTP (for the acknowledgements). -->
         <tns:Transport tns:transportId="{concat(functx:createID($partner), '_T_HTTP')}">
            <tns:TransportSender>
               <tns:TransportProtocol tns:version="1.1">HTTP</tns:TransportProtocol>
            </tns:TransportSender>
            <tns:TransportReceiver>
               <tns:TransportProtocol tns:version="1.1">HTTP</tns:TransportProtocol>
               <tns:Endpoint tns:uri="{concat('http://',$partner, '_TransportUrl_PH')}"
                  tns:type="allPurpose"/>
            </tns:TransportReceiver>
         </tns:Transport>
      </xsl:if>
      <!--
         *****************************************************
         Check the occurrence of a transport WITH SSL. 
         Please note that this can occur together with the occurrence of other transports without SSL!
         *****************************************************
      -->
      <!-- 
         If a message does use transport security, or a referenced profile does specify transport security (if not, we fall-back to the default and should check the default), then declare HTTPS transport. 
      -->
      <xsl:if test="($allMyMessages[(contains(@security, 'transport')) or ((not (@security)) and ((@profile=$transport/@name) or (contains($defaultMessage/default/@security, 'transport'))))])
         or
         (contains($defaultMessage/default/@security, 'transport'))"><!-- In case the Default Delivery Channel is using HTTPS (for the acknowledgements). -->
         <tns:Transport tns:transportId="{concat(functx:createID($partner), '_T_HTTPS')}">
            <tns:TransportSender>
               <tns:TransportProtocol tns:version="1.1">HTTP</tns:TransportProtocol>
               <tns:TransportClientSecurity>
                  <tns:TransportSecurityProtocol tns:version="3.0">SSL</tns:TransportSecurityProtocol>
                  <tns:ClientCertificateRef tns:certId="{concat(functx:createID($partner), '_ClientCert')}"/>
                  <tns:ServerSecurityDetailsRef
                     tns:securityId="{concat(functx:createID($partner), '_TransportSecurity')}"/>
               </tns:TransportClientSecurity>
            </tns:TransportSender>
            <tns:TransportReceiver>
               <tns:TransportProtocol tns:version="1.1">HTTP</tns:TransportProtocol>
               <tns:Endpoint tns:uri="{concat('https://',$partner, '_TransportUrl_PH')}"
                  tns:type="allPurpose"/>
               <tns:TransportServerSecurity>
                  <tns:TransportSecurityProtocol tns:version="3.0">SSL</tns:TransportSecurityProtocol>
                  <tns:ServerCertificateRef tns:certId="{concat(functx:createID($partner), '_ServerCert')}"/>
                  <tns:ClientSecurityDetailsRef
                     tns:securityId="{concat(functx:createID($partner), '_TransportSecurity')}"/>
               </tns:TransportServerSecurity>
            </tns:TransportReceiver>
         </tns:Transport>
      </xsl:if>
   </xsl:template>

   <!-- 
      ****************************************************************************************
      Creation of the DocExchange element.
      ****************************************************************************************
   -->
   <xsl:template name="DocExchanges">
      <xsl:param name="partner"/>
      <xsl:call-template name="declareDocExchange">
         <!-- The default docExchanges for the signalling and default ebMS operations. -->
         <xsl:with-param name="partner" select="$partner"/>
         <xsl:with-param name="Id" select="concat(functx:createID($partner), $defaultDocExchange)"/>
         <xsl:with-param name="message" select="$defaultMessage/default"/>
      </xsl:call-template>
      <xsl:for-each select="$uniqueDocExchangeIDs">
         <!-- The docExchanges for the different Actions -->
         <xsl:variable name="inherited" select="functx:inheritDefault(current(), $defaultMessage/default, //msg:profiles)"/>
         <xsl:variable name="currentServiceId" select="current()/parent::node()/@serviceId"/>
         <xsl:variable name="docExchangeID"
            select="functx:createDocExchangeID($currentServiceId, $inherited)"/>
         <xsl:if test="($defaultDocExchange != $docExchangeID)">
            <xsl:call-template name="declareDocExchange">
               <xsl:with-param name="partner" select="$partner"/>
               <xsl:with-param name="Id" select="concat(functx:createID($partner), $docExchangeID)"/>
               <xsl:with-param name="message" select="$inherited"/>
            </xsl:call-template>
         </xsl:if>
      </xsl:for-each>
   </xsl:template>

   <xsl:template name="declareDocExchange">
      <xsl:param name="partner"/>
      <xsl:param name="Id"/>
      <xsl:param name="message"/>
      <tns:DocExchange tns:docExchangeId="{$Id}">
         <tns:ebXMLSenderBinding tns:version="2.0">
            <xsl:if test="$message/@transport='reliable'">
               <tns:ReliableMessaging>
                  <tns:Retries><xsl:value-of select="$message/@retryCount"/></tns:Retries>
                  <tns:RetryInterval><xsl:value-of select="$message/@retryInterval"/></tns:RetryInterval>
                  <tns:MessageOrderSemantics>NotGuaranteed</tns:MessageOrderSemantics>
               </tns:ReliableMessaging>
               <tns:PersistDuration><xsl:value-of select="functx:deriveSafePersistDuration($message/@retryCount, $message/@retryInterval)"/></tns:PersistDuration>
            </xsl:if>
            <xsl:if test="contains($message/@security, 'payload') or contains($message/@security, 'sign')">
               <tns:SenderNonRepudiation>
                  <tns:NonRepudiationProtocol>http://www.w3.org/2000/09/xmldsig#</tns:NonRepudiationProtocol>
                  <tns:HashFunction>http://www.w3.org/2000/09/xmldsig#sha1</tns:HashFunction>
                  <tns:SignatureAlgorithm>http://www.w3.org/2000/09/xmldsig#rsa-sha1</tns:SignatureAlgorithm>
                  <tns:SigningCertificateRef tns:certId="{concat(functx:createID($partner), '_SigningCert')}"/>
               </tns:SenderNonRepudiation>
            </xsl:if>
            <xsl:if test="contains($message/@security, 'payload')">
               <tns:SenderDigitalEnvelope>
                  <tns:DigitalEnvelopeProtocol tns:version="1.0">XMLENC</tns:DigitalEnvelopeProtocol>
                  <tns:EncryptionAlgorithm tns:w3c="http://www.w3.org/2001/04/xmlenc#aes256-cbc"
                     tns:minimumStrength="256" tns:oid="2.16.840.1.101.3.4.1.42"
                     >http://www.w3.org/2001/04/xmlenc#aes256-cbc</tns:EncryptionAlgorithm>
                  <tns:EncryptionSecurityDetailsRef tns:securityId="{concat(functx:createID($partner), '_Security')}"/>
               </tns:SenderDigitalEnvelope>
            </xsl:if>
         </tns:ebXMLSenderBinding>
         <tns:ebXMLReceiverBinding tns:version="2.0">
            <xsl:if test="$message/@transport='reliable'">
               <tns:ReliableMessaging>
                  <tns:Retries><xsl:value-of select="$message/@retryCount"/></tns:Retries>
                  <tns:RetryInterval><xsl:value-of select="$message/@retryInterval"/></tns:RetryInterval>
                  <tns:MessageOrderSemantics>NotGuaranteed</tns:MessageOrderSemantics>
               </tns:ReliableMessaging>
               <tns:PersistDuration><xsl:value-of select="functx:deriveSafePersistDuration($message/@retryCount, $message/@retryInterval)"/></tns:PersistDuration>
            </xsl:if>
            <xsl:if test="contains($message/@security, 'payload') or contains($message/@security, 'sign')">
               <tns:ReceiverNonRepudiation>
                  <tns:NonRepudiationProtocol>http://www.w3.org/2000/09/xmldsig#</tns:NonRepudiationProtocol>
                  <tns:HashFunction>http://www.w3.org/2000/09/xmldsig#sha1</tns:HashFunction>
                  <tns:SignatureAlgorithm>http://www.w3.org/2000/09/xmldsig#rsa-sha1</tns:SignatureAlgorithm>
                  <tns:SigningSecurityDetailsRef tns:securityId="{concat(functx:createID($partner), '_Security')}"/>
               </tns:ReceiverNonRepudiation>
            </xsl:if>
            <xsl:if test="contains($message/@security, 'payload')">
               <tns:ReceiverDigitalEnvelope>
                  <tns:DigitalEnvelopeProtocol tns:version="1.0">XMLENC</tns:DigitalEnvelopeProtocol>
                  <tns:EncryptionAlgorithm tns:w3c="http://www.w3.org/2001/04/xmlenc#aes256-cbc"
                     tns:minimumStrength="256" tns:oid="2.16.840.1.101.3.4.1.42"
                     >http://www.w3.org/2001/04/xmlenc#aes256-cbc</tns:EncryptionAlgorithm>
                  <tns:EncryptionCertificateRef tns:certId="{concat(functx:createID($partner), '_EncryptionCert')}"/>
               </tns:ReceiverDigitalEnvelope>
            </xsl:if>
         </tns:ebXMLReceiverBinding>
      </tns:DocExchange>
   </xsl:template>

   <!-- 
      ****************************************************************************************
      Creation of the OverrideMshActionBinding element.
      NOT USED
      ****************************************************************************************
   -->
   <xsl:template name="OverrideMshActionBinding">
      <xsl:param name="partner"/>
      <xsl:param name="otherPartner"/>
      <xsl:param name="serviceId"/>
      <xsl:param name="role"/>
      <xsl:param name="otherRole"/>
      <xsl:apply-templates
         select="//msg:service[@serviceId=$serviceId]/msg:message[@from=$role][@to=$otherRole]"
         mode="override-msh-action">
         <xsl:with-param name="partner" select="$partner"/>
         <xsl:with-param name="otherPartner" select="$otherPartner"/>
      </xsl:apply-templates>
   </xsl:template>
   <!--
      ****************************************************************************************
      Creation of the OverrideMshActionBinding, based on the profiles.
      NOT USED
      ****************************************************************************************
   -->
   <xsl:template match="msg:service/msg:message" mode="override-msh-action">
      <xsl:param name="partner"/>
      <xsl:param name="otherPartner"/>
      <xsl:variable name="actionId">
         <xsl:choose>
            <xsl:when test="@id!=''">
               <xsl:value-of select="functx:createID(@id)"/>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="functx:createID(descendant-or-self::text())"/>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>
      <xsl:variable name="actionChannelId"
         select="concat(functx:createID($partner), functx:createChannelID(parent::node()/@serviceId, .))"/>
      <xsl:choose>
         <xsl:when
            test="(concat(functx:createID($partner),'_',$defaultDeliveryChannel) != $actionChannelId)">
            <tns:OverrideMshActionBinding tns:action="{descendant-or-self::text()}"
               tns:channelId="{$actionChannelId}"/>
         </xsl:when>
         <xsl:otherwise/>
      </xsl:choose>
   </xsl:template>
   <!-- 
      ****************************************************************************************
      Creation of the SimplePart.
      ****************************************************************************************
   -->
   <xsl:template name="SimpePart">
      <tns:SimplePart tns:id="DefaultSimplePart" tns:mimetype="application/xml"/>
   </xsl:template>
   <!-- 
      ****************************************************************************************
      Creation of the Packaging
      ****************************************************************************************
   -->
   <xsl:template name="Packaging">
      <tns:Packaging tns:id="defaultPackage_Profile">
         <tns:ProcessingCapabilities tns:parse="true" tns:generate="true"/>
         <tns:CompositeList>
            <tns:Composite tns:id="DefaultComposite" tns:mimetype="type=text/xml">
               <tns:Constituent tns:idref="DefaultSimplePart"/>
            </tns:Composite>
         </tns:CompositeList>
      </tns:Packaging>
   </xsl:template>
   <!-- 
      ****************************************************************************************
      Creation of the Comment
      ****************************************************************************************
   -->
   <xsl:template name="Comments">
      <xsl:variable name="creatiedatum" select="current-dateTime()"/>
      <tns:Comment xml:lang="en-UK">This CPA is created by means of the CPA Creation Toolkit 4.0.</tns:Comment>
      <tns:Comment xml:lang="en-UK">The toolkit is provided by the Justitiele Informatiedienst, department EBV, The Netherlands (http://www.justid.nl).</tns:Comment>
      <tns:Comment xml:lang="en-UK">The sourcecode of the toolkit is licensed under the European Union Public Licence - EUPL v.1.1.</tns:Comment>
      <tns:Comment xml:lang="en-UK">Creation date/time: <xsl:value-of select="$creatiedatum"/></tns:Comment>
   </xsl:template>

</xsl:stylesheet>
