<?xml version="1.0" encoding="utf-8"?>
<!--
   
   ****************************************************************************************
   Transformation to update the CPA ID, Creation Date and End Date, as well
   as some 'postprocessing related to the values for:
   - HREF and UUID in the ProcessSpecification and Role (See also CPA template xslt).
   - HTTPS and HTTP port values.
   
   June 2010
   
   European Union Public Licence - EUPL v.1.1
   
   Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl
   
   Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
   European Commission - subsequent versions of the EUPL (the "Licence");
   You may not use this work except in compliance with the Licence.
   You may obtain a copy of the Licence at:
   
   http://ec.europa.eu/idabc/eupl
   
   Unless required by applicable law or agreed to in writing, software distributed
   under the Licence is distributed on an "AS IS" basis, 
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the Licence for the specific language governing permissions and limitations
   under the Licence.
   
   ****************************************************************************************
   
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
   xmlns:xs="http://www.w3.org/2001/XMLSchema"
   xmlns:tns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
   xmlns:ds="http://www.w3.org/2000/09/xmldsig#" 
   xmlns:xlink="http://www.w3.org/1999/xlink"
   xmlns:functx="http://www.functx.com"
   xsi:schemaLocation="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd                  http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
   version="2.0"
   exclude-result-prefixes="xsl functx xs xsi">
   
   <xsl:output  method="xml" encoding="utf-8" indent="yes"/>

   <xsl:param name="CPA_ID"/>
   <xsl:param name="CPA_ID_POSTFIX">AAAA</xsl:param>
   <xsl:param name="CPA_STARTDATE"/>
   <xsl:param name="CPA_ENDDATE"/>
   <xsl:param name="BASE_HREF"/>
   <xsl:param name="BASE_UUID"/>
   <xsl:param name="HTTPS_PORTNUMBER_INCREMENT">363</xsl:param>

   <xsl:include href="functions.xsl"/>

   <xsl:template match="tns:CollaborationProtocolAgreement/@tns:cpaid">
      <xsl:variable name="service" select="//tns:PartyInfo[1]/tns:CollaborationRole/tns:ServiceBinding/tns:Service[1]/descendant-or-self::text()"/>
      <xsl:variable name="cpaIdPrefix" select="concat(//tns:PartyInfo[1]/tns:PartyId[1]/descendant-or-self::text(), '-', //tns:PartyInfo[2]/tns:PartyId[1]/descendant-or-self::text())">
      </xsl:variable>
      <xsl:attribute name="tns:cpaid">
         <xsl:choose>
            <xsl:when test="$CPA_ID !=''">
               <xsl:value-of select="$CPA_ID"/>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of  select="functx:cpaid(concat($service[1], '_',$cpaIdPrefix, '_', $CPA_ID_POSTFIX))"/>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:attribute>
   </xsl:template>

   <xsl:template match="tns:CollaborationProtocolAgreement/tns:Start">
      <xsl:element name="tns:Start">
         <xsl:choose>
            <xsl:when test="$CPA_STARTDATE !=''">
               <xsl:value-of select="$CPA_STARTDATE"/>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="descendant-or-self::text()"/>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:element>
   </xsl:template>

   <xsl:template match="tns:CollaborationProtocolAgreement/tns:End">
      <xsl:element name="tns:End">
         <xsl:choose>
            <xsl:when test="$CPA_ENDDATE !=''">
               <xsl:value-of select="$CPA_ENDDATE"/>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="descendant-or-self::text()"/>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:element>
   </xsl:template>

   <xsl:template match="//tns:Role/@xlink:href">
      <xsl:attribute name="xlink:href" select="functx:replace('HTTP://BASE-HREF-PH/', $BASE_HREF, .)"/><!-- Replace this .. by .. in ..  -->
   </xsl:template>

   <xsl:template match="//tns:ProcessSpecification/@tns:uuid">
      <xsl:attribute name="tns:uuid" select="functx:replace('URN:BASE:UUID:PH:', $BASE_UUID, .)"/><!-- Replace this .. by .. in ..  -->
   </xsl:template>

   <xsl:template match="//tns:ProcessSpecification/@xlink:href">
      <xsl:attribute name="xlink:href" select="functx:replace('HTTP://BASE-HREF-PH/', $BASE_HREF, .)"/><!-- Replace this .. by .. in ..  -->
   </xsl:template>

   <xsl:template match="//tns:Endpoint/@tns:uri">
      <xsl:variable name="httpPrefix" select="substring-before(., '://')"/>
      <xsl:variable name="removedHttpPrefix" select="substring-after(., '://')"/>
      <xsl:variable name="path" select="substring-after($removedHttpPrefix, '/')"/>
      <xsl:variable name="portNumber" select="substring-after(substring-before($removedHttpPrefix, '/'), ':')"/>
      <xsl:variable name="containsHttps" select="contains($httpPrefix, 's') or contains($httpPrefix, 'S')"/>
      <xsl:variable name="domain">
         <xsl:choose>
            <xsl:when test="$portNumber">
               <xsl:value-of select="substring-before($removedHttpPrefix, ':')"/>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="substring-before($removedHttpPrefix, '/')"/>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>
      <xsl:variable name="httpsURL">
         <xsl:choose>
            <xsl:when test="($containsHttps)">
               <xsl:value-of select="."/>
            </xsl:when>
            <xsl:when test="not ($portNumber)">
               <xsl:value-of select="concat('https://', $removedHttpPrefix)"/>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="concat('https://', $domain, ':', string(xs:integer($portNumber) + xs:integer($HTTPS_PORTNUMBER_INCREMENT)), '/', $path)"/>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>
      <xsl:variable name="httpURL">
         <xsl:choose>
            <xsl:when test="not ($containsHttps)">
               <xsl:value-of select="."/>
            </xsl:when>
            <xsl:when test="not ($portNumber)">
               <xsl:value-of select="concat('http://', $removedHttpPrefix)"/>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="concat('http://', $domain, ':', string(xs:integer($portNumber) - xs:integer($HTTPS_PORTNUMBER_INCREMENT)), '/', $path)"/>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>
      <xsl:choose>
         <xsl:when test="parent::node()/parent::node()/tns:TransportServerSecurity">            
            <xsl:attribute name="tns:uri" select="$httpsURL"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:attribute name="tns:uri" select="$httpURL"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="@*|node()">
      <xsl:copy>
         <xsl:apply-templates select="@*"/>
         <xsl:apply-templates/>
      </xsl:copy>
   </xsl:template>

</xsl:stylesheet>
