<?xml version="1.0" ?>
<!--
    
    ****************************************************************************************
    This XSLT  will create NDDs for JustID JAB/JUBES profiles.
    March 2009
    
    European Union Public Licence - EUPL v.1.1
    
    Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl
    
    Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
    European Commission - subsequent versions of the EUPL (the "Licence");
    You may not use this work except in compliance with the Licence.
    You may obtain a copy of the Licence at:
    
    http://ec.europa.eu/idabc/eupl
    
    Unless required by applicable law or agreed to in writing, software distributed
    under the Licence is distributed on an "AS IS" basis, 
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the Licence for the specific language governing permissions and limitations
    under the Licence.
    
    ****************************************************************************************
    
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:tns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
    xmlns:ds="http://www.w3.org/2000/09/xmldsig#"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd 
    http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
    xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias" version="2.0"
    exclude-result-prefixes="xsl axsl">
    
    <xsl:output method="xml" encoding="utf-8" indent="yes"/>

    <xsl:namespace-alias stylesheet-prefix="axsl" result-prefix="xsl"/>

    <xsl:param name="resultDocumentDir"/>
    <xsl:param name="namePrefix"/>
    <xsl:param name="parameterdir"/>

    <xsl:template match="NegotiationDescriptorDocument">
        <xsl:apply-templates select="descendant-or-self::Document">
            <xsl:with-param name="resultDocumentDir" select="$resultDocumentDir"/>
            <xsl:with-param name="parameterdir" select="$parameterdir"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="NegotiationDescriptorDocument/Documents/Document[@type='cpa']">

        <xsl:param name="resultDocumentDir"/>
        <xsl:param name="parameterdir"/>

        <xsl:variable name="DocId" select="@id"/>
        <xsl:variable name="NddId">
            <xsl:value-of select="/NegotiationDescriptorDocument/@NddId"/>
        </xsl:variable>
        <xsl:variable name="xsltstylesheet">
            <xsl:choose>
                <xsl:when test="$namePrefix=''">
                    <!--
                        *******************************************************************************************************************
                        README
                        The following line is simplified for the CPA Creation Toolkit. The original line was:
                        <xsl:value-of select="concat($resultDocumentDir, '/', $NddId,'.xsl')"/>
                        *******************************************************************************************************************
                    -->
                    <xsl:value-of
                        select="concat($resultDocumentDir, '/', 'cpa-template.xsl')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of
                        select="concat($resultDocumentDir, '/', $namePrefix, '.', $NddId,  '.xsl')"
                    />
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:result-document href="{$xsltstylesheet}">
            <axsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:tns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns:xlink="http://www.w3.org/1999/xlink"
                xmlns:msg="http://www.sonnenglanz.net/2010/07/09/smf.services.2.0.xsd"
                xmlns:msgpar="http://www.sonnenglanz.net/2010/07/09/smf.parameters.2.0.xsd"
                xmlns:functx="http://www.functx.com"
                xsi:schemaLocation="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd 
                http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
                version="2.0"
                exclude-result-prefixes="xsl functx msg msgpar">

                <axsl:param name="resultDocumentDir"/>
                
                <!--
                    *******************************************************************************************************************
                    README
                    The following param spec is not needed by CPA Creation Toolkit. But it is needed in the commandline version!!!
                    <axsl:param name="collaborationSpec">../04-collaborations/OMAF.xml</axsl:param>
                    *******************************************************************************************************************
                -->
                <xsl:apply-templates select="//Partner" mode="paramsource">
                    <xsl:with-param name="parameterdir" select="$parameterdir"/>
                    <xsl:with-param name="DocId" select="$DocId"/>
                </xsl:apply-templates>
                
                <axsl:param name="CPA_ID"> </axsl:param>
                <axsl:param name="CPA_STARTDATE"> </axsl:param>
                <axsl:param name="CPA_ENDDATE"> </axsl:param>

                <xsl:variable name="fromParty" select="//Partner[1]/@role"/>
                <xsl:variable name="toParty" select="//Partner[2]/@role"/>
                
                <xsl:apply-templates select="//Binding[@docref=$DocId]" mode="conditions"/>
                <!--<xsl:apply-templates select="//Binding[@docref=$DocId]" mode="inputparams"/>-->
                <xsl:apply-templates select="//Binding[@docref=$DocId]" mode="body"/>

                <axsl:template match="@*|node()">
                    <axsl:copy>
                        <axsl:apply-templates select="@*"/>
                        <axsl:apply-templates/>
                    </axsl:copy>
                </axsl:template>

                <!-- 
                    Added to proces PartyIdList based on the element PartyId within the (CPA) template.
                    Does only work in the mode 'MultipleIds' .
                -->
                <axsl:template match="tns:CollaborationProtocolAgreement/tns:PartyInfo/tns:PartyId"
                    mode="MultipleIds">
                    <axsl:param name="aPartyId"/>
                    <axsl:copy>
                        <axsl:apply-templates select="@*"/>
                        <axsl:value-of select="$aPartyId"/>
                    </axsl:copy>
                </axsl:template>
                
            </axsl:stylesheet>
            
        </xsl:result-document>

    </xsl:template>

    <xsl:template match="NegotiationDescriptorDocument/Documents/Document[@type='wsdl']">

        <xsl:param name="resultDocumentDir"/>

        <xsl:variable name="DocId" select="@id"/>
        <xsl:variable name="NddId">
            <xsl:value-of select="/NegotiationDescriptorDocument/@NddId"/>
        </xsl:variable>
        <xsl:variable name="xsltstylesheet">
            <xsl:value-of select="concat($resultDocumentDir, $NddId, '.', $DocId,'.xsl')"/>
        </xsl:variable>

        <xsl:result-document href="{$xsltstylesheet}">
            <axsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" version="2.0"
                xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/"
                xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
                xmlns="http://schemas.xmlsoap.org/wsdl/">

                <xsl:apply-templates select="//Partner" mode="paramsource">
                    <xsl:with-param name="parameterdir" select="$parameterdir"/>
                    <xsl:with-param name="DocId" select="$DocId"/>
                </xsl:apply-templates>
                <xsl:apply-templates select="//Binding[@docref=$DocId]" mode="conditions"/>
                <xsl:apply-templates select="//Binding[@docref=$DocId]" mode="inputparams"/>
                <xsl:apply-templates select="//Binding[@docref=$DocId]" mode="body"/>

                <axsl:template match="@*|node()">
                    <axsl:copy>
                        <axsl:apply-templates select="@*"/>
                        <axsl:apply-templates/>
                    </axsl:copy>
                </axsl:template>
            </axsl:stylesheet>

        </xsl:result-document>
    </xsl:template>

    <xsl:template match="Partner" mode="paramsource">

        <xsl:param name="parameterdir"/>
        <xsl:param name="DocId"/>
        <xsl:param name="collaborationSpec"/>

        <xsl:variable name="NddId">
            <xsl:value-of select="//ancestor::node()/@NddId"/>
        </xsl:variable>
        <xsl:variable name="Partner">
            <xsl:value-of select="@role"/>
        </xsl:variable>
        <xsl:variable name="ParametersSource">
            <!--
                *******************************************************************************************************************
                README
                The following line is not needed by CPA Creation Toolkit. But it is needed in the commandline version!!!
                <xsl:value-of select="concat('concat(','''../../'',','document($collaborationSpec)//binding[@name=''', $Partner,''']/instance/@file)')"/>
                
                The following line was used to create more elaborate filenames.
                <xsl:value-of select="concat($parameterdir,$NddId,'.', $DocId, '.', @role,'.','parameters.xml')"/>
                
                Actually, because all files are already 'isolated' based on foldernames, we can use simple filenames.
                The filename is based on the rolename only.
                *******************************************************************************************************************
            -->
            <xsl:value-of
                select="concat($parameterdir,@role,'.','parameters.xml')"/>
        </xsl:variable>
        
        <axsl:param name="{concat('ParametersSource',$Partner)}">
            <xsl:value-of select="concat('../',$ParametersSource)"/>
        </axsl:param>

    </xsl:template>

    <xsl:template match="Binding[@xmlType='attribute']" mode="conditions">
        <xsl:variable name="default" select="descendant-or-self::text()"/>
        <xsl:variable name="name">
            <xsl:value-of select="parent::Parameter/@name"/>
        </xsl:variable>
        <xsl:if test="parent::Parameter/@mode = 'conditional'">
            <axsl:param name="{concat('PARAM_CONDITION_',ancestor-or-self::Partner/@role)}"
            >default</axsl:param>
        </xsl:if>
    </xsl:template>

    <!--
        ************************
        xmlType = attribute
        ************************
    -->
    <xsl:template match="Binding[@xmlType='attribute']" mode="body">
        <xsl:variable name="required" select="parent::Parameter/@required"/>
        <xsl:variable name="default" select="descendant-or-self::text()"/>
        <xsl:variable name="name" select="parent::Parameter/@name"/>
        <xsl:variable name="fileName"
            select="concat('$ParametersSource',ancestor-or-self::Partner/@role)"/>

        <axsl:template match="{concat(@xmlContext,'/@',@xmlAttribute)}">
            <xsl:choose>
                <xsl:when test="parent::Parameter/@mode = 'conditional'">
                    <axsl:variable
                        name="{concat(ancestor-or-self::Partner/@role,'_',$name,parent::Parameter/@type)}">
                        <axsl:copy-of
                            select="document({$fileName})//msgpar:parameter[@name='{$name}']/child::node()"
                        />
                    </axsl:variable>
                    <axsl:variable
                        name="{concat(@xmlOtherPartnerContext,'_',$name,parent::Parameter/@type)}">
                        <axsl:copy-of
                            select="document({concat('$ParametersSource',@xmlOtherPartnerContext)})//msgpar:parameter[@name='{$name}']/child::node()"
                        />
                    </axsl:variable>
                </xsl:when>
                <xsl:otherwise>
                    <axsl:variable
                        name="{concat(ancestor-or-self::Partner/@role,'_',$name,parent::Parameter/@type)}">
                        <axsl:copy-of
                            select="document({$fileName})//msgpar:parameter[@name='{$name}']/child::text()"
                        />
                    </axsl:variable>
                </xsl:otherwise>
            </xsl:choose>

            <xsl:choose>
                <xsl:when test="parent::Parameter/@mode = 'conditional'">

                    <axsl:variable name="condition"
                        select="{concat('$PARAM_CONDITION_',ancestor-or-self::Partner/@role)}"/>
                    <axsl:variable name="switch"
                        select="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}/msgpar:condition[@name=$condition]/@switch"/>
                    <axsl:variable name="value"
                        select="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}/msgpar:condition[@name=$condition]"/>
                    <axsl:variable name="otherValue"
                        select="{concat('$',@xmlOtherPartnerContext,'_',$name)}/msgpar:condition[@name=$condition]"/>
                    <axsl:attribute name="{@xmlAttribute}">
                        <axsl:choose>
                            <axsl:when test="$switch !='true'">
                                <axsl:value-of select="$value"/>
                            </axsl:when>
                            <axsl:otherwise>
                                <axsl:value-of select="$otherValue"/>
                            </axsl:otherwise>
                        </axsl:choose>
                    </axsl:attribute>

                </xsl:when>
                <xsl:otherwise>

                    <axsl:attribute name="{@xmlAttribute}">
                        <axsl:choose>
                            <axsl:when
                                test="{concat('$',ancestor-or-self::Partner/@role,'_',$name)} !=''">
                                <axsl:value-of
                                    select="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}"
                                />
                            </axsl:when>
                            <axsl:otherwise>

                                <xsl:choose>
                                    <xsl:when test="$required ='true'">
                                        <axsl:value-of select="descendant-or-self::node()"/>
                                        <axsl:message terminate="yes">
                                            <axsl:text>
                                                <xsl:value-of
                                                  select="concat('The required parameter ', $name, ' is not given.')"
                                                />
                                            </axsl:text>
                                        </axsl:message>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <axsl:value-of>
                                            <xsl:value-of select="$default"/>
                                        </axsl:value-of>
                                    </xsl:otherwise>
                                </xsl:choose>

                            </axsl:otherwise>
                        </axsl:choose>
                    </axsl:attribute>

                </xsl:otherwise>
            </xsl:choose>
        </axsl:template>
    </xsl:template>
    
    <!--
        ************************
        xmlType = text
        ************************
    -->
    <xsl:template match="Binding[@xmlType='text']" mode="body">
        <xsl:variable name="required" select="parent::Parameter/@required"/>
        <xsl:variable name="default" select="descendant-or-self::text()"/>
        <xsl:variable name="name">
            <xsl:value-of select="parent::Parameter/@name"/>
        </xsl:variable>

        <xsl:variable name="fileName">
            <xsl:value-of select="concat('$ParametersSource',ancestor-or-self::Partner/@role)"/>
        </xsl:variable>

        <axsl:template match="{@xmlContext}">
            <axsl:variable name="{concat(ancestor-or-self::Partner/@role,'_',$name)}">
                <axsl:copy-of
                    select="document({$fileName})//msgpar:parameter[@name='{$name}']/child::text()"/>
            </axsl:variable>
            <axsl:copy>
                <axsl:apply-templates select="@*"/>

                <axsl:choose>
                    <axsl:when test="{concat('$',ancestor-or-self::Partner/@role,'_',$name)} !=''">
                        <axsl:copy-of
                            select="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}"/>
                    </axsl:when>
                    <axsl:otherwise>

                        <xsl:choose>
                            <xsl:when test="$required ='true'">
                                <axsl:value-of select="descendant::node()"/>
                                <axsl:message terminate="yes">
                                    <axsl:text>
                                        <xsl:value-of
                                            select="concat('The required parameter ', $name, ' is not given.')"
                                        />
                                    </axsl:text>
                                </axsl:message>
                            </xsl:when>
                            <xsl:otherwise>
                                <axsl:value-of>
                                    <xsl:value-of select="$default"/>
                                </axsl:value-of>
                            </xsl:otherwise>
                        </xsl:choose>

                    </axsl:otherwise>
                </axsl:choose>

            </axsl:copy>
        </axsl:template>
    </xsl:template>

    <!--
        ************************
        xmlType = value
        ************************
    -->
    <xsl:template match="Binding[@xmlType='value']" mode="body">
        <xsl:variable name="required" select="parent::Parameter/@required"/>
        <xsl:variable name="default" select="node()"/>
        <xsl:variable name="name">
            <xsl:value-of select="parent::Parameter/@name"/>
        </xsl:variable>

        <xsl:variable name="fileName">
            <xsl:value-of select="concat('$ParametersSource',ancestor-or-self::Partner/@role)"/>
        </xsl:variable>

        <axsl:template match="{@xmlContext}">
            <axsl:variable name="{concat(ancestor-or-self::Partner/@role,'_',$name)}">
                <axsl:copy-of
                    select="document({$fileName})//msgpar:parameter[@name='{$name}']/child::node()"/>
            </axsl:variable>

            <axsl:copy>
                <axsl:choose>
                    <axsl:when test="{concat('$',ancestor-or-self::Partner/@role,'_',$name)} !=''">
                        <axsl:apply-templates select="@*"/>
                        <axsl:copy-of
                            select="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}"/>
                    </axsl:when>
                    <axsl:otherwise>
                        <xsl:choose>
                            <xsl:when test="$required ='true'">
                                <axsl:apply-templates select="@*"/>
                                <axsl:copy-of select="node()"/>
                                <axsl:message terminate="yes">
                                    <axsl:text>
                                        <xsl:value-of
                                            select="concat('The required parameter ', $name, ' is not given.')"
                                        />
                                    </axsl:text>
                                </axsl:message>
                            </xsl:when>
                            <xsl:otherwise>
                                <axsl:apply-templates select="@*"/>
                                <xsl:copy-of select="$default"/>
                            </xsl:otherwise>
                        </xsl:choose>
                        <!--
                        <axsl:apply-templates select="@*"/>
                        <axsl:copy-of select="node()"/>
                        <xsl:if test="$required ='true'">
                            <axsl:message terminate="yes">
                                <axsl:text>
                                    <xsl:value-of
                                        select="concat('The required parameter ', $name, ' is not given.')"
                                    />
                                </axsl:text>
                            </axsl:message>
                        </xsl:if>
                        -->
                    </axsl:otherwise>
                </axsl:choose>
            </axsl:copy>
        </axsl:template>
    </xsl:template>

    <!--
        ************************
        xmlType = valueList
        ************************
    -->
    <xsl:template match="Binding[@xmlType='valueList']" mode="body">
        <xsl:variable name="required" select="parent::Parameter/@required"/>
        <xsl:variable name="default" select="node()"/>
        <xsl:variable name="name">
            <xsl:value-of select="parent::Parameter/@name"/>
        </xsl:variable>

        <xsl:variable name="fileName">
            <xsl:value-of select="concat('$ParametersSource',ancestor-or-self::Partner/@role)"/>
        </xsl:variable>

        <axsl:template match="{@xmlContext}">
            <axsl:variable name="{concat(ancestor-or-self::Partner/@role,'_',$name)}">
                <axsl:copy-of
                    select="document({$fileName})//msgpar:parameter[@name='{$name}']/child::node()"/>
            </axsl:variable>

            <axsl:variable name="thisNode" select="."/>
            <axsl:choose>
                <axsl:when test="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}  !=''">
                    <axsl:for-each
                        select="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}/msgpar:value">
                        <axsl:apply-templates select="$thisNode" mode="MultipleIds">
                            <axsl:with-param name="aPartyId" select="child::text()"/>
                        </axsl:apply-templates>
                    </axsl:for-each>
                </axsl:when>
                <axsl:otherwise>
                    <xsl:choose>
                        <xsl:when test="$required ='true'">
                            <axsl:apply-templates select="@*"/>
                            <axsl:copy-of select="node()"/>
                            <axsl:message terminate="yes">
                                <axsl:text>
                                    <xsl:value-of
                                        select="concat('The required parameter ', $name, ' is not given.')"
                                    />
                                </axsl:text>
                            </axsl:message>
                        </xsl:when>
                        <xsl:otherwise>
                            <axsl:apply-templates select="@*"/>
                            <xsl:copy-of select="$default"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </axsl:otherwise>
            </axsl:choose>

        </axsl:template>
    </xsl:template>

    <!--
        ************************
        xmlType = replacenode
        ************************
    -->
    <xsl:template match="Binding[@xmlType='replacenode']" mode="body">
        <xsl:variable name="required" select="parent::Parameter/@required"/>
        <xsl:variable name="default" select="node()"/>
        <xsl:variable name="name">
            <xsl:value-of select="parent::Parameter/@name"/>
        </xsl:variable>

        <xsl:variable name="fileName">
            <xsl:value-of select="concat('$ParametersSource',ancestor-or-self::Partner/@role)"/>
        </xsl:variable>

        <axsl:template match="{@xmlContext}">
            <axsl:variable name="{concat(ancestor-or-self::Partner/@role,'_',$name)}">
                <axsl:copy-of
                    select="document({$fileName})//msgpar:parameter[@name='{$name}']/child::node()"/>
            </axsl:variable>

            <axsl:choose>
                <axsl:when test="{concat('$',ancestor-or-self::Partner/@role,'_',$name)} !=''">
                    <axsl:copy-of select="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}"/>
                </axsl:when>
                <axsl:otherwise>

                    <xsl:choose>
                        <xsl:when test="$required ='true'">
                            <axsl:apply-templates select="@*"/>
                            <axsl:copy-of select="node()"/>
                            <axsl:message terminate="yes">
                                <axsl:text>
                                    <xsl:value-of
                                        select="concat('The required parameter ', $name, ' is not given.')"
                                    />
                                </axsl:text>
                            </axsl:message>
                        </xsl:when>
                        <xsl:otherwise>
                            <axsl:apply-templates select="@*"/>
                            <xsl:copy-of select="$default"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </axsl:otherwise>
            </axsl:choose>
        </axsl:template>
    </xsl:template>

    <!--
        ************************
        xmlType = attributeSelect
        ************************
    -->
    <xsl:template match="Binding[@xmlType='attributeSelect']" mode="body">
        <xsl:variable name="required" select="parent::Parameter/@required"/>
        <xsl:variable name="default" select="node()"/>
        <xsl:variable name="name">
            <xsl:value-of select="parent::Parameter/@name"/>
        </xsl:variable>

        <xsl:variable name="name">
            <xsl:value-of select="parent::Parameter/@name"/>
        </xsl:variable>
        <xsl:variable name="fileName">
            <xsl:value-of select="concat('$ParametersSource',ancestor-or-self::Partner/@role)"/>
        </xsl:variable>

        <axsl:template match="{concat(@xmlContext,'/@',@xmlAttribute)}">
            <axsl:variable
                name="{concat(ancestor-or-self::Partner/@role,'_',$name,parent::Parameter/@type)}">
                <axsl:copy-of
                    select="document({$fileName})//msgpar:parameter[@name='{$name}']/child::node()"/>
            </axsl:variable>

            <axsl:attribute name="{@xmlAttribute}">
                <axsl:choose>
                    <axsl:when test="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}  !=''">
                        <axsl:value-of
                            select="{concat('$',ancestor-or-self::Partner/@role,'_',$name)}"/>
                    </axsl:when>
                    <axsl:otherwise>
                        <xsl:choose>
                            <xsl:when test="$required ='true'">
                                <axsl:apply-templates select="@*"/>
                                <axsl:copy-of select="node()"/>
                                <axsl:message terminate="yes">
                                    <axsl:text>
                                        <xsl:value-of
                                            select="concat('The required parameter ', $name, ' is not given.')"
                                        />
                                    </axsl:text>
                                </axsl:message>
                            </xsl:when>
                            <xsl:otherwise>
                                <axsl:apply-templates select="@*"/>
                                <xsl:copy-of select="$default"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </axsl:otherwise>
                </axsl:choose>
            </axsl:attribute>
        </axsl:template>
    </xsl:template>

    <xsl:template match="@*|node()" mode="conditions">
        <xsl:apply-templates mode="conditions"/>
    </xsl:template>

    <xsl:template match="@*|node()" mode="inputparams">
        <xsl:apply-templates mode="inputparams"/>
    </xsl:template>

    <xsl:template match="@*|node()" mode="body">
        <xsl:apply-templates mode="body"/>
    </xsl:template>

    <xsl:template match="@*|node()">
        <xsl:apply-templates/>
    </xsl:template>

</xsl:stylesheet>
