<?xml version="1.0" encoding="utf-8"?>
<!--
   ****************************************************************************************
   This XSLT file will create an NDD document for a given profile and bilateral collaboration.
   May 2009
   
   European Union Public Licence - EUPL v.1.1
   
   Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl
   
   Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
   European Commission - subsequent versions of the EUPL (the "Licence");
   You may not use this work except in compliance with the Licence.
   You may obtain a copy of the Licence at:
   
   http://ec.europa.eu/idabc/eupl
   
   Unless required by applicable law or agreed to in writing, software distributed
   under the Licence is distributed on an "AS IS" basis, 
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the Licence for the specific language governing permissions and limitations
   under the Licence.
   
   ****************************************************************************************
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"  version="2.0" 
   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:functx="http://www.functx.com"
   xmlns:xs="http://www.w3.org/2001/XMLSchema"
   xmlns:msg="http://www.sonnenglanz.net/2010/07/09/smf.services.2.0.xsd"
   xmlns:tns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
   xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:xlink="http://www.w3.org/1999/xlink"
   xsi:schemaLocation="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd                  http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
   xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias"
   exclude-result-prefixes="xsl functx axsl">
   
   <xsl:output indent="yes" encoding="utf-8" method="xml"/>
   
   <xsl:include href="functions.xsl"/>
   
   <xsl:param name="profile">Profile</xsl:param>
   <xsl:param name="fromRole"><xsl:value-of select="//msg:rolebindings/msg:binding[1]/@name"/></xsl:param>
   <xsl:param name="toRole"><xsl:value-of select="//msg:rolebindings/msg:binding[2]/@name"/></xsl:param>
   
   <!--
      ****************************************************************************************
      Identifiers are derived from the input, so some cleanup is required before using them. 
      ****************************************************************************************
   -->
   <xsl:template match="/">
      <xsl:param name="profile" select="$profile"/> 
      <xsl:param name="fromRoleBinding" select="$fromRole"/>
      <xsl:param name="toRoleBinding" select="$toRole"/>
     <xsl:call-template name="createNDD">
        <xsl:with-param name="profile" select="$profile"/>
        <xsl:with-param name="fromRoleBinding" select="$fromRoleBinding"/>
        <xsl:with-param name="toRoleBinding" select="$toRoleBinding"/>
     </xsl:call-template>
   </xsl:template>
   
   <xsl:template name="createNDD">
      <xsl:param name="profile"/> 
      <xsl:param name="fromRoleBinding" select="//msg:rolebindings/msg:binding[1]/@name"/>
      <xsl:param name="toRoleBinding" select="//msg:rolebindings/msg:binding[2]/@name"/>
      <!--
         *******************************************************************************************************************
         README
         The following line was used to create more elaborate filenames.
         <xsl:variable name="CpaTemplate" select="concat('cpa.',functx:createID($fromRoleBinding), '-', functx:createID($toRoleBinding), '.xml')"/>
         
         Actually, because all files are already 'isolated' based on foldernames, we can use simple filenames.
         The filename static.
         *******************************************************************************************************************
      -->
      <xsl:variable name="CpaTemplate" select="'cpa-template.xml'"/>
      <xsl:variable name="fromparameterId" select="//msg:rolebindings/msg:binding[@name=$fromRoleBinding]/@parameterId"/>
      <xsl:variable name="toparameterId" select="//msg:rolebindings/msg:binding[@name=$toRoleBinding]/@parameterId"/>
      
      <NegotiationDescriptorDocument
         xmlns:tns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
         version="0.2" NddId="{concat( $fromRoleBinding, '-', $toRoleBinding)}">
         <Documents>
            <Document id="cpa" href="{$CpaTemplate}" type="cpa" version="2.0"
               ns="http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd"
            > </Document>
         </Documents>

         <Partner role="{$fromRoleBinding}">
            <xsl:apply-templates
               select="//msg:parameters/msg:group[@parameterId=$fromparameterId]/msg:parameter">
               <xsl:with-param name="partner" select="$fromRoleBinding"/>
               <xsl:with-param name="otherpartner" select="$toRoleBinding"/>
            </xsl:apply-templates>
         </Partner>

         <Partner role="{$toRoleBinding}">
            <xsl:apply-templates
               select="//msg:parameters/msg:group[@parameterId=$toparameterId]/msg:parameter">
               <xsl:with-param name="partner" select="$toRoleBinding"/>
               <xsl:with-param name="otherpartner" select="$fromRoleBinding"/>
            </xsl:apply-templates>
         </Partner>

      </NegotiationDescriptorDocument>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='CPAID']">
      <xsl:param name="partner"/>
      <Parameter name="CPAID" required="{attribute::required}">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="tns:cpaid"
            xmlContext="tns:CollaborationProtocolAgreement">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>
   
   <xsl:template match="msg:parameter[@name='StartDate']">
      <xsl:param name="partner"/>
      <Parameter name="StartDate" required="{attribute::required}">
         <Binding docref="cpa" xmlType="text"
            xmlContext="tns:CollaborationProtocolAgreement/tns:Start">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>
   
   <xsl:template match="msg:parameter[@name='EndDate']">
      <xsl:param name="partner"/>
      <Parameter name="EndDate" required="{attribute::required}">
         <Binding docref="cpa" xmlType="text"
            xmlContext="tns:CollaborationProtocolAgreement/tns:End">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>
   
   <xsl:template match="msg:parameter[@name='PartyName']">
      <xsl:param name="partner"/>
      <Parameter name="PartyName" required="{attribute::required}">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="tns:partyName"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='ProcessName']">
      <xsl:param name="partner"/>
      <Parameter name="ProcessName" required="{attribute::required}">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="tns:name"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:CollaborationRole/tns:ProcessSpecification">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='ProcessHref']">
      <xsl:param name="partner"/>
      <Parameter name="ProcessHref" required="{attribute::required}">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="xlink:href"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:CollaborationRole/tns:ProcessSpecification">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>
   
   <xsl:template match="msg:parameter[@name='PartyId']">
      <xsl:param name="partner"/>
      <Parameter name="PartyId" required="{attribute::required}">
         <Binding docref="cpa" xmlType="text"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:PartyId">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='PartyRef']">
      <xsl:param name="partner"/>
      <Parameter name="PartyRef" required="{attribute::required}">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="xlink:href"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:PartyRef">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>
   
   <xsl:template match="msg:parameter[@name='PartyIdType']">
      <xsl:param name="partner"/>
      <Parameter name="PartyIdType" required="{attribute::required}">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="tns:type"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:PartyId">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='PartyIdList']">
      <xsl:param name="partner"/>
      <Parameter name="PartyIdList" required="{attribute::required}">
         <Binding docref="cpa" xmlType="valueList"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:PartyId">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='PartyIdNode']">
      <xsl:param name="partner"/>
      <Parameter name="PartyIdNode" required="{attribute::required}">
         <Binding docref="cpa" xmlType="replacenode"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:PartyId">
            <xsl:copy-of select="node()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='Service']">
      <xsl:param name="partner"/>
      <Parameter name="Service" required="{attribute::required}">
         <Binding docref="cpa" xmlType="text"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:CollaborationRole/tns:ServiceBinding/tns:Service">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>
   
   <xsl:template match="msg:parameter[@name='ServiceType']">
      <xsl:param name="partner"/>
      <Parameter name="ServiceType" required="{attribute::required}">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="tns:type"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:CollaborationRole/tns:ServiceBinding/tns:Service">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='PayloadCert']">
      <xsl:param name="partner"/>
      <Parameter name="PayloadCert" required="{attribute::required}">
         <Binding docref="cpa" xmlType="value"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Certificate[@tns:certId='{concat(functx:createID($partner), '_SigningCert')}']">
            <xsl:copy-of select="node()"/>
         </Binding>
         <Binding docref="cpa" xmlType="value"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Certificate[@tns:certId='{concat(functx:createID($partner), '_EncryptionCert')}']">
            <xsl:copy-of select="node()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='TransportCert']">
      <xsl:param name="partner"/>
      <Parameter name="TransportCert" required="{attribute::required}">
         <Binding docref="cpa" xmlType="value"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Certificate[@tns:certId='{concat(functx:createID($partner), '_ClientCert')}']">
            <xsl:copy-of select="node()"/>
         </Binding>
         <Binding docref="cpa" xmlType="value"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Certificate[@tns:certId='{concat(functx:createID($partner), '_ServerCert')}']">
            <xsl:copy-of select="node()"/>
         </Binding>
      </Parameter>
   </xsl:template>
   
   <xsl:template match="msg:parameter[@name='SigningCert']">
      <xsl:param name="partner"/>
      <Parameter name="SigningCert" required="{attribute::required}">
         <Binding docref="cpa" xmlType="value"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Certificate[@tns:certId='{concat(functx:createID($partner), '_SigningCert')}']">
            <xsl:copy-of select="node()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='EncryptionCert']">
      <xsl:param name="partner"/>
      <Parameter name="EncryptionCert" required="{attribute::required}">
         <Binding docref="cpa" xmlType="value"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Certificate[@tns:certId='{concat(functx:createID($partner), '_EncryptionCert')}']">
            <xsl:copy-of select="node()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='ClientCert']">
      <xsl:param name="partner"/>
      <Parameter name="ClientCert" required="{attribute::required}">
         <Binding docref="cpa" xmlType="value"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Certificate[@tns:certId='{concat(functx:createID($partner), '_ClientCert')}']">
            <xsl:copy-of select="node()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='ServerCert']">
      <xsl:param name="partner"/>
      <Parameter name="ServerCert" required="{attribute::required}">
         <Binding docref="cpa" xmlType="value"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Certificate[@tns:certId='{concat(functx:createID($partner), '_ServerCert')}']">
            <xsl:copy-of select="node()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='RootCert']">
      <xsl:param name="partner"/>
      <Parameter name="RootCert" required="{attribute::required}">
         <Binding docref="cpa" xmlType="value"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Certificate[@tns:certId='{concat(functx:createID($partner), '_RootCert')}']">
            <xsl:copy-of select="node()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='EndpointUri' ]">
      <xsl:param name="partner"/>
      <xsl:param name="otherpartner"/>
      <Parameter name="EndpointUri" required="{attribute::required}">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="tns:uri"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Endpoint[@tns:uri='{concat('http://',$partner, '_TransportUrl_PH')}']">
            <xsl:value-of select="concat('http://',descendant-or-self::text())"/>
         </Binding>
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="tns:uri"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Endpoint[@tns:uri='{concat('https://',$partner, '_TransportUrl_PH')}']">
            <xsl:value-of select="concat('https://',descendant-or-self::text())"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='EndpointUriHTTP' ]">
      <xsl:param name="partner"/>
      <xsl:param name="otherpartner"/>
      <Parameter name="EndpointUriHTTP" required="{attribute::required}">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="tns:uri"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Endpoint[@tns:uri='{concat('http://',$partner, '_TransportUrl_PH')}']">
            <xsl:value-of select="concat('http://',descendant-or-self::text())"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='ConditionalEndpointUri' ]">
      <xsl:param name="partner"/>
      <xsl:param name="otherpartner"/>
      <Parameter name="ConditionalEndpointUri" required="{attribute::required}" mode="conditional">
         <Binding docref="cpa" xmlType="attribute" xmlAttribute="tns:uri"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']//tns:Endpoint"
            xmlOtherPartnerContext="{$otherpartner}">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='RetryCount']">
      <xsl:param name="partner"/>
      <xsl:choose>
         <xsl:when test="@bothsides='true'">
            <Parameter name="Retries" required="{attribute::required}">
               <Binding docref="cpa" xmlType="text"
                  xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo/tns:DocExchange//tns:Retries">
                  <xsl:value-of select="descendant-or-self::text()"/>
               </Binding>
            </Parameter>         
         </xsl:when>
         <xsl:otherwise>
            <Parameter name="Retries" required="{attribute::required}">
               <Binding docref="cpa" xmlType="text"
                  xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:DocExchange//tns:Retries">
                  <xsl:value-of select="descendant-or-self::text()"/>
               </Binding>
            </Parameter>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='RetryInterval']">
      <xsl:param name="partner"/>
      <xsl:choose>
         <xsl:when test="@bothsides='true'">
            <Parameter name="RetryInterval" required="{attribute::required}">
               <Binding docref="cpa" xmlType="text"
                  xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo//tns:DocExchange//tns:RetryInterval">
                  <xsl:value-of select="descendant-or-self::text()"/>
               </Binding>
            </Parameter>
         </xsl:when>
         <xsl:otherwise>
            <Parameter name="RetryInterval" required="{attribute::required}">
               <Binding docref="cpa" xmlType="text"
                  xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:DocExchange//tns:RetryInterval">
                  <xsl:value-of select="descendant-or-self::text()"/>
               </Binding>
            </Parameter>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="msg:parameter[@name='PersistDuration']">
      <xsl:param name="partner"/>
      <Parameter name="PersistDuration" required="{attribute::required}">
         <Binding docref="cpa" xmlType="text"
            xmlContext="tns:CollaborationProtocolAgreement/tns:PartyInfo[@tns:partyName='{concat($partner, '_PartyName_PH')}']/tns:DocExchange//tns:PersistDuration">
            <xsl:value-of select="descendant-or-self::text()"/>
         </Binding>
      </Parameter>
   </xsl:template>


</xsl:stylesheet>
