<?xml version="1.0" encoding="UTF-8"?>
<!--
    
    ****************************************************************************************
    Helper functions for CPA Toolkit.
    June 2010
    
    European Union Public Licence - EUPL v.1.1
    
    Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl
    
    Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
    European Commission - subsequent versions of the EUPL (the "Licence");
    You may not use this work except in compliance with the Licence.
    You may obtain a copy of the Licence at:
    
    http://ec.europa.eu/idabc/eupl
    
    Unless required by applicable law or agreed to in writing, software distributed
    under the Licence is distributed on an "AS IS" basis, 
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the Licence for the specific language governing permissions and limitations
    under the Licence.
    
    ****************************************************************************************
    
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:functx="http://www.functx.com" 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:msg="http://www.sonnenglanz.net/2010/07/09/smf.services.2.0.xsd"
    >
    <!--
        ****************************************************************************************
        Identifiers are derived from the input, so some cleanup is required before using them. 
        ****************************************************************************************
    -->
    <xsl:function name="functx:capitalize-first" as="xs:string?"
        xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:sequence select="concat(upper-case(substring($arg,1,1)),substring($arg,2))"/>
    </xsl:function>

    <xsl:function name="functx:words-to-camel-case" as="xs:string"
        xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:sequence
            select="string-join((tokenize($arg,'\s+')[1], for $word in tokenize($arg,'\s+')[position() > 1] return functx:capitalize-first($word)),'')"
        />
    </xsl:function>

    <xsl:function name="functx:replace" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="this" as="xs:string?"/>
        <xsl:param name="by" as="xs:string?"/>
        <xsl:param name="in" as="xs:string?"/>
        <xsl:variable name="tail" select="substring-after($in, $this)"/>
        <xsl:variable name="pre" select="substring-before($in, $this)"/>
        <xsl:choose>
            <xsl:when test="$tail=''">
                <xsl:sequence select="$in"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="concat($pre, $by, $tail)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="functx:createID" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:sequence
            select="string-join((for $word in tokenize($arg,'[\s:./\-]+') return functx:capitalize-first($word)),'')"
        />
    </xsl:function>

    <xsl:function name="functx:cpaid" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="arg" as="xs:string?"/>
        <xsl:sequence
            select="string-join((for $word in tokenize($arg,'[\s:./\-]+') return upper-case($word)),'-')"
        />
    </xsl:function>

    <xsl:function name="functx:inheritDefault" xmlns:functx="http://www.functx.com">
        <xsl:param name="message"/>
        <xsl:param name="default"/>
        <xsl:param name="profiles"/>
        <xsl:variable name="profileName" select="$message/@profile"></xsl:variable>
        <xsl:variable name="profile" select="$profiles/msg:profile[@name=$profileName]"/>
        <xsl:variable name="transport">
            <xsl:choose>
                <xsl:when test="$message/@transport">
                    <xsl:value-of select="$message/@transport"/>
                </xsl:when>
                <xsl:when test="$profile/@transport">
                    <xsl:value-of select="$profile/@transport"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$default/@transport"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="security">
            <xsl:choose>
                <xsl:when test="$message/@security">
                    <xsl:value-of select="$message/@security"/>
                </xsl:when>
                <xsl:when test="$profile/@security">
                    <xsl:value-of select="$profile/@security"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$default/@security"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="retryCount">
                <xsl:choose>
                    <xsl:when test="$message/@retryCount">
                        <xsl:value-of select="$message/@retryCount"/>
                    </xsl:when>
                    <xsl:when test="$profile/@retryCount">
                        <xsl:value-of select="$profile/@retryCount"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$default/@retryCount"/>
                    </xsl:otherwise>
                </xsl:choose>
        </xsl:variable>
        <xsl:variable name="retryInterval">
                <xsl:choose>
                    <xsl:when test="$message/@retryInterval">
                        <xsl:value-of select="$message/@retryInterval"/>
                    </xsl:when>
                    <xsl:when test="$profile/@retryInterval">
                        <xsl:value-of select="$profile/@retryInterval"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$default/@retryInterval"/>
                    </xsl:otherwise>
                </xsl:choose>
        </xsl:variable>
        <xsl:element name="message">
            <xsl:attribute name="transport" select="$transport"/>            
            <xsl:attribute name="security" select="$security"/>
            <xsl:if test="$transport='reliable'">
                <xsl:attribute name="retryCount" select="$retryCount"/>
                <xsl:attribute name="retryInterval" select="$retryInterval"/>
            </xsl:if>
        </xsl:element>
    </xsl:function>

    <xsl:function name="functx:createChannelID" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="id" as="xs:string?"/>
        <xsl:param name="message"/>
        <xsl:variable name="myRetry">
            <xsl:if test="$message/@transport='reliable'"><xsl:value-of select="concat($message/@retryCount, ' ', $message/@retryInterval)"/></xsl:if>
        </xsl:variable>
        <xsl:sequence
            select="functx:createID(concat('_DC ', ' _T_ ', $message/@transport,' _S_ ', $message/@security, ' _R_ ', $myRetry))"
        />
    </xsl:function>

    <xsl:function name="functx:createDocExchangeID" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="id" as="xs:string?"/>
        <xsl:param name="message"/>
        <xsl:variable name="myRetry">
            <xsl:if test="$message/@transport='reliable'"><xsl:value-of select="concat($message/@retryCount, ' ', $message/@retryInterval)"/></xsl:if>
        </xsl:variable>
        <xsl:variable name="mySecurity">
            <xsl:choose>
                <xsl:when test="contains($message/@security, 'payload')"><xsl:value-of select="'SignAndEncryptPayload'"/></xsl:when>
                <xsl:when test="contains($message/@security, 'sign')"><xsl:value-of select="'SignPayload'"/></xsl:when>
            </xsl:choose>
        </xsl:variable>
        <xsl:sequence
            select="functx:createID(concat('_DE_S_ ', $mySecurity,' _R_ ', $myRetry))"
        />
    </xsl:function>

    <xsl:function name="functx:ackRequested" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="transport" as="xs:string?"/>
        <xsl:variable name="value">
            <xsl:choose>
                <xsl:when test="$transport='reliable'">
                    <xsl:value-of select="'always'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'never'"></xsl:value-of>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:sequence select="$value"/>
    </xsl:function>
    
    <xsl:function name="functx:ackSignatureRequested" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="transport" as="xs:string?"/>
        <xsl:param name="security" as="xs:string?"/>
        <xsl:variable name="value">
            <xsl:choose>
                <xsl:when test="$transport='reliable' and (contains($security, 'payload') or contains($security, 'sign'))">
                    <xsl:value-of select="'always'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'never'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:sequence select="$value"/>
    </xsl:function>
    
    <xsl:function name="functx:transportSecurity" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="security" as="xs:string?"/>
        <xsl:variable name="value">
            <xsl:choose>
                <xsl:when test="contains($security, 'transport')">
                    <xsl:value-of select="'HTTPS'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'HTTP'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:sequence select="$value"/>
    </xsl:function>

    <xsl:function name="functx:isAuthenticated" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="security" as="xs:string?"/>
        <xsl:variable name="value">
            <xsl:choose>
                <xsl:when test="(contains($security, 'payload') or contains($security, 'sign'))">
                    <xsl:value-of>transient-and-persistent</xsl:value-of>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of>transient</xsl:value-of>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:sequence select="$value"/>
    </xsl:function>
    <xsl:function name="functx:isNonRepudiationRequired" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="security" as="xs:string?"/>
        <xsl:variable name="value">
            <xsl:choose>
                <xsl:when test="(contains($security, 'payload') or contains($security, 'sign'))">
                    <xsl:value-of>true</xsl:value-of>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of>false</xsl:value-of>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:sequence select="$value"/>
    </xsl:function>
    <xsl:function name="functx:isConfidential" as="xs:string" xmlns:functx="http://www.functx.com">
        <xsl:param name="security" as="xs:string?"/>
        <xsl:variable name="value">
            <xsl:choose>
                <xsl:when test="(contains($security, 'payload'))">
                    <xsl:value-of>transient-and-persistent</xsl:value-of>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of>transient</xsl:value-of>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:sequence select="$value"/>
    </xsl:function>

    <xsl:function name="functx:Duration2Seconds" as="xs:decimal" xmlns:functx="http://www.functx.com">
        <!--
            Calculate the duration in seconds.
        -->
       <xsl:param name="duration" as="xs:duration?"/>
        <xsl:variable name="seconds" select="seconds-from-duration($duration)"/>
        <xsl:variable name="minutes" select="60 * minutes-from-duration($duration)"/>
        <xsl:variable name="hours" select="3600 * hours-from-duration($duration)"/>
        <xsl:variable name="days" select="24 * 3600 * days-from-duration($duration)"/>
        <xsl:variable name="months" select="31* 24 * 3600 * months-from-duration($duration)"/>
        <xsl:sequence select="$seconds+$minutes+$hours+$days+$months"/>
    </xsl:function>

    <xsl:function name="functx:deriveSafePersistDuration" as="xs:string" xmlns:functx="http://www.functx.com">
        <!--
            Determine a safe value for the persist duration (assuming it is not provided by the user).
            A minimum persist duration of 7 days is assumed.
        -->
        <xsl:param name="count" as="xs:integer?"/>
        <xsl:param name="duration" as="xs:duration?"/>
        <xsl:sequence select="concat('P', 6+ceiling((($count+1) * functx:Duration2Seconds($duration)) div 86400), 'D')"/>
    </xsl:function>

    <xsl:function name="functx:deriveSafeTimeToPerform" as="xs:string" xmlns:functx="http://www.functx.com">
        <!--
            Determine a safe value for the time to perform duration (assuming it is not provided by the user).
            A minimum time to perform of 1 hour is assumed.
        -->
        <xsl:param name="count" as="xs:integer?"/>
        <xsl:param name="duration" as="xs:duration?"/>
        <xsl:sequence select="concat('PT', ceiling((($count+1) * functx:Duration2Seconds($duration)) div 3600), 'H')"/>
    </xsl:function>

</xsl:stylesheet>
