"""
    ******************************************************

	European Union Public Licence - EUPL v.1.1

	Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl

	Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
	European Commission - subsequent versions of the EUPL (the "Licence");
	You may not use this work except in compliance with the Licence.
	You may obtain a copy of the Licence at:

	http://ec.europa.eu/idabc/eupl

	Unless required by applicable law or agreed to in writing, software distributed
	under the Licence is distributed on an "AS IS" basis, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the Licence for the specific language governing permissions and limitations
	under the Licence.

    ******************************************************
"""

import os, sys, subprocess, string
import shutil
import xml.dom.minidom
import operator
import uuid
import utilities
import getRoles
import logging
import myProperties
import ConfigParser

def correctJavaPath(thisPath):
    return thisPath.replace(' ', u'%20') # Deal with the spaces in a path (Java is not capable in handling this)

class CpaToolkit:
    def __init__(self):
        config = myProperties.Properties()
        self.config = config
        if (config.error):
            print "ERROR collaboration.py; Could not initialize. Check configuration.<BR>"
            print config.error,"<BR>"
            return

        myPropertyFilePath  = config.ToolkitPath
        self.toolkitRoot    = os.path.abspath(myPropertyFilePath)
        self.errors         = ""
        self.results        = True
        logging.basicConfig(level=logging.DEBUG,
                            filename=os.path.join(self.toolkitRoot,'cpaToolkit.log'),
                            format='%(asctime)s %(message)s')
        self.logger         = logging.getLogger('')
        self.uuid           = str(uuid.uuid1())
        self.Executer       = utilities.Executer(self.toolkitRoot)

        if self.config.repository['debug'] == 'Verbose':
            self.debug = True
        else:
            self.debug = False

        if self.debug:
            print "ToolkitRootFilePath: "+myPropertyFilePath+"<BR>"

        # Determine Java executable and Saxon jars
        self.rootFileIO = self.toolkitRoot
        javaexe         = os.path.join(myPropertyFilePath,self.config.repository['java'])
        saxon           = os.path.join(myPropertyFilePath,self.config.repository['saxon'])
        if self.debug:
            print "<BR>javaexe: "+javaexe+"<BR>saxon: "+saxon+"<BR>"
        self.xslt       = javaexe + ' -jar ' + saxon + ' -xi:on'
        
        # Create root for the filepath for the xslt's.
        self.root = self.toolkitRoot.replace('\\','/')

        # For all given profiles, define which XSLT to use...
        # Very Simple Solution provided below...
        profiles = self.config.repository['profiles'].split(";")
        self.profiles  = {}
        self.profiles[profiles[0].split(":")[0]] = profiles[0].split(":")[1]
        if self.debug:
            print "Profile List: ", self.profiles, "<BR>"

        self.createTemplateCpa = {}
        self.createTemplateCpa['1'] = os.path.join( self.rootFileIO, '1', 'xslt', self.config.repository['createTemplateCpa.1'])
        if self.debug:
            print "createTemplateCpa List: ", self.createTemplateCpa, "<BR>"

        self.createNdd = {}
        self.createNdd['1'] = os.path.join( self.rootFileIO, 'xslt', self.config.repository['createNdd.1'])

        self.compileNdd = {}
        self.compileNdd['1'] = os.path.join(self.rootFileIO, 'xslt', self.config.repository['compileNdd.1'])

        self.postprocessCPAID = {}
        self.postprocessCPAID['1'] = os.path.join(self.rootFileIO, 'xslt', self.config.repository['postprocessCPAID.1'])
        # End of simple solution...
        
    def Transform(self, stylesheet,input, resultDocument=None, resultDocumentDir=None, xsltparams=None):
        if resultDocument != None:
            output = ' >'+ resultDocument
        else:
            output = ' '
        if resultDocumentDir != None:
            resultDocumentExpr = "resultDocumentDir="+resultDocumentDir
        else:
            resultDocumentExpr = ''
        if xsltparams != None:
            paramstring = ''
            for p, v in xsltparams.iteritems():
                paramstring = paramstring + ' '+p+'='+v
        else:
            paramstring = ''
        command = self.xslt +' '+ input +' '+ stylesheet +" "+output+' '+resultDocumentExpr + paramstring
        if self.debug:
            print "<BR><BR>"+command+"<BR><BR>"

        if not self.Executer.cmd(command, stylesheet):
            self.results = False
            self.errors = self.errors + self.Executer.errors
            if self.debug:
                print "<BR><BR>Collaboration.py - Transform/Executer ERRORS:<BR>"+self.Executer.errors+"<BR><BR>"

class Params:
    def __init__(self, name, value):
        self.Name = name
        self.Value = value
        
class Binding:
    def __init__(self, role, party):
        self.role = role
        self.party = party
    def GetParty(self):
        return self.party
    def GetRole(self):
        return self.role

class Bindings:
    def __init__(self):
        self.bindings = []
    def Add(self, role, party):
        self.bindings.append(Binding(role, party))
    def Get(self):
        return self.bindings

class CreateTemplateCpa:
    def __init__(self, collaboration, identifier, fromRole, toRole, processSpecificationName=None, serviceName=None):
        self.collabName = fromRole+'-'+toRole
        self.toRole = toRole
        self.fromRole = fromRole
        roles = {}
        roles['fromRole'] = fromRole
        roles['toRole'] = toRole

        roles['profile'] = collaboration.profile
        if processSpecificationName <> None:
            roles['processSpecificationName'] = processSpecificationName
        if serviceName <> None:
            roles['serviceName'] = serviceName
        self.roles = roles

        self.collabRoot = os.path.join(collaboration.root, identifier, self.collabName)
        self.collabRootFileIO = os.path.join(collaboration.rootFileIO, identifier, self.collabName)
        if not os.path.isdir(self.collabRootFileIO):
            os.mkdir(self.collabRootFileIO)
           
        self.collabTemplate = os.path.join(collaboration.root, identifier, self.collabName, '')
        self.collabTemplateFileIO = os.path.join(collaboration.rootFileIO, identifier, self.collabName, '')
        if not os.path.isdir(self.collabTemplateFileIO):
            os.mkdir(self.collabTemplateFileIO)
        self.collabTemplateXslt = '"' + collaboration.rootXslt + '"' + identifier + '/' + self.collabName + '/'
        
        self.templateCpa = os.path.join(self.collabTemplate, 'cpa-template.xml')
        self.templateCpaFileIO = os.path.join(self.collabTemplateFileIO, 'cpa-template.xml')
        self.nddDoc = os.path.join(self.collabTemplate, 'ndd.xml') 
        self.nddDocFileIO = os.path.join(self.collabTemplateFileIO, 'ndd.xml')
        self.outputPath = self.collabTemplate
        self.outputPathXslt = 'file:///' + self.collabTemplateXslt

        if collaboration.hasTemplateCpa:
            # A template has been provided, so use that one instead...
            shutil.copy(collaboration.templateCpaFileName, self.templateCpaFileIO)
        else:
            # Create the Template for this potential CPA based on the profile CPA.
            collaboration.toolkit.Transform(collaboration.toolkit.createTemplateCpa[collaboration.profile],
                                            collaboration.messageFileName,
                                            resultDocument=self.templateCpa,
                                            resultDocumentDir=self.outputPath,
                                            xsltparams=self.roles )

    def GetTemplateCpa(self):
        return self.templateCpaFileIO
    
    def GetNdd(self):
        return self.nddDocFileIO
    
class CollaborationInstance:
    def __init__(self, identifier, collaboration):
        self.id = identifier
        self.collaboration = collaboration
        self.cpa = {} # For each bi-lateral collaboration one cpa file
        self.params = {} # For each instantiated role one parameter file
        self.collabRoot = ()
        self.collabRootFileIO = ()
        self.cpaParamDirXslt = ()
        self.partyCountReceivedSoFar = 0
        self.partyParamList = {}
        self.roles = ()

    def GetIdentifier(self):
        return self.id
    
    def SelectAllFromMessages(self, processSpecificationName=None, serviceName=None):
        pairList = self.collaboration.allPairs
        for [roleA, roleB] in pairList:
            self.SelectFromMessages(roleA, roleB, processSpecificationName, serviceName)
        return
    
    def SelectFromMessages(self, fromRole, toRole, processSpecificationName=None, serviceName=None):
        self.cpa[fromRole, toRole] = CreateTemplateCpa(self.collaboration,
                                                       self.id,
                                                       fromRole,
                                                       toRole,
                                                       processSpecificationName,
                                                       serviceName)
        if self.cpa.has_key((fromRole,toRole)):
            cpa = self.cpa[fromRole,toRole]
            self.collaboration.toolkit.Transform(self.collaboration.toolkit.createNdd[self.collaboration.profile],
                                                 self.collaboration.parameterFileName,
                                                 resultDocument=cpa.nddDoc,
                                                 resultDocumentDir=cpa.outputPath,
                                                 xsltparams=cpa.roles )
            # Compile the created NDD document into an CPA xslt
            self.collaboration.toolkit.Transform(self.collaboration.toolkit.compileNdd[self.collaboration.profile],
                                                 cpa.nddDoc,
                                                 resultDocumentDir=cpa.outputPathXslt)
        else:
            print "No Template CPA Available for (",fromRole,",",toRole,")."
        
    def ProvideParameters(self, myRole, otherRole=None, paramValues=None):
        # If only myRole is given, it is assumed that a single parameter file is sufficient.
        # In case of intermediary messaging hubs, the parameters have to be provided in pairs.
        dstFile = myRole+'.parameters.xml'
        dstFilePath = os.path.join(self.collabRootFileIO, dstFile)
        self.params['ParametersSource'+myRole] = 'file:///'+ self.cpaParamDirXslt + '/' + dstFile
        if self.collaboration.koppeling:
            self.params['PARAM_CONDITION_'+myRole] = self.collaboration.koppeling
        shutil.copy(paramValues, dstFilePath)
        if not self.partyParamList.has_key(myRole):    
            self.partyCountReceivedSoFar = self.partyCountReceivedSoFar + 1
        self.partyParamList[myRole] = dstFilePath
        if self.AllParametersAvailable():
            self.SignalCreateCpa()
        
    def AllParametersAvailable(self):
        if self.partyCountReceivedSoFar == self.collaboration.partyCount:
            return True
        else:
            return False
        
    def SignalCreateCpa(self):
        iter = self.cpa.iteritems()
        if self.collaboration.toolkit.debug:
            print "Iterations: ", iter,"<BR>"
            print "Roles: ", self.roles,"<BR>"
        for ((k1, k2), v) in iter:
            if self.collaboration.toolkit.debug:
                print "Pair : (%s, %s)" % (k1, k2)
            self.CreateCpa(k1, k2)
            
    def changeCPAID(self, fromRole, toRole, myParams):
        result = self.cpa[fromRole,toRole].cpa
        source = os.path.join(result+'.temp')
        shutil.copy(result, source)
        stylesheet = self.collaboration.toolkit.postprocessCPAID[self.collaboration.profile]
        self.collaboration.toolkit.Transform(stylesheet, source, resultDocument=result, xsltparams=myParams)
        
    def CreateCpa(self, fromRole, toRole):
        DOMtree = xml.dom.minidom.parse(self.cpa[fromRole,toRole].nddDocFileIO)
        NddId = DOMtree.getElementsByTagName("NegotiationDescriptorDocument").item(0).getAttribute("NddId")
        Documents = DOMtree.getElementsByTagName("Document")

        result = EvalResult(id=self.id)
        for Document in Documents:
            docid = Document.getAttribute('id')
            type = Document.getAttribute('type')

            # Case 1: referenced template
            sourcef = Document.getAttribute('href')
            source = os.path.join(self.cpa[fromRole,toRole].collabTemplate, sourcef)

            stylesheetf = 'cpa-template.xsl'
            stylesheet = '"' + os.path.join( self.cpa[fromRole,toRole].collabTemplateFileIO, stylesheetf) + '"'

            outputf = NddId+'.'+docid+'.xml'
            output = os.path.join(self.cpaParamDirXslt,outputf)
            self.cpa[fromRole,toRole].cpa = output
            
            self.collaboration.toolkit.Transform(stylesheet, source, resultDocument=output, xsltparams=self.params)
            result.addOutput(docid, file=output)
        DOMtree.unlink()
        return result

    def GetCpa(self, fromRole, toRole, destination=None):
        if destination == None:
            return self.cpa[fromRole,toRole].cpa
        else:
            (destinationFilePath, destinationFileName) = os.path.split(destination)
            if not os.path.exists(destinationFilePath):
                os.mkdir(destinationFilePath)
            shutil.copy(self.cpa[fromRole,toRole].cpa, destination)
            return destination
      
class Collaboration:
    def __init__(self, cpatoolkit, profileName, Name, role, koppeling=None, CPAID=None, CPAVersion=None, StartDate=None, EndDate=None, assignedRoles=None):
        if not ((role == 'collaboration') or (role == 'participant') or (role == 'adhoc') or (role == 'specifier')):
            role = 'temp'
        self.koppeling = koppeling
        self.debug = cpatoolkit.debug
        self.toolkit = cpatoolkit
        self.status = False
        self.instances = {}
        self.profile = profileName
        self.name = Name
        self.cpaID = CPAID
        self.cpaVersion = CPAVersion
        self.startDate = StartDate
        self.endDate = EndDate
        self.root = os.path.join(self.toolkit.rootFileIO, profileName, role[0], Name)
        self.rootFileIO = os.path.join(self.toolkit.rootFileIO, profileName, role[0], Name)
        self.rootXslt = self.toolkit.root + '/' + profileName + '/'+role[0]+'/' + Name + '/'
        self.messageFileName = os.path.join(self.root, Name+'.msg.xml')
        self.parameterFileName = os.path.join(self.root, Name+'.msg.xml')
        self.templateCpaFileName = os.path.join(self.rootFileIO, Name+'.cpa-template.xml')
        self.hasTemplateCpa = False
        self.partyCount = ()
        self.collaborationPairs = {}
        self.userrole = None
        self.allRoles = []
        self.allPairs = []
        self.validInput = True
        #if not self.hasTemplateCpa:
        #    self.isValidSpecification(assignedRoles)
        if self.debug:
            print "Koppeling type: "+self.koppeling+"<BR>"
            print "Root: "+self.root+"<BR>"
            print "RootFileIO: "+self.rootFileIO+"<BR>"
            print "RootXslt: "+self.rootXslt+"<BR>"
            print "validInput: ",self.validInput,"<BR>"

    def checkRoles(self):
        check = False
        if not self.hasTemplateCpa:
            myMsg = getRoles.ServiceSpecification()
            if os.path.exists(self.messageFileName):
                check = True
                if not myMsg.set(self.messageFileName, 'smf'):
                    check = False
                    if self.debug:
                        print "Error: "+myMsg.errorMessage()+"<br>"
                    self.validInput = False
                elif not myMsg.get():
                    check = False
                    if self.debug:
                        print "Error: "+myMsg.errorMessage()+"<br>"
                    self.validInput = False
                else:
                    myPairs = myMsg.getPairs()
                    if not myPairs:
                        check = False
                        if self.debug:
                            print "Error: "+myMsg.errorMessage()+"<br>"
                        self.validInput = False
                    else:
                        self.allPairs = myPairs
                    myRoles = myMsg.getRoles()
                    if not myRoles:
                        check = False
                        if self.debug:
                            print "Error: "+myMsg.errorMessage()+"<br>"
                        self.validInput = False
                    else:
                        self.allRoles = myRoles
                        if self.debug:
                            print "Roles ", myRoles
        return check

    def isOK(self):
        return self.toolkit.results

    def myErrors(self):
        return self.toolkit.errors
    
    def Debug(self):
        return self.debug

    def isValidSpecification(self, file=None, assignedRoles=None):
        if not file:
            file = self.messageFileName
        #
        if not self.hasTemplateCpa:
            myMsg = getRoles.ServiceSpecification(file)
            if self.debug:
                print "messageFileName: ",self.messageFileName,"<br>"
                print "getRoles.ServiceSpecification(): ",myMsg,"<br>"
            if os.path.exists(file): #self.messageFileName)
                if not myMsg.set(file, 'smf'): #self.messageFileName
                    if self.debug:
                        print "Error: "+myMsg.errorMessage()+"<br>"
                    self.validInput = False
                elif not myMsg.get():
                    if self.debug:
                        print "Error: "+myMsg.errorMessage()+"<br>"
                    self.validInput = False
                else:
                    myRoles = myMsg.getFilteredRoles(assignedRoles)
                    if not myRoles:
                        if self.debug:
                            print "Error: "+myMsg.errorMessage()+"<br>"
                        self.validInput = False
                    else:
                        self.allRoles = myRoles
                        if self.debug:
                            print "Roles ", myRoles
                    myPairs = myMsg.getFilteredPairs(assignedRoles)
                    if not myPairs:
                        if self.debug:
                            print "Error: "+myMsg.errorMessage()+"<br>"
                        self.validInput = False
                    else:
                        self.allPairs = myPairs
        if self.debug:
            print "isValidSpecification: ", self.validInput, "<BR>"
        return self.validInput

    def getRoles(self):
        return self.allRoles

    def getPairs(self):
        return self.allPairs

    def Find(self):
        if (os.path.isdir(self.rootFileIO)) and (os.path.exists(self.rootFileIO)):
            return True
        return False

    def Create(self):
        if not os.path.isdir(self.rootFileIO):
            try:
                os.mkdir(self.rootFileIO)
            except:
                return False
        return True
    
    def StoreSequence(self, aSequence):
        filePath = os.path.join(self.rootFileIO, 'aSequence')
        utilities.SaveSequence(aSequence, filePath)

    def RetrieveSequence(self):
        filePath = os.path.join(self.rootFileIO, 'aSequence')
        return utilities.GetSequence(filePath)
    
    def SpecifyTemplateCpa(self, templateCpa):
        if cmp(os.path.normpath(templateCpa), os.path.normpath(self.templateCpaFileName)) <> 0:
            shutil.copy(templateCpa, self.templateCpaFileName)
            self.hasTemplateCpa = True

    def SpecifyMsg(self, messageSpecification):
        if cmp(os.path.normpath(messageSpecification), os.path.normpath(self.messageFileName)) <> 0:
            shutil.copy(messageSpecification, self.messageFileName)
            return self.checkRoles()

    def SpecifyParams(self, paramSpecification, numberOfPartiesInvolved):
        self.partyCount = numberOfPartiesInvolved
        if cmp(os.path.normpath(paramSpecification), os.path.normpath(self.parameterFileName))<> 0:
            #shutil.copy(paramSpecification, self.parameterFileName) # Since version 3.1 not needed: is included in the msg file
            return

    def SpecifyParameters(self, myRole, paramValues, myPartyId=None):
        # Save the parameter file for later use.
        dstFileName = self.name+'.parameters.xml'
        dstFilePath = os.path.join(self.rootFileIO)
        if not os.path.exists(dstFilePath):
            os.mkdir(dstFilePath)
        dstFile = os.path.join(dstFilePath, dstFileName)
        shutil.copy(paramValues, dstFile)

    def UseSpecification(self):
        if os.path.exists(self.templateCpaFileName):
            self.hasTemplateCpa = True
        return
        
    def UseParams(self, numberOfPartiesInvolved):
        self.partyCount = numberOfPartiesInvolved

    def UseParameters(self, myRole, myID=None):
        if myID == None:
            dstFile = self.name+'.parameters.xml'
            return os.path.join(self.rootFileIO, dstFile)
        else:
            rootFileIO = os.path.join(self.toolkit.rootFileIO, self.profile, 'p', myID)
            dstFile = myID+'.parameters.xml'
            paramFile = os.path.join(rootFileIO, dstFile)
            if os.path.exists(paramFile):
                return paramFile
            else:
                return None
        
    def SpecifyCollaborations(self, collaborationPairs, numberOfPartiesInvolved):
        self.partyCount = numberOfPartiesInvolved
        self.collaborationPairs = collaborationPairs

    def CreateInstance(self, identifier):
        myInstance = CollaborationInstance(identifier, self)
        myInstance.collabRoot = os.path.join(self.root, identifier)
        myInstance.collabRootFileIO = os.path.join(self.rootFileIO, identifier)
        myInstance.cpaParamDirXslt = self.rootXslt + identifier
        if not os.path.isdir(myInstance.collabRootFileIO):
            os.mkdir(myInstance.collabRootFileIO)
        self.instances[identifier] = myInstance
        return myInstance

    def DeleteInstance(self, identifier):
        if not self.instances[identifier]:
            return
        myInstance = self.instances[identifier]
        
        if not (myInstance.collabRootFileIO and os.path.isdir(myInstance.collabRootFileIO)):
            return

        for root, dirs, files in os.walk(myInstance.collabRootFileIO, topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
                #print "Remove file: "+os.path.join(root, name)+"<BR>"
            for name in dirs:
                os.rmdir(os.path.join(root, name))
                #print "Remove folder: "+os.path.join(root, name)+"<BR>"
        os.rmdir(myInstance.collabRootFileIO)
        #print "Remove folder: "+myInstance.collabRootFileIO+"<BR>"
        return

class EvalResult:
    def __init__(self, id=None):
        self.files = {}
        self.doms = {}
        if id != None:
            self.id = id

    def addOutput(self, document, dom=None, file=None):
        self.files[document] = file
        self.doms[document] = dom
        pass

    def dom(self, document ):
        return self.doms[document]

    def location(self, document):
        return self.files[document]

    def prettyPrint(self):
        print self.id
        print self.files

class EvalParams:
    def __init__(self, id=None):
        self.paramfiles = ()

    def add(self, param, file):
        self.paramfiles[param] = file

    def listall(self):
        res = ''
        return res