"""
    ******************************************************

	European Union Public Licence - EUPL v.1.1

	Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl

	Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
	European Commission - subsequent versions of the EUPL (the "Licence");
	You may not use this work except in compliance with the Licence.
	You may obtain a copy of the Licence at:

	http://ec.europa.eu/idabc/eupl

	Unless required by applicable law or agreed to in writing, software distributed
	under the Licence is distributed on an "AS IS" basis, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the Licence for the specific language governing permissions and limitations
	under the Licence.

    ******************************************************
"""

import utilities
import os
import sys
import shutil
import uuid
import myProperties
import messages

def _(id, language):
    return messages.Translation(id, language)

class Cpa2Xml:
    def __init__(self, Executer, myJavaExe=None, interactive=False, myTmpDir=None, option=None):
        self.Executer = Executer
        config = myProperties.Properties()
        self.config = config
        if (config.error):
            print "Could not initialize.<BR>"
            print config.error,"<BR>"
            return

        myPropertyToolkitFilePath  = config.ToolkitPath
        myJava = os.path.join(myPropertyToolkitFilePath,config.repository['java'])
        self.mySaxon = os.path.join(myPropertyToolkitFilePath,config.repository['saxon'])

        XSLTfilePath = config.ResultTemplateDir
        if not myJavaExe:
            self.javaExe    = myJava
        else:
            self.javaExe    = myJavaExe
        
        if myTmpDir:
            self.tmpDir     = myTmpDir
        else:
            self.tmpDir     = os.path.join(XSLTfilePath, 'tmp')
            
        if option == "ketenkaart":
            self.xslt       = os.path.join(XSLTfilePath, 'cpa2kk.xsl')
        elif option == "mmd":
            self.xslt       = os.path.join(XSLTfilePath, 'cpa2mmd.xsl')
        elif option == "parameters":
            self.xslt       = os.path.join(XSLTfilePath, 'cpa2par.xsl')
        elif option == "excel":
            self.xslt       = os.path.join(XSLTfilePath, 'excel2msg.xsl')
        elif option == "ebbp2smf":
            self.xslt       = os.path.join(XSLTfilePath, 'ebbp2smf-v2.xsl')
        elif (option == 'infra-in') or (option == 'infra-out') or (option == 'infra-inout'):
            self.xslt       = os.path.join(XSLTfilePath, 'remove-certificates.xsl')
        else:
            self.xslt       = os.path.join(XSLTfilePath, 'cpa2msg.xsl')
        #print self.xslt,"\n"
            
        self.fileId         = str(uuid.uuid1())
        self.betterXmlFile  = os.path.join(self.tmpDir,self.fileId+'.filtered.xml')
        self.msgFile        = os.path.join(self.tmpDir,self.fileId+'.msg.xml')
        self.interactive    = interactive
        self.MsXmlFile      = ""
        self.status         = None
        self.message        = ""

    def transformXml(self, myFile, options=None):
        self.MsXmlFile      = os.path.abspath(myFile)
        if os.path.exists(self.MsXmlFile):
            self.status = 'InputFile'
        else:
            self.status = 'InvalidFile'
            self.message = self.message + _(16, self.config.Language)+"<BR>"
            if self.interactive:
                print self.status," ; ",self.message
        if not self.improve():
            if self.interactive:
                print self.status," ; ",self.message
        if not self.transform(input=self.betterXmlFile,stylesheet=self.xslt,resultDocument=self.msgFile,xsltparams=options):
            if self.interactive:
                print self.status," ; ",self.message
        return self.getMsgFile()
        
    def improve(self):
        try:
            microsoftExcelXml = utilities.getFile(self.MsXmlFile)
            betterXml = microsoftExcelXml.replace("xmlns=\"urn:schemas-microsoft-com:office:spreadsheet\"","xmlns:urn=\"urn:schemas-microsoft-com:office:spreadsheet\"")
            utilities.saveFile(betterXml, self.betterXmlFile)
            self.status     = 'Improved'
            return True
        except:
            self.message    = self.message + sys.exc_info()[1] + "<BR>"
            self.status     = 'NotImproved'
            return False
        return False

    def transform(self,input=None, stylesheet=None, resultDocument=None, resultDocumentDir=None, xsltparams=None):
        if not input:
            input=self.betterXmlFile
        if not stylesheet:
            stylesheet=self.xslt
        if resultDocument != None:
            output = ' > "'+ resultDocument + '" '
        else:
            output = ' '
        if resultDocumentDir != None:
            resultDocumentExpr = 'resultDocumentDir="'+resultDocumentDir+'"'
        else:
            resultDocumentExpr = ''
        if xsltparams != None:
            paramstring = ''
            for p, v in xsltparams.iteritems():
                paramstring = paramstring + ' '+p+'='+v
        else:
            paramstring = ''
        
        command = '"'+'"'+self.javaExe + '" -jar '+self.mySaxon+' "' + input + '" "' + stylesheet +'" '+output+' '+resultDocumentExpr + paramstring+'"'
        command = command.replace('\\','/')
        #print command
        
        try:
            self.Executer.cmd(command, stylesheet)
            if self.Executer.results:
                self.status         = 'Transformed'
                return True
            else:
                self.status         = 'TransformationError'
                self.message        = self.message + self.Executer.errors + "<BR>"
                return False
            return True
        except:
            self.status         = 'TransformationError'
            self.message        = self.message + sys.exc_info()[1] + "<BR>"
            return False
        return False
        
    def getMsgFile(self):
        if self.status == 'Transformed':
            return self.msgFile
        else:
            return None
        return None
    
    def copyMsgFile(self, outputFile):
        try:
            shutil.copy(self.getMsgFile(), outputFile)
            return True
        except:
            return False
        return False

def transformInteractive(scriptname, inputFile=None, outputFile=None, interactive=None, *args):
    if not inputFile:
        return
    
    if not interactive:
        interactive = False
    myTransformer = Excel2Xml(interactive=interactive)
    if myTransformer.transformXml(inputFile):
        if outputFile:
            if myTransformer.copyMsgFile(outputFile) and interactive:
                print "Could not copy msg file"
    print myTransformer.getMsgFile()
    print utilities.getFile(myTransformer.getMsgFile())
    print outputFile
        
if __name__ == '__main__':
    transformInteractive(*sys.argv)
