"""
    ******************************************************

	European Union Public Licence - EUPL v.1.1

	Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl

	Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
	European Commission - subsequent versions of the EUPL (the "Licence");
	You may not use this work except in compliance with the Licence.
	You may obtain a copy of the Licence at:

	http://ec.europa.eu/idabc/eupl

	Unless required by applicable law or agreed to in writing, software distributed
	under the Licence is distributed on an "AS IS" basis, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the Licence for the specific language governing permissions and limitations
	under the Licence.

    ******************************************************
"""

import cgi
import os, sys, time
import uuid
import shutil
import cpa2msg
import utilities
import myProperties
import messages

def _(id, language):
    return messages.Translation(id, language)

ssloffloading_ui_message = {
(u'nl','infra-inout')   : u'(<i>In- en uitgaand</i>)',
(u'nl','infra-in')      : u'(<i>Ingaand</i>)',
(u'nl','infra-out')     : u'(<i>Uitgaand</i>)',
(u'en','infra-inout')   : u'(<i>In- and outbound</i>)',
(u'en','infra-in')      : u'(<i>Inbound</i>)',
(u'en','infra-out')     : u'(<i>Outbound</i>)',
(u'nl','notchanged')     : u'(<i>Niet veranderd</i>)',
(u'en','notchanged')     : u'(<i>Not changed</i>)',
}

class CreateWorker:
    def __init__(self):
        self.status         = True
        self.errormessage   = ""
        self.notification   = ""

        # Configuration...
        config = myProperties.Properties()
        self.config = config
        if (config.error):
            self.errormessage = "ERROR cpa2msgworker.py; Could not initialize, reading property file.<BR>"
            self.errormessage = self.errormessage + config.error + "<BR>"
            self.status = False
            return
        
        apacheDir           = config.htdocs
        apacheUrl           = config.Url
        uploadDir           = config.UploadDir
        downloadDir         = config.DownloadDir
        downloadUrl         = config.DownloadUrl
        uniqueid            = str(uuid.uuid1())
        self.uniqueid       = uniqueid
        self.cpaAutoImport  = True

        #Prepare directories and url
        
        resultFilePath      = config.ResultTemplateDir
        self.HTMLresultFilePath  = "" # Depends on the language config.HTMLResultTemplate
        self.resultFilePath = os.path.abspath(os.path.join(resultFilePath))
        self.htmlBaseUrl    = "" # Depends on the language config.HTMLBaseUrl
        self.upload_dir     = os.path.join(uploadDir,uniqueid)
        self.download_dir   = os.path.join(downloadDir,uniqueid)
        self.download_url   = downloadUrl + '/' + uniqueid + '/'

        self.formType       = 'msg'
        self.myCpaForm      = None
        self.refID          = None
        self.SMTPServer     = config.SMTPServer
        self.myEmail        = config.emailCpaManager
        self.xml            = ""
        self.link           = "Show/Download Message"
        self.options        = "all"
        self.mmdFileDropPath= config.fileDropPath
        self.mmdPartyId     = config.partyId
        self.cpaImport      = config.cpaAutoImport
        self.cpaImportOther = config.cpaAutoImportOther
        self.newEndpoint    = config.NewEndpoint
        self.cpaAutoImport  = False
        self.payloadPath    = config.payloadPath
        # Executer environment (including logging)
        if os.path.exists(os.path.abspath(config.ToolkitPath)):
            self.Executer   = utilities.Executer(os.path.abspath(config.ToolkitPath))
        else:
            self.errormessage = self.errormessage + "ERROR cpa2msgworker.py; Could not initialize the executer:<BR>"
            self.errormessage = self.errormessage + _(79, u'en') +"<BR>Please check the configuration.<BR>"
            self.status = False

    def PrepareErrorPages(self, errorTemplate):
        """
        Prepare ERROR html page
        """
        filePath=os.path.join(self.config.HTMLResultTemplate,self.formType, errorTemplate)
        errorPage = utilities.stringObject(filePath)
        if not errorPage.isOK():
            self.status = False
            self.errormessage = self.errormessage + "Could not read the template:<BR>"+filePath+"<BR>Please check the configuration.<BR>"
            return False

        self.dataERR = errorPage.getString()
        return True

    def PresentErrorPages(self):
        """
        Present ERROR html page
        """
        dataHEAD = self.dataERR
        dataHEAD = dataHEAD.replace('__XMLINPUT_PH__', self.xml)
        dataHEAD = dataHEAD.replace('__NOTIFICATION_PH__', self.notification)
        dataHEAD = dataHEAD.replace('__ERR_PH__',self.errormessage)
        dataHEAD = dataHEAD.replace('__PARTYID_PH__', self.mmdPartyId)
        dataHEAD = dataHEAD.replace('__FILEDROP_PH__', self.mmdFileDropPath)
        if self.options in ['infra-in', 'infra-out', 'infra-inout']:
            dataHEAD = dataHEAD.replace('__SSLOFFLOADING_PH__', ssloffloading_ui_message[(self.config.Language,self.options)])
        if self.newEndpoint:
            dataHEAD = dataHEAD.replace('__NEWENDPOINT_PH__', self.newEndpoint)
        else:
            dataHEAD = dataHEAD.replace('__NEWENDPOINT_PH__', ssloffloading_ui_message[(self.config.Language,u'notchanged')])
        if self.cpaAutoImport:
            dataHEAD = dataHEAD.replace('__CPAIMPORT_PH__', self.cpaImport)
            dataHEAD = dataHEAD.replace('__CPAIMPORTOTHER_PH__', self.cpaImportOther)
        else:
            dataHEAD = dataHEAD.replace('__CPAIMPORT_PH__', '-')
            dataHEAD = dataHEAD.replace('__CPAIMPORTOTHER_PH__', '-')

        print dataHEAD
        return True

    def Presentation(self, resultTemplate):
        """
        Present Resultpage
        """
        filePath=os.path.join(self.config.HTMLResultTemplate,self.formType, resultTemplate)
        resultPage = utilities.stringObject(filePath)
        if not resultPage.isOK():
            self.status = False
            self.errormessage = self.errormessage + "Could not read the template:<BR>"+filePath+"<BR>Please check the configuration.<BR>"
            return False

        dataHEAD = resultPage.getString()
        dataHEAD = dataHEAD.replace('__XMLINPUT_PH__', self.xml)
        dataHEAD = dataHEAD.replace('__DOWNLOADURL_PH__', self.download_url)
        dataHEAD = dataHEAD.replace('__NOTIFICATION_PH__', self.notification)
        dataHEAD = dataHEAD.replace('__PARTYID_PH__', self.mmdPartyId)
        dataHEAD = dataHEAD.replace('__FILEDROP_PH__', self.mmdFileDropPath)
        if self.options in ['infra-in', 'infra-out', 'infra-inout']:
            dataHEAD = dataHEAD.replace('__SSLOFFLOADING_PH__', ssloffloading_ui_message[(self.config.Language,self.options)])
        if self.newEndpoint:
            dataHEAD = dataHEAD.replace('__NEWENDPOINT_PH__', self.newEndpoint)
        else:
            dataHEAD = dataHEAD.replace('__NEWENDPOINT_PH__', ssloffloading_ui_message[(self.config.Language,u'notchanged')])
        if self.cpaAutoImport:
            dataHEAD = dataHEAD.replace('__CPAIMPORT_PH__', self.cpaImport)
            dataHEAD = dataHEAD.replace('__CPAIMPORTOTHER_PH__', self.cpaImportOther)
        else:
            dataHEAD = dataHEAD.replace('__CPAIMPORT_PH__', '-')
            dataHEAD = dataHEAD.replace('__CPAIMPORTOTHER_PH__', '-')

        print dataHEAD
        return True

    def ProcessForm(self):
        if not os.path.exists(self.upload_dir):
            #print "mk upload dir", self.upload_dir, "<BR>"
            os.mkdir(self.upload_dir)
        # endif
        if not os.path.exists(self.download_dir):
            #print "mk download dir",self.download_dir,"<BR>"
            os.mkdir(self.download_dir)
        # endif

        form = cgi.FieldStorage()
        # Checkbox autoImport
        if (form.has_key('autoImport') and form['autoImport'].value != ""):
            self.cpaAutoImport = True
            #print self.cpaAutoImport,"\n"
        # endif
            
        if (form.has_key('language') and form['language'].value != ""):
            language = (form['language'].value)[0:2]
        else:
            language = u'en' # Default
        # endif
        
        # Set the proper language and file path/url
        self.config.setLanguage(language)
        self.htmlBaseUrl = self.config.HTMLBaseUrl

        # If SSL-offloading, check the newEndpoint field (maybe, use default! See property file)
        if not (form.has_key('defaultendpoint') and form['defaultendpoint'].value != ""):
            self.newEndpoint = None # Do not change the endpoint in the CPA!
        #
        if form.has_key('endpoint') and form['endpoint'].value != "":
            self.newEndpoint = form['endpoint'].value
        # endif

        if form.has_key('options') and form['options'].value != "":
            options = form['options'].value
            if options in ['all','services','parameters','ketenkaart','mmd' , 'testmmd', 'infra-in', 'infra-out', 'infra-inout', 'excel', 'ebbp2smf']:
                self.options = options
            else:
                self.status = False
        # endif

        status = True
        if self.options == 'testmmd':
            # Prepare Test MMD error page
            status = self.PrepareErrorPages('error_cpa2mmd.html')
        elif (self.options == 'infra-in') or (self.options == 'infra-out') or (self.options == 'infra-inout'):
            # Prepare CPA Infra error page
            status = self.PrepareErrorPages('error_cpa2infra.html')
        elif self.options == 'excel':
            # Prepare Transform Excel into SMF error page
            status = self.PrepareErrorPages('error_excel2msg.html')
        elif self.options == 'ebbp2smf':
            # Prepare Transform ebBP into SMF error page
            status = self.PrepareErrorPages('error_ebbp2smf.html')
        else:
            # Prepare Transform CPA error page
            status = self.PrepareErrorPages('error_cpa2msg.html')
        # endif
        if not status:
            # Well, maybe we can't use our own error page while an error occurred...
            return False

        #=================================================================================
        if form.has_key('partyId') and form['partyId'].value != "":
            self.mmdPartyId = str(form['partyId'].value)
        # endif
        if (self.options=='mmd') or (self.options=='testmmd'):              
           #print "<BR>Filedrop path: ",self.mmdFileDropPath,"<BR>"
           if form.has_key('fileDropPath') and form['fileDropPath'].value != "":
               self.mmdFileDropPath = str(form['fileDropPath'].value)
           # endif
           self.mmdFileDropPath = self.mmdFileDropPath.replace(' ','%20')
           self.mmdFileDropPath = "file:///" + self.mmdFileDropPath.replace('\\','/')

           if not os.path.exists(self.mmdFileDropPath.replace("file:///","")):
               self.notification = self.notification + _(17, self.config.Language) + self.mmdFileDropPath.replace("file:///","") + "<BR>"
           #
           if not os.path.exists(self.payloadPath):
               self.errormessage = self.notification + "The MMD payload file does not exist (see property file, item 'mmd.payloadPath').<BR>"
               self.status = False
        #
        #
        if not (form.has_key('cpa2msg') and form['cpa2msg'].value != ""):
            self.errormessage = self.errormessage + utilities.redHtmlFormat(_(18, self.config.Language))+"<BR>"
            self.status = False
        else:
            xml = form['cpa2msg']
            self.xml  = form['cpa2msg'].filename
            #----------------------------------
            # MS *NIX IE en FIREFOR crazyness    
            index1 = self.xml.rfind('/')
            index2 = self.xml.rfind('\\')
            index = 0
            if index1 > -1:
                index = index1+1
            if index2 > -1:
                index = index2+1
            self.xml = self.xml[index:]
            #
            #----------------------------------
            # If something is wrong, we can also show the filename.
            if not self.status:
                self.PresentErrorPages()
                return True
            #----------------------------------
            #
            try:
                if not xml.file:
                    self.errormessage = self.errormessage + utilities.redHtmlFormat(_(19, self.config.Language))+"<BR>"
                    self.status = False
                else:
                    # fileMsg = form["FileMsg"].file.read()
                    # print "<h1>Hello FileMsg</h1>" #, cgi.escape(fileMsg)
                    if not os.path.isdir(self.upload_dir):
                        os.mkdir(self.upload_dir)
                    filePath= os.path.join(self.upload_dir, self.xml)
                    fout = file (filePath, 'wb')
                    #print "read input form<br>"
                    while 1:
                        chunk = xml.file.read(100000)
                        if not chunk: break
                        fout.write (chunk)
                    fout.close()
                    self.status = True
                    #
                    # --------------------------
                    # Auto Import?
                    if self.cpaAutoImport:
                        if (not os.path.exists(self.cpaImport)):
                            self.status = False
                            self.errormessage = self.errormessage + utilities.redHtmlFormat(_(20, self.config.Language)+self.cpaImport)+"<BR>"
                        if (not os.path.exists(self.cpaImportOther)):
                            self.status = False
                            self.errormessage = self.errormessage + utilities.redHtmlFormat(_(21, self.config.Language)+self.cpaImportOther)+"<BR>"
                        #
                        if not self.status:
                            self.PresentErrorPages()
                            return True
                        #
                        
                        shutil.copy(filePath,self.cpaImport)
                        shutil.copy(filePath,self.cpaImportOther)
                        interval = 1
                        
                        # Check import at local party
                        count = 0
                        while (os.path.exists(self.cpaImport)) and (count < 15):
                            time.sleep(interval)
                            count = count + 1
                        rejectedFile = os.path.join(self.cpaImport, 'rejected', self.xml)
                        if os.path.exists(rejectedFile):
                            self.status = False
                            self.errormessage = self.errormessage + utilities.redHtmlFormat(_(22, self.config.Language))+"<BR>"
                        # Check import at the other party
                        count = 0
                        while (os.path.exists(self.cpaImportOther)) and (count < 15):
                            time.sleep(interval)
                            count = count + 1
                        rejectedFile = os.path.join(self.cpaImportOther, 'rejected', self.xml)
                        if os.path.exists(rejectedFile):
                            self.status = False
                            self.errormessage = self.errormessage + utilities.redHtmlFormat(_(23, self.config.Language))+"<BR>"
                    # endif Auto Import
                # endif
            except:
                self.status = False
                self.errormessage = self.errormessage + utilities.redHtmlFormat(_(24, self.config.Language))+"<BR>"+sys.exc_info()[1]+"<BR>"
            # end try
        # endif

        # --------------------------------------------
        # Everything going well?
        if not self.status:
            self.PresentErrorPages()
            return True

        # ----------------------------------------------
        if self.options == 'all':
            messageFilename = self.xml+'.msg.xml'
        elif self.options == 'services':
            messageFilename = self.xml+'.services.xml'
        elif self.options == 'parameters':
            messageFilename = self.xml+'.parameters.xml'
        elif (self.options == 'mmd') or (self.options == 'testmmd'):
            messageFilename = self.xml+'.mmd-collection.xml'
        elif self.options == 'ketenkaart':
            messageFilename = self.xml+'.ketenkaart.xml'
        elif (self.options == 'infra-in') or (self.options == 'infra-out') or (self.options == 'infra-inout'):
            messageFilename = self.xml+'.infra.xml'
        elif self.options == 'excel':
            messageFilename = self.xml+'.msg.xml'
        elif self.options == 'ebbp2smf':
            messageFilename = self.xml+'.msg.xml'
        else:
            messageFilename = self.xml+'.unknown.xml'
            #print messageFilePath,"<BR>"
        options = self.options
        if options == 'testmmd':
            options = "mmd"

        #
        myOptions = {}
        myOptions['options']    = options
        myOptions['filepath']   = self.mmdFileDropPath
        myOptions['partner']    = self.mmdPartyId
        myOptions['location']   = self.payloadPath
        #
        if self.newEndpoint:
            myOptions['otherPartnerEndpoint'] = str(self.newEndpoint)
        else:
            myOptions['otherPartnerEndpoint'] = ""
        #print "Options: ",myOptions,"<br>",self.options,"<br>"

        # -------------------------
        # Transform
        myTransformer = cpa2msg.Cpa2Xml(self.Executer, myTmpDir=self.upload_dir,option=options)
        if myTransformer.transformXml(filePath,options=myOptions):
            if utilities.FileExists([myTransformer.getMsgFile()]):
                downloadFile = os.path.join(self.download_dir,messageFilename)
                myTransformer.copyMsgFile(outputFile=downloadFile)
                self.download_url = self.download_url+messageFilename
            else:
                self.errormessage = self.errormessage + _(25, self.config.Language) + "<br>"
                self.download_url = ''
                self.status = False
        else:
            self.errormessage = self.errormessage + _(26, self.config.Language) +"<br>" + myTransformer.message.replace("\n","<BR>")
            self.download_url = ''
            self.status = False
        self.notification = self.notification + str(myTransformer.message.replace("\n","<BR>"))

        # ----------------------------
        # Present web page with results
        if not self.status:
            self.PresentErrorPages()
            return True

        status = True
        if self.options == 'testmmd':
            status = self.Presentation('result_cpa2mmd.html')
        elif (self.options == 'infra-in') or (self.options == 'infra-out') or (self.options == 'infra-inout'):
            status = self.Presentation('result_cpa2infra.html')
        elif self.options == 'excel':
            status = self.Presentation('result_excel2msg.html')
        elif self.options == 'ebbp2smf':
            status = self.Presentation('result_ebbp2smf.html')
        else:
            status = self.Presentation('result_cpa2msg.html')

        # Check presentation
        if not status:
            return False # Somehow, the presentation did't work, so fallback to our 'basic' html.

        return True
