"""
    ******************************************************

	European Union Public Licence - EUPL v.1.1

	Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl

	Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
	European Commission - subsequent versions of the EUPL (the "Licence");
	You may not use this work except in compliance with the Licence.
	You may obtain a copy of the Licence at:

	http://ec.europa.eu/idabc/eupl

	Unless required by applicable law or agreed to in writing, software distributed
	under the Licence is distributed on an "AS IS" basis, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the Licence for the specific language governing permissions and limitations
	under the Licence.

    ******************************************************
"""

import collaboration
import os, sys
import uuid
import shutil
import datetime
import time
import utilities
import string
import re
import messages

def _(id, language):
    return messages.Translation(id, language)

class CreateCpa:
    def __init__(self):
        self.myCollab = None
        self.referenceID = None
        return
        
    def DeleteInstances(self):
        if self.myCollab and self.referenceID:
            self.myCollab.DeleteInstance(self.referenceID)
        return

    def CreateInstances(self, cpaType, *args):
        if ((cpaType == 'onestep') or (cpaType == 'cciat')):
            return self.createCPA_OneStep(*args)
        elif (cpaType == 'specifier'):
            return self.createCPA_Specifier(*args)
        elif (cpaType == 'sp'):
            return self.createCPA_SP(*args)
        elif (cpaType == 'sr') or (cpaType == 'ccirs'):
            return self.createCPA_SR(*args)
        elif (cpaType == 'participant'):
            return self.createCPA_Participant(*args)

        return [1, 'Onbekend CPA Formulier Type', '', '']

    def createCPA_OneStep(self, dir, myCpaForm, *args):
        arg2 = myCpaForm.partyA
        arg3 = myCpaForm.partyB
        arg4 = myCpaForm.profile
        arg5 = myCpaForm.name
        arg6 = myCpaForm.persoonsnaam
        arg7 = myCpaForm.email

        referenceID = myCpaForm.versie
        if ((referenceID == "") or (not referenceID)):
            referenceID = str(uuid.uuid1())
        startDatum = myCpaForm.start
        if not startDatum:
            startDatum = datetime.date.today().isoformat()+'T00:00:00Z'
        else:
            startDatum = datetime.datetime(*time.strptime(startDatum, "%Y-%m-%d")[0:6]).isoformat()+'Z'
            
        eindDatum = myCpaForm.eind
        if not eindDatum: 
            eindDatum = datetime.datetime(*time.strptime(startDatum, "%Y-%m-%dT00:00:00Z")[0:6]) + datetime.timedelta(weeks=104)
            eindDatum = eindDatum.isoformat()+'Z'
        else:
            eindDatum = datetime.datetime(*time.strptime(eindDatum, "%Y-%m-%d")[0:6]).isoformat()+'Z'
            
        params = {}
        params['CPA_STARTDATE'] = startDatum
        params['CPA_ENDDATE']   = eindDatum
        
        # Initiate one or more collaborations
        cpatoolkit = collaboration.CpaToolkit()
        if myCpaForm.specificationType == 'id':
            myCollab = collaboration.Collaboration(cpatoolkit, arg4, arg5, 'collaboration', myCpaForm.koppeling)
            myCollab.isValidSpecification()
            # Check the name
            if not myCollab.Find():
                return [1, _(72, myCpaForm.language), '', '']
            # Provide the Message Specification.
            myCollab.UseSpecification()
            # Provide the Parameter Specification.
            myCollab.UseParams(2)
        elif myCpaForm.specificationType == 'id2':
            myCollab = collaboration.Collaboration(cpatoolkit, arg4, arg5, 'collaboration', myCpaForm.koppeling, assignedRoles=myCpaForm.assignedRoles)
            #print arg4," ",arg5,"<BR>"
            # Check the name
            myCollab.isValidSpecification(assignedRoles=myCpaForm.assignedRoles)
            if not myCollab.Find():
                return [1, _(72, myCpaForm.language), '', '']
            # Provide the Message Specification.
            myCollab.UseSpecification()
            # Provide the Parameter Specification.
            myCollab.UseParams(len(myCpaForm.assignedRoles))
        else:
            myCollab = collaboration.Collaboration(cpatoolkit, arg4, arg5, 'adhoc', myCpaForm.koppeling, assignedRoles=myCpaForm.assignedRoles)
            myCollab.isValidSpecification()
            if myCollab.Debug():
                print "Ad Hoc<BR>"
                print "Assigned roles: ",myCpaForm.assignedRoles,"<BR>"
                print "Form: ", myCpaForm,"<BR>"
            #
            if not myCollab.Create():
                # Check the name    myCollab.Create():
                return [1, _(73, myCpaForm.language), '', '']
            myCollab.StoreSequence([myCpaForm.persoonsnaam,myCpaForm.email])
            #
            #-----------------------------------            
            # Specify the Message Specification.
            if (myCpaForm.specificationType == 'template'):
                myCollab.SpecifyTemplateCpa(os.path.join(dir,arg5+'.msg.xml'))
                # Specify the params.xml Parameters Specification.
                # Since version 3.1 not needed: is included in the msg file
                #myCollab.SpecifyParams(os.path.join(dir,arg5+'.cpa-template.xml'), numberOfPartiesInvolved=2)
            else:
                myCollab.SpecifyMsg(os.path.join(dir,arg5+'.msg.xml'))
                # Specify the params.xml Parameters Specification.
                # Since version 3.1 not needed: is included in the msg file
                #myCollab.SpecifyParams(os.path.join(dir,arg5+'.msg.xml'), numberOfPartiesInvolved=2)
            #
            #-----------------------------------
            # Check input specification
            if not myCollab.isValidSpecification():
                if myCollab.Debug():
                    print "CreateCpa.py: Invalid Input Specification"
                return [1, _(74, myCpaForm.language), '', '']
            #
            #-----------------------------------
            myCollab.UseSpecification()
            # Provide the Parameter Specification.
            myCollab.UseParams(2)

        aSeq = myCollab.RetrieveSequence()
        
        if not (myCpaForm.specificationType == 'id2'):
            myInstance = myCollab.CreateInstance(referenceID)
            #
            if myCpaForm.specificationType == 'smf':
                myInstance.SelectAllFromMessages()
                allPairs = myCollab.getPairs()
                allRoles = myCollab.getRoles()
                [roleA, roleB] = allPairs[0]
                if not (((roleA == arg2) and (roleB == arg3)) or ((roleA == arg3) and (roleB == arg2))):
                    error = utilities.redHtmlFormat(_(75, myCpaForm.language) + "(%s, %s) - versus - (%s, %s).\n" % (roleA, roleB, arg2, arg3))
                    return [1, error, '', '']
            elif myCpaForm.specificationType == 'template':
                # if a template has been given, we rely on the given rolenames in the input form!!!
                roleA = arg2
                roleB = arg3
                allPairs = []
                allPairs.append([arg2, arg3])
                allRoles = []
                allRoles.append(arg2)
                allRoles.append(arg3)
                myInstance.SelectFromMessages(arg2, arg3)
            else:
                return [1, 'ERROR createcpa.py; Unknown specification type (incorrect webform?)', '', '']
            #

            #=================================================================================
            # Debug info
            if myCollab.Debug():
                print "Roles found in specification: ", allRoles,"<BR>"
                print "Pairs found in specification: ", allPairs,"<BR>"
            #

            #=================================================================================
            # Parameters Partij A
            if (myCpaForm.partyIdA == None):
                parametersA = os.path.join(dir,arg2+'.parameters.xml')
                partyIDA = arg2
            else:
                parametersA = myCollab.UseParameters(arg2, myID=myCpaForm.partyIdA)
                partyIDA = myCpaForm.partyIdA
                if (parametersA == None):
                    return [1, _(76, myCpaForm.language) + "A.", '', '']
            try:
                myInstance.ProvideParameters(myRole=arg2, paramValues=parametersA)
            except:
                exceptionMessage = "\nAn exception occurred: \n"+str(sys.exc_info()[1])
                if not myCollab.isOK():
                    exceptionMessage = exceptionMessage +"\n\nDetails from the XSLT transformations\n"+myCollab.myErrors()+"<BR>"
                return [1, exceptionMessage, None, None]

            #=================================================================================
            # Parameters Partij B   
            if (myCpaForm.partyIdB == None):
                parametersB = os.path.join(dir,arg3+'.parameters.xml')
                partyIDB = arg3
            else:
                parametersB = myCollab.UseParameters(arg3, myID=myCpaForm.partyIdB)
                partyIDB = myCpaForm.partyIdB
                if (parametersSR == None):
                    return [1, _(76, myCpaForm.language) + "B.", '', '']
            try:
                myInstance.ProvideParameters(myRole=arg3, paramValues=parametersB)
            except:
                exceptionMessage = "\nAn exception occurred: \n"+str(sys.exc_info()[1])
                if not myCollab.isOK():
                    exceptionMessage = exceptionMessage +"\n\nDetails from the XSLT transformations\n"+myCollab.myErrors()+"<BR>"
                return [1, exceptionMessage, None, None]
            #myInstance.ProvideParameters(myRole=arg3, paramValues=parametersB)
            # Once all parameters are available, the CPA is created...

        else:
            myInstance = myCollab.CreateInstance(referenceID)
            myInstance.SelectAllFromMessages()
            allPairs = myCollab.getPairs()
            allRoles = myCollab.getRoles()

            # Debug info
            if myCollab.Debug():
                print "Roles found in specification: ", allRoles,"<BR>"
                print "Pairs found in specification: ", allPairs,"<BR>"

            count = 0
            for role in myCpaForm.assignedRoles.keys():
                if myCollab.Debug():
                    print "ROLE: ",role,"<BR>"
                count = count + 1
                party = myCpaForm.assignedRoles[role]
                parametersXmlFile = myCollab.UseParameters(role, myID=party) # [parametersXmlFile, aSequence] : uitbreiden met aSequence info
                if (parametersXmlFile == None):
                    return [1, _(76, myCpaForm.language)+party, None, None]
                try:
                    myInstance.ProvideParameters(myRole=role, paramValues=parametersXmlFile)
                except:
                    exceptionMessage = "\nAn exception occurred: \n"+str(sys.exc_info()[1])
                    if not myCollab.isOK():
                        exceptionMessage = exceptionMessage +"\n\nDetails from the XSLT transformations\n"+myCollab.myErrors()+"<BR>"
                    return [1, exceptionMessage, None, None]
            # Once all parameters are available, the CPA's are created...
            if (count < 2):
                return [1, _(63, myCpaForm.language), None, None]
                
        cpaList = []
        self.myCollab = myCollab
        self.referenceID = referenceID
            
        if (myCpaForm.versie == 'UUID'):
            versie = 'NoVersion'
        else:
            versie = str(myCpaForm.versie)
               
        assignedRoles = myCpaForm.assignedRoles
        if myCollab.Debug():
            print "Assigned roles: ", assignedRoles,"<BR>"

        # Check if the selected roles are part of a collaboration
        if not allPairs:
            return [1, _(87, myCpaForm.language), None, None]
        
        count = 1
        for [roleA, roleB] in allPairs:
            if myCollab.Debug():
                print "RoleA = ",roleA,":",myCpaForm.assignedRoles.has_key(roleA),"<BR>"
                print "RoleB = ",roleB,":",myCpaForm.assignedRoles.has_key(roleB),"<BR>"
            if not (myCpaForm.assignedRoles.has_key(roleA) and myCpaForm.assignedRoles.has_key(roleB)):
                continue    # We assume that these roles were not selected by the user...

            validArguments = True
            if not (myCpaForm.specificationType == 'id2'):
                partyIDA = "PartyA"
                partyIDB = "PartyB"
            elif myCpaForm.assignedRoles.has_key(roleA) and myCpaForm.assignedRoles.has_key(roleB):
                participantIDA = str(myCpaForm.assignedRoles[roleA])
                participantIDB = str(myCpaForm.assignedRoles[roleB])
                if myCollab.Debug():
                    print "A="+participantIDA+"<BR>"
                    print "B="+participantIDB+"<BR>"
                """
                indexA = participantIDA.rfind('.')
                indexB = participantIDB.rfind('.')
                """
                partyIDA = participantIDA
                """
                if indexA > -1:
                    partyIDA = participantIDA[participantIDA.rfind('.')+1:]
                """
                partyIDB = participantIDB
                """
                if indexB > -1:
                    partyIDB = participantIDB[participantIDB.rfind('.')+1:]
                if myCollab.Debug():
                    print "partyIDA="+partyIDA+"<BR>"
                    print "partyIDB="+partyIDB+"<BR>"
                """
            else:
                validArguments = False
                error = ""
                if not myCpaForm.assignedRoles.has_key(roleA):
                    error = _(77, myCpaForm.language)+roleA
                if not myCpaForm.assignedRoles.has_key(roleB):
                    error = error + _(77, myCpaForm.language)+roleB
                return [1, error+"\n\nDetails from the XSLT transformations\n"+myCollab.myErrors(), None, None]
            #
            if validArguments:    
                #cpaid = arg5.replace('.','-')+'.'+roleA+'-'+roleB+'.'+partyIDA+'-'+partyIDB+'.'+referenceID
                servicenaam = arg5
                if myCollab.Debug():
                    print "Service ID: ",servicenaam,"<BR>"
                servicetype = 'services:'
                i = servicenaam.find(servicetype)
                if i > -1:
                    servicenaam = servicenaam[i+len(servicetype):]
                servicenaam = string.join([string.capitalize(w) for w in (re.split( '[\.\ \-\:]',string.lower(servicenaam)))],'-')
                #
                #cpaid = servicenaam +'_'+roleA+'-'+partyIDA+'-'+roleB+'-'+partyIDB+'_'+referenceID.replace('-','')
                cpaid = servicenaam +'_'+partyIDA+'-'+partyIDB+'_'+referenceID.replace('-','')
                #print "CPAID ",cpaid,"<BR>"
                if myCpaForm.useAsCpaId:
                    if count > 1:
                        params['CPA_ID'] = referenceID.upper()+'-'+str(count)
                    else:
                        params['CPA_ID'] = referenceID.upper()
                else:
                    params['CPA_ID'] = ''
                    params['CPA_ID_POSTFIX'] = (referenceID.replace('-','')).upper() # cpaid.upper()
                #
                count = count + 1

                #====================================
                # Quick FIX to enable the definition of the default BASE HREF and BASE UUID as defined in the CPA TEMPLATE
                if myCpaForm.baseHREF:
                    params['BASE_HREF'] = myCpaForm.baseHREF
                else:
                    params['BASE_HREF'] = ""
                if myCpaForm.baseUUID:
                    params['BASE_UUID'] = myCpaForm.baseUUID
                else:
                    params['BASE_UUID'] = ""
                if myCpaForm.httpsIncrement:
                    params['HTTPS_PORTNUMBER_INCREMENT'] = myCpaForm.httpsIncrement
                else:
                    params['HTTPS_PORTNUMBER_INCREMENT'] = ""
                # End quick fix
                #====================================
                
                myInstance.changeCPAID(roleA, roleB, params)
                filename = cpaid[0:144]+'.xml'
                filename = filename.upper()
                file = os.path.join(dir,filename)
                myInstance.GetCpa(roleA, roleB, file)
                if myCollab.Debug():
                    print "Roles: ", roleA," ", roleB,"<BR>"
                    print "IDs: ",partyIDA," ", partyIDB,"<BR>"
                    print "CPA ID: ",params['CPA_ID'],"<BR>"
                    print "CPA filename: ",filename,"<BR>"
                    print "Collab Errors: ",myCollab.myErrors(),"<BR>"
                cpaList.append([file,roleA,roleB,partyIDA,partyIDB])
        #
        # END for [roleA, roleB] in allPairs:
        # =============================================

        if cpaList == []:
            error = "\n\nDetails from the XSLT transformations\n"+myCollab.myErrors()
            if (myCpaForm.specificationType == 'smf') or (myCpaForm.specificationType == 'template'):
                if not allPairs == []:
                    [roleA, roleB] = allPairs[0]
                    if not (((roleA == arg2) and (roleB == arg3)) or ((roleA == arg3) and (roleB == arg2))):
                        error = utilities.redHtmlFormat(_(75, myCpaForm.language)+ "(%s, %s) - versus - (%s, %s).\n" % (roleA, roleB, arg2, arg3)) + error
                else:
                    error = utilities.redHtmlFormat(_(78, myCpaForm.language))+error
            return [1, error, None, None]
        #
        if myCollab.isOK():
            return [0, cpaList, referenceID, aSeq]
        else:
            return [1, "\n\nDetails from the XSLT transformations\n"+myCollab.myErrors(), None, None]

        # End CPA creation
            
    def createCPA_Specifier(self, dir, myCpaForm, *args):
        arg2 = myCpaForm.partyA
        arg3 = myCpaForm.partyB
        arg4 = myCpaForm.profile
        arg5 = myCpaForm.name
        arg6 = myCpaForm.persoonsnaam
        arg7 = myCpaForm.email
        
        # Initiate a collaboration
        # Currently this is used for a particular collaboration,
        # but should be extended to take care of the collaboration ID.
        cpatoolkit = collaboration.CpaToolkit()    
        myCollab = collaboration.Collaboration(cpatoolkit, arg4, arg5, 'collaboration', koppeling=myCpaForm.koppeling)

        # Check input specification. Only works for SMF files!!! (Sorry)
        if not (myCpaForm.specificationType == 'template'):
            if not myCollab.isValidSpecification(file=os.path.join(dir,arg5+'.msg.xml')):
                if myCollab.Debug():
                    print "CreateCpa.py: Invalid Input Specification"
                return [1, _(74, myCpaForm.language), '', '']

        #
        # Check the name
        if not myCollab.Create():
            if myCollab.Debug():
                print "CreateCpa.py: Cannot create ID"
            return [1, _(73, myCpaForm.language), '', '']
        myCollab.StoreSequence([myCpaForm.persoonsnaam,myCpaForm.email])

        # Specify the Message Specification, select the roles in the collaboration.
        if (myCpaForm.specificationType == 'template'):
            myCollab.SpecifyTemplateCpa(os.path.join(dir,arg5+'.msg.xml'))
        else:
            myCollab.SpecifyMsg(os.path.join(dir,arg5+'.msg.xml'))
        
        # Specify the params.xml Parameters Specification.
        # myCollab.SpecifyParams(os.path.join(dir,arg5+'.params.xml'), numberOfPartiesInvolved=2)
        # Since v3.1 it is included in the msg file
        myCollab.SpecifyParams(os.path.join(dir,arg5+'.msg.xml'), numberOfPartiesInvolved=2)
        
        return [0, [], arg5, '']

    def createCPA_SP(self, dir, myCpaForm, *args):
        arg2 = myCpaForm.partyA
        arg3 = myCpaForm.partyB
        arg4 = myCpaForm.profile
        arg5 = myCpaForm.name
        arg6 = myCpaForm.persoonsnaam
        arg7 = myCpaForm.email
        
        # Initiate a collaboration
        # Currently this is used for a particular collaboration,
        # but should be extended to take care of the collaboration ID.
        cpatoolkit = collaboration.CpaToolkit()    
        myCollab = collaboration.Collaboration(cpatoolkit, arg4, arg5, 'collaboration')
        if not myCollab.Create():
        # Check the name    myCollab.Create():
            return [1, _(73, myCpaForm.language), '', '']
        myCollab.StoreSequence([myCpaForm.persoonsnaam,myCpaForm.email])

        # Specify the Message Specification, select the roles in the collaboration.
        if (myCpaForm.specificationType == 'template'):
            myCollab.SpecifyTemplateCpa(os.path.join(dir,arg5+'.msg.xml'))
        else:
            myCollab.SpecifyMsg(os.path.join(dir,arg5+'.msg.xml'))
        
        # Specify the params.xml Parameters Specification.
        myCollab.SpecifyParams(os.path.join(dir,arg5+'.params.xml'), numberOfPartiesInvolved=2)

        # Specify the parameters.xml for a certain role.   
        myCollab.SpecifyParameters(myRole=arg2, paramValues=os.path.join(dir,arg4+'.ndd.'+arg5+'.cpa.'+arg2+'.parameters.xml'))
        
        return [0, [], arg5, '']

    def createCPA_SR(self, dir, myCpaForm, *args):
        arg2 = myCpaForm.partyA
        arg3 = myCpaForm.partyB
        arg4 = myCpaForm.profile
        arg5 = myCpaForm.name
        arg6 = myCpaForm.persoonsnaam
        arg7 = myCpaForm.email
        
        # Use the specification as specified by the name
        myCpaForm.partyIdA = arg5

        # Initiate a collaboration
        # Currently this is used for a particular collaboration,
        # but should be extended to take care of the collaboration ID.
        cpatoolkit = collaboration.CpaToolkit()
        myCollab = collaboration.Collaboration(cpatoolkit, arg4, arg5, 'collaboration')
        # Check the name
        if not myCollab.Find():
            return [1, _(72, myCpaForm.language), '', '']
        aSeq = myCollab.RetrieveSequence()
        # Based on a predefined Message Specification, select the roles in the collaboration.
        myCollab.UseSpecification()
        
        # Provide the Parameter Specification.
        myCollab.UseParams(2)
        
        # Now I have to give my parameters.
        # I have prepared them in this example directory
        referenceID = str(uuid.uuid1())
        myInstance = myCollab.CreateInstance(referenceID)
        myInstance.SelectFromMessages(arg2, arg3)

        # Parameters SP
        if (myCpaForm.partyIdA == None):
            parametersSP = os.path.join(dir,arg4+'.ndd.'+arg5+'.cpa.'+arg2+'.parameters.xml')
        else:
            parametersSP = myCollab.UseParameters(arg2)
            if (parametersSP == None):
                return [1, _(72, myCpaForm.language), '', '']
        myInstance.ProvideParameters(myRole=arg2, paramValues=parametersSP)

        # Parameters SR    
        if (myCpaForm.partyIdB == None):
            parametersSR = os.path.join(dir,arg4+'.ndd.'+arg5+'.cpa.'+arg3+'.parameters.xml')
        else:
            parametersSR = myCollab.UseParameters(arg3, myID=myCpaForm.partyIdB)
            if (parametersSR == None):
                return [1, _(72, myCpaForm.language), '', '']
        myInstance.ProvideParameters(myRole=arg3, paramValues=parametersSR)
        # Once all parameters are available, the CPA is created...
        
        return [0, myInstance.GetCpa(arg2, arg3, os.path.join(dir,arg4+'.ndd.'+arg2+'_'+arg3+'.cpa.xml')), referenceID, aSeq]
        # End CPA creation

    def createCPA_Participant(self, dir, myCpaForm, *args):
        arg2 = myCpaForm.partyA
        arg3 = myCpaForm.partyB
        arg4 = myCpaForm.profile
        arg5 = myCpaForm.name
        arg6 = myCpaForm.persoonsnaam
        arg7 = myCpaForm.email
        
        cpatoolkit = collaboration.CpaToolkit()    
        myCollab = collaboration.Collaboration(cpatoolkit, arg4, arg5, 'participant', koppeling=myCpaForm.koppeling)
        if not myCollab.Create():
        # Check the name    myCollab.Create():
            return [1, _(73, myCpaForm.language), '', '']
        myCollab.StoreSequence([myCpaForm.persoonsnaam,myCpaForm.email])
        
        # Specify the parameters.xml for a certain role.   
        myCollab.SpecifyParameters(myRole=arg3, paramValues=os.path.join(dir,arg5+'.parameters.xml'))
        
        return [0, [], arg5, '']

