"""
Worker module to do the real work.
"""

"""
    ******************************************************

    European Union Public Licence - EUPL v.1.1

	Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl

	Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
	European Commission - subsequent versions of the EUPL (the "Licence");
	You may not use this work except in compliance with the Licence.
	You may obtain a copy of the Licence at:

	http://ec.europa.eu/idabc/eupl

	Unless required by applicable law or agreed to in writing, software distributed
	under the Licence is distributed on an "AS IS" basis, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the Licence for the specific language governing permissions and limitations
	under the Licence.

    ******************************************************
"""

import cgi
import cgitb; cgitb.enable()
import os, sys
import uuid
import createcpa
import shutil
import utilities
import cpaform
import sendmail
import myProperties
import messages

def _(id, language):
    return messages.Translation(id, language)

class CreateCpaWorker:
    def __init__(self):
        self.error          = ""
        self.status         = True
        config = myProperties.Properties()
        self.config = config
        if (config.error):
            #print "ERROR: createcpaworker.py; Could not initialize. Check configuration.","<BR>"
            #print config.error,"<BR>"
            self.status = False
            self.error = "ERROR: createcpaworker.py; Could not initialize, reading property file.<BR>"
            self.error = self.error + config.error + "<BR>"
            return

        apacheUrl           = config.Url 
        uploadDir           = config.UploadDir
        downloadDir         = config.DownloadDir
        downloadUrl         = config.DownloadUrl
        uniqueid            = str(uuid.uuid1())
        self.uniqueid       = uniqueid
        self.cpaAutoImport  = None
        self.dataERR        = ""

        """
        Prepare directories and url
        """
        self.resultFilePath = "" # Depends on language
        self.htmlBaseUrl    = "" # Depends on language
        self.upload_dir     = os.path.join(uploadDir,uniqueid)
        self.download_dir   = os.path.join(downloadDir,uniqueid)
        self.download_url   = downloadUrl + '/' + uniqueid + '/'
        self.formType       = None
        self.myCpaForm      = None
        self.refID          = ""
        self.notification   = ""
        self.SMTPServer     = config.SMTPServer 
        self.myEmail        = config.emailCpaManager 

        if not os.path.exists(self.upload_dir):
            os.mkdir(self.upload_dir)
        if not os.path.exists(self.download_dir):
            os.mkdir(self.download_dir)

    def PrepareErrorPages(self):
        self.status = True

        if (self.formType == 'cciat'):
            self.cpaAutoImport  = self.config.cpaAutoImport             

        """
        Prepare ERROR html page
        """
        subfolder = self.formType
        if (self.formType == 'onestep') or (self.formType == 'cciat'):
            if self.myCpaForm.specificationType == "smf":
                subfolder = "onestep"
            elif self.myCpaForm.specificationType == "template":
                subfolder = "onestep"
            elif (self.myCpaForm.specificationType == "id") or (self.myCpaForm.specificationType == "id2"):
                subfolder = "id"
        filePath=os.path.join(self.resultFilePath,subfolder,'error.html')
        errorPage = utilities.stringObject(filePath)
        if not errorPage.isOK():
            self.status = False
            self.error = "Could not read the template: "+filePath+"<BR>"
            return
        
        errorPage.replace('__ID_PH__', self.myCpaForm.name)
        errorPage.replace('__CPAVERSIEID_PH__', self.myCpaForm.versie)
        errorPage.replace('__CPASTART_PH__', self.myCpaForm.start)        
        errorPage.replace('__CPAEIND_PH__', self.myCpaForm.eind)
        errorPage.replace('__MSG_PH__', self.myCpaForm.msg)
        errorPage.replace('__PARAMS_PH__', self.myCpaForm.params)
        errorPage.replace('__PARAMETERSA_PH__', self.myCpaForm.paramsA)
        errorPage.replace('__PARAMETERSB_PH__', self.myCpaForm.paramsB)
        errorPage.replace('__PERSOONSNAAM_PH__', self.myCpaForm.persoonsnaam)
        errorPage.replace('__EMAIL_PH__', self.myCpaForm.email)
        errorPage.replace('__DOWNLOADURL_PH__', self.download_url)
        errorPage.replace('__REFID_PH__', self.refID)
        errorPage.replace('__NOTIFICATION_PH__', self.notification)
        errorPage.replace('__ERR_PH__',self.myCpaForm.errormessage+'__ERR_PH__')

        self.dataERR = errorPage.getString()

    def ProcessForm(self):
        # Class for creating cpa's
        cpaCreator = createcpa.CreateCpa()

        # Process the CPA web form
        # Take care of the use of a default Email and Name
        self.myCpaForm  = cpaform.CpaForm()
        self.myCpaForm.Process(self.upload_dir, emailName = self.config.DefaultName, emailAddress = self.config.DefaultEmail)

        # Set the proper language and file path/url
        self.config.setLanguage(self.myCpaForm.language)
        self.resultFilePath = self.config.HTMLResultTemplate
        self.htmlBaseUrl    = self.config.HTMLBaseUrl

        #====================================
        # Quick FIX to enable the definition of the default BASE HREF and BASE UUID as defined in the CPA TEMPLATE
        self.myCpaForm.baseHREF = self.config.baseHREF
        self.myCpaForm.baseUUID = self.config.baseUUID
        # HTTPS and HTTP port difference:
        self.myCpaForm.httpsIncrement = self.config.httpsIncrement
        # End quick fix
        #====================================

        # Form Type dependent post-processing:
        self.formType   = self.myCpaForm.formType
        # Prepare error page
        self.PrepareErrorPages()
        if not self.status:
            return False

        if (self.formType == 'collaboration'):
            self.dataERR = self.dataERR.replace('__ERR_PH__', "This form is not supported... yet...  :-(")
            print self.dataERR
            return True
        if not self.myCpaForm.status:
            self.dataERR = self.dataERR.replace('__ERR_PH__', "")
            print self.dataERR
            return True
        else:
            # Calculate the CPA
            if ((self.formType == 'onestep') or (self.formType == 'cciat')) and (self.myCpaForm.specificationType == "id2"):
                filepath = self.download_dir
            else:
                filepath = self.upload_dir

            """try:
                [result, cpaList, self.refID, aSeq] = cpaCreator.CreateInstances(self.formType,filepath,self.myCpaForm)
                #print result, "<BR>", cpaList, "<BR>", self.refID, "<BR>", aSeq
            except:
                [result, cpaList, self.refID, aSeq] = [1,str(sys.exc_info()[1]), None, None]
            #"""
            [result, cpaList, self.refID, aSeq] = cpaCreator.CreateInstances(self.formType,filepath,self.myCpaForm)

            if not result == 0:
                self.dataERR = self.dataERR.replace('__ERR_PH__', cpaList.replace('\n','<br/>'))
                print self.dataERR
                return True
            else:
                if (self.formType == 'sr') or (self.formType == 'cciat') or (self.formType == 'onestep'):
                    if not (self.myCpaForm.specificationType == "id2"):
                        cpaInfo = cpaList[0]
                        cpaFile = os.path.join(cpaInfo[0])
                        (cpaFilePath, cpaFileName) = os.path.split(cpaFile)
                        shutil.copy(cpaFile, os.path.join(self.download_dir,cpaFileName))
                        #print "Copy From ",cpaFile," To ",os.path.join(self.download_dir,cpaFileName)
                    if (self.formType == 'cciat') or (self.formType == 'ccirs'):
                        c = 1
                        for cpaFile in cpaList:
                            shutil.copy(cpaFile[0], os.path.join(self.cpaAutoImport,'cpa_'+self.uniqueid+'.'+str(c)+'.xml'))
                            c = c + 1

                # Do we use an SMTP server?
                if (not self.SMTPServer) or (self.SMTPServer == ''):
                    self.notification = _(1, self.config.Language)
                else:
                    self.notification = _(2, self.config.Language)

                # Inform Service Provider
                if (self.formType == 'sp'):
                    if not self.Notify(self.SMTPServer, "messageConfirm_template.txt", self.formType):
                        self.notification = self.notification + utilities.redHtmlFormat("<p>"+ _(3, self.config.Language)+"<BR>"+self.error)
                elif (self.formType == 'specifier'):
                    if not self.Notify(self.SMTPServer, "messageConfirmSpecifier_template.txt", self.formType):
                        self.notification = self.notification + utilities.redHtmlFormat("<p>"+_(4, self.config.Language)+"<BR>"+self.error)
                elif self.formType == 'participant':
                    if not self.Notify(self.SMTPServer, "messageConfirmParticipant_template.txt", self.formType):
                        self.notification = self.notification + utilities.redHtmlFormat("<p>"+_(5, self.config.Language)+"<BR>"+self.error)
                elif (self.formType == 'onestep') or (self.formType == 'cciat'):
                    if (self.myCpaForm.specificationType == "id"):
                        if aSeq:
                            [naamSP, mailSP,empty] = aSeq
                        else:
                            mailSP = None
                        if not self.Notify(self.SMTPServer, "messageIDNewCPA_template.txt", self.formType, emailSP=mailSP):
                            self.notification = self.notification + utilities.redHtmlFormat("<p>"+_(4, self.config.Language)+"<BR>"+self.error)
                    elif (self.myCpaForm.specificationType == "id2"):
                        if aSeq:
                            [naamSP, mailSP,empty] = aSeq
                        else:
                            mailSP = None
                        
                        for cpaFile in cpaList:
                            if utilities.FileExists(cpaFile):
                                (cpaFilePath, cpaFileName) = os.path.split(os.path.join(cpaFile[0]))
                                #print "Email CPA :"+cpaFileName+"<BR>"
                                download = self.download_url+cpaFileName
                                #print download+"<BR>"
                                if not self.Notify(self.SMTPServer, "messageIDNewCPA_template.txt", self.formType, emailSP=mailSP, myDownloadUrl=download):
                                    if self.SMTPServer:
                                        self.notification = self.notification + utilities.redHtmlFormat("<p>"+_(4, self.config.Language)+"<BR>"+self.error)
                                
                elif (self.formType == 'sr') or (self.formType == 'ccirs'):
                    if aSeq:
                        [naamSP, mailSP,empty] = aSeq
                    else:
                        mailSP = None
                    if not self.Notify(self.SMTPServer, "messageNewCPA_template.txt", self.formType, emailSP=mailSP):
                        self.notification = self.notification + utilities.redHtmlFormat("<p>"+_(6, self.config.Language)+"<BR>"+self.error)

                # Present web page with results
                self.Presentation(cpaList)
                # Should the results NOT be saved (in the repository)?
                if not self.myCpaForm.saveresults:
                    cpaCreator.DeleteInstances()
        #
        return True

    def Presentation(self, myCpaFileList):
        """
        Present Resultpage
        """
        subfolder = self.formType
        if(self.formType == 'onestep') or (self.formType == 'cciat'):
            if self.myCpaForm.specificationType == "smf":
                subfolder = "onestep"
            elif self.myCpaForm.specificationType == "template":
                subfolder = "template"
            elif (self.myCpaForm.specificationType == "id") or (self.myCpaForm.specificationType == "id2"):
                subfolder = "id"

        filePath=os.path.join(self.resultFilePath,subfolder,'result.html')
        f = open(filePath,'rb')
        dataHEAD = f.read(-1)
        f.close()
        
        dataHEAD = dataHEAD.replace('__ID_PH__', self.myCpaForm.name)
        dataHEAD = dataHEAD.replace('__MSG_PH__', self.myCpaForm.msg)
        dataHEAD = dataHEAD.replace('__PARAMS_PH__', self.myCpaForm.params)
        dataHEAD = dataHEAD.replace('__PARAMETERSA_PH__', self.myCpaForm.paramsA)
        dataHEAD = dataHEAD.replace('__PARAMETERSB_PH__', self.myCpaForm.paramsB)
        dataHEAD = dataHEAD.replace('__PERSOONSNAAM_PH__', self.myCpaForm.persoonsnaam)
        dataHEAD = dataHEAD.replace('__EMAIL_PH__', self.myCpaForm.email)
        downloadHtml = utilities.getStringPartition(dataHEAD,'<!--DOWNLOAD_START-->','<!--DOWNLOAD_END-->')
        insertHtml = ""
        #for cpaFile in os.listdir(self.download_dir):
        failed = False
        for cpaFile in myCpaFileList:
            myString = downloadHtml
            (cpaFilePath, cpaFileName) = os.path.split(os.path.join(cpaFile[0]))
            (cpaFilePath2, downloadFolder) = os.path.split(cpaFilePath)
            if not utilities.FileExists(cpaFile):
                download_url = "<a href=\"/cpa/4/0/"+self.config.Language+"/id/failed.html\" "
                cpaFileName  = "title=\""+_(7, self.config.Language)+"\" target=\"_\">"+_(8, self.config.Language)+"</a>"
                myString = myString.replace('__DOWNLOADURL_PH__', download_url)
                failed = True
            else:
                download_url = "<a href=\""+self.download_url
                cpaFileName  = cpaFileName+"\" target=\"_blank\">CPA</a>"
                myString = myString.replace('__DOWNLOADURL_PH__', download_url)
            myString = myString.replace('__CPAFILENAME_PH__', cpaFileName)
            myString = myString.replace('__REFFOLDER_PH__', downloadFolder)
            myString = myString.replace('__CPAPARTIES_PH__', cpaFile[3]+' - '+cpaFile[4])
            myString = myString.replace('__CPAROLES_PH__', cpaFile[1]+' - '+cpaFile[2])
            insertHtml = insertHtml + myString
        replaceString = '<!--DOWNLOAD_START-->'+downloadHtml+'<!--DOWNLOAD_END-->'
        dataHEAD = dataHEAD.replace(replaceString,insertHtml)
        dataHEAD = dataHEAD.replace('__REFID_PH__', self.refID)
        if failed:
           self.notification = utilities.redHtmlFormat(_(86, self.config.Language)) + "<BR>" + self.notification
        dataHEAD = dataHEAD.replace('__NOTIFICATION_PH__', self.notification)

        print dataHEAD
        
    def Notify(self, SMTPServer, messageFile, formType, emailSP=None, myDownloadUrl=None):
        # Sending email is optional. See properties files. If no SMTP server defined: do nothing.
        if not SMTPServer:
            return True
        elif SMTPServer == "":
            return True
        
        """
        Send an Email to the Service Provider
        """
        placeholders = {}
        placeholders['RefID']       = self.refID
        if not myDownloadUrl:
            placeholders['DownloadID']  = self.download_url
        else:
            placeholders['DownloadID']  = myDownloadUrl
        placeholders['Naam']        = self.myCpaForm.persoonsnaam
        placeholders['Email']       = self.myCpaForm.email
        placeholders['EmailSP']     = emailSP

        if (formType == 'sp'):
            subject = _(9, self.config.Language)
        elif (formType == 'sr'):
            subject = _(10, self.config.Language)
        elif (formType == 'onestep'):
            subject = _(11, self.config.Language) 
        elif (formType == 'cciat'):
            subject = _(12, self.config.Language) 
        elif (formType == 'specifier'):
            subject = _(13, self.config.Language) 
        elif (formType == 'participant'):
            subject = _(14, self.config.Language) 
        else:
            subject = _(15, self.config.Language) 
        
        #"""
        #print "sendmail:",SMTPServer,":",messageFile,":",self.myEmail,":",subject,":",placeholders['Email'] ,":",placeholders['EmailSP']
        try:
            sendmail.SendMail('sendmail.py', SMTPServer, os.path.join(self.resultFilePath,messageFile), self.myEmail, subject, placeholders)
            #print "Email: "+placeholders['DownloadID']+"<BR>"
            return True
        except:
            self.error = str(sys.exc_info()[1])
            #print "Error sending mail"
            return False
        #"""
        #sendmail.SendMail('sendmail.py', SMTPServer, messageFile, self.myEmail, subject, placeholders)
    
        return False
    