"""
    ******************************************************

	European Union Public Licence - EUPL v.1.1

	Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl

	Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
	European Commission - subsequent versions of the EUPL (the "Licence");
	You may not use this work except in compliance with the Licence.
	You may obtain a copy of the Licence at:

	http://ec.europa.eu/idabc/eupl

	Unless required by applicable law or agreed to in writing, software distributed
	under the Licence is distributed on an "AS IS" basis, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the Licence for the specific language governing permissions and limitations
	under the Licence.

    ******************************************************
"""

import xml.dom.minidom
import sys
import os
import string

class Pairs:
    def __init__(self):
        self.pairs = {}
        self.roles = []
    def insertPair(self, a, b):
        if (a == b):
            return
        if a < b:
            c = a
            d = b
        else:
            c = b
            d = a
        #if not ([c,d] in self.pairs):
        #    self.pairs.append([c,d])
        # Add roles to role-list
        if not (c in self.roles):
            self.roles.append(c)
        if not (d in self.roles):
            self.roles.append(d)
        # Add pair to pair-list
        if self.pairs.has_key(c):
            if not (d in self.pairs[c]):
                self.pairs[c].append(d)
        elif self.pairs.has_key(d):
            if not (c in self.pairs[d]):
                self.pairs[d].append(c)
        else:
            self.pairs[c] = [d]
    def getPairs(self):
        pairs = self.pairs
        self.pairs = []
        for fromRole in pairs.keys():
            toRoles = pairs[fromRole]
            c = 0
            while c < len(toRoles):
                self.pairs.append([fromRole,toRoles[c]])
                c = c + 1
        return self.pairs
    def getRoles(self):
        return self.roles
            
class ServiceSpecification:
    def __init__(self, File=None):
        self.status     = 'Empty'
        self.service    = ""
        self.name       = ""
        self.type       = ""
        if File:
            self.file   = File
        else:
            self.file   = ""
        self.msg        = ""
        self.roles      = []
        self.pairs      = []
        self.filteredRoles      = []
        self.filteredPairs      = []
        self.filtered   = False
        self.error      = ""
        self.partners   = {}
        
    def getRoles(self):
        if self.status == 'Parsed':
            return self.roles
        elif self.status == 'Imported':
            if not self.getPairs():
                return self.roles
        else:
            self.error = "Error - getRoles.py: No specification available."
            return None

    def getFilteredRoles(self, assignedRoles):
        if self.filtered:
            return self.filteredRoles
        if not self.status == 'Parsed':
            self.getRoles()

        filteredRoles = []
        for role in self.roles:
            if assignedRoles:
                if assignedRoles.has_key(role):
                    filteredRoles.append(role)
            else:
                filteredRoles.append(role)

        self.filteredRoles = filteredRoles
        self.getFilteredPairs(assignedRoles)
        self.filtered = True
        return self.filtered

    def getFilteredPairs(self, assignedRoles):
        if self.filtered:
            return self.filteredPairs
        if not self.status == 'Parsed':
            return None

        filteredPairs = []
        for [A, B] in self.pairs:
            if assignedRoles:
                if assignedRoles.has_key(A) and assignedRoles.has_key(B):
                    filteredPairs.append([A,B])
            else:
                filteredPairs.append([A,B])

        self.filteredPairs = filteredPairs
        return self.filteredPairs

    def getRoleBindings(self):
        if self.status == 'Imported':
            try:
                roleBindingsElement = self.msg.getElementsByTagName("rolebindings")[0]
                partnerElements     = roleBindingsElement.getElementsByTagName("binding")
                roleBindings = {}
                for partner in partnerElements:
                    roleBindingName = partner.getAttribute('name')
                    partnerRoles = partner.getElementsByTagName("role")
                    for role in partnerRoles:
                        roleName = role.getAttribute('name')
                        if roleBindings.has_key(roleName):
                            if not (roleBindingName in roleBindings[roleName]):
                                roleBindings[roleName].append(roleBindingName)
                        else:
                            roleBindings[roleName] = [roleBindingName]
                return roleBindings
            except:
                self.error = "Error - getRoles.py: Could not derive roleBinding information."
                return {}
        return {}
    
    def getPairs(self):
        if self.status == 'Imported':               
            try:
                self.partners = self.getRoleBindings()
                
                messages = self.msg.getElementsByTagName("message")
                pairs = {}
                roles = []
                for message in messages:
                    fromRole = message.getAttribute('from')
                    toRole = message.getAttribute('to')
                    # Add roles to role-list
                    if not (fromRole in roles):
                        roles.append(fromRole)
                    if not (toRole in roles):
                        roles.append(toRole)
                    # Add pair to pair-list
                    if pairs.has_key(fromRole):
                        if not (toRole in pairs[fromRole]):
                            pairs[fromRole].append(toRole)
                    elif pairs.has_key(toRole):
                        if not (fromRole in pairs[toRole]):
                            pairs[toRole].append(fromRole)
                    else:
                        pairs[fromRole] = [toRole]
                self.status = 'Parsed'
                self.roles = roles
                for fromRole in pairs.keys():
                    toRoles = pairs[fromRole]
                    c = 0
                    while c < len(toRoles):
                        self.pairs.append([fromRole,toRoles[c]])
                        c = c + 1
                self.pairBindings()
                return self.pairs
            except:
                self.error = "Error - getRoles.py: Could not derive requested information."
                return None    
        elif self.status == 'Parsed':
            return self.pairs
        else:
            self.error = "Error - getRoles.py: No specification available."
            return None
        
    def pairBindings(self):
        if not self.partners:
            return
        
        newPairs = Pairs()
        partners = self.partners
        try:
            for [fromRole,toRole] in self.pairs:
                fromBindings = partners[fromRole]
                toBindings   = partners[toRole]
                for roleA in fromBindings:
                    for roleB in toBindings:
                        newPairs.insertPair(roleA, roleB)
        except:
            self.error= "Error - getRoles.py: Could not create newPairs in pairBindings."

        self.pairs = newPairs.getPairs()
        self.roles = newPairs.getRoles()
        return newPairs
    
    def set(self, filename, type):
        self.type = type # smf, template
        thisFile = os.path.abspath(os.path.join(filename))
        #print "thisFile: ", thisFile,"<BR>"
        if os.path.exists(thisFile):
            self.file       = thisFile
            self.status     = 'FileSpecified'
            return True
        else:
            self.error = "Error - getRoles.py: File "+filename+" cannot be accessed."
            return False
    def get(self):
        if not (self.status == 'FileSpecified'):
            self.error = "Error - getRoles.py: No filename specified."
            return False
        try:
            self.msg = xml.dom.minidom.parse(self.file)
            messages = self.msg.getElementsByTagName("services")[0]
            self.name = messages.getAttribute("process")
            self.service = messages.getAttribute("service")
            self.status = 'Imported'
            return True
        except:
            self.error = "Error - getRoles.py: Could not parse specified file"
            return False
    def toString(self):
        if self.status == 'Imported':
            return self.msg.toxml()
        else:
            return ""
    def clear(self):
        self.status = 'Empty'
    def errorMessage(self):
        return self.error
    def printPairs(self):
        print "** pairs:"
        for [fromRole,toRole] in self.pairs:
            print "CPA(",fromRole,",",toRole,")"
    def printRoles(self):
        print "** roles:"
        for role in self.roles:
            print role
    def printRoleBindings(self):
        print "** roleBindings:"
        print self.partners
        print "**"

if __name__ == '__main__':
    myMsg = ServiceSpecification()
    if not myMsg.set(sys.argv[1], 'smf'):
        print myMsg.errorMessage()
    if myMsg.get():
        print myMsg.errorMessage()
    myPairs = myMsg.getPairs()
    if myPairs:
        myMsg.errorMessage()
    myMsg.printPairs()
    myRoles = myMsg.getRoles()
    if not myRoles:
        print myMsg.errorMessage()
    myMsg.printRoles()
    #myMsg.printRoleBindings()
    #myMsg.pairBindings()
    