"""
KeyInfo Writer Functionality
"""

"""
    ******************************************************

    This program converts a public certificate (*.cer or *.p7b) into a KeyInfo.xml structure
    for use in the XMLDSIG structure within the CPA's.
    ==> It uses a Java program, provided by Axway (under the Apache 2.0 licence) to do the actual work!
    This python script is just a wrapper to call do a cmd execution.

    With special thanks to Axway for the open source contribution of the Java packages.
    (See lib2 folder in the repository under the software/java/keyinfo tree.)

    European Union Public Licence - EUPL v.1.1

	Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl

	Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
	European Commission - subsequent versions of the EUPL (the "Licence");
	You may not use this work except in compliance with the Licence.
	You may obtain a copy of the Licence at:

	http://ec.europa.eu/idabc/eupl

	Unless required by applicable law or agreed to in writing, software distributed
	under the Licence is distributed on an "AS IS" basis, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the Licence for the specific language governing permissions and limitations
	under the Licence.

    ******************************************************
"""

import os
import sys
import time
import datetime

class KeyInfoWriter:
    def __init__(self, Executer, myJava, toolkitroot=None):
        self.Executer = Executer
        libpath = os.path.join(toolkitroot, 'software', 'java', 'keyinfo')
        libstring = """ -classpath ../lib2/KeyInfoWriter-1.0.jar;\
../lib2/commons-logging-1.1.1.jar;../lib2/junit-4.7.jar\
../lib2/xml-apis-1.3.04.jar;../lib2/xercesImpl-2.9.1.jar;../lib2/xmlsec-1.4.3.jar \
com.axway.ebxml.KeyInfoWriter"""
        libstring = libstring.replace('..',libpath)
        if sys.platform == 'win32':
            self.javaExec = myJava +libstring
        else:
            self.javaExec = ''
        
    def Write(self, input, output):
        result = 1
        try:
            command = self.javaExec + ' "' + input + '" "' + output + '"'
            #print command + "<BR>"
            if sys.platform == 'win32':
                if self.Executer.cmd(command, '<no stylesheet>'):
                    return 0
            if sys.platform == 'linux2':
                retcode = os.system(command + ' > /dev/null')
                return retcode
        except:
            return 1
        
        return result