"""
    ******************************************************

	European Union Public Licence - EUPL v.1.1

	Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl

	Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
	European Commission - subsequent versions of the EUPL (the "Licence");
	You may not use this work except in compliance with the Licence.
	You may obtain a copy of the Licence at:

	http://ec.europa.eu/idabc/eupl

	Unless required by applicable law or agreed to in writing, software distributed
	under the Licence is distributed on an "AS IS" basis, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the Licence for the specific language governing permissions and limitations
	under the Licence.

    ******************************************************
"""

import os, sys, string
import ConfigParser

def absORrel(propertyString, propertyFile, rootFolder, section=None):
    if section:
        values = (propertyFile.get(section, propertyString)).split("|")
    else:
        print "ERROR myProperties.py; No section specified.<br>"
        return None
    thisPath = values[1]
    if values[0] == 'REL':
        return os.path.abspath(os.path.join(rootFolder, thisPath)) # Make it absolute
    elif values[0] == 'ABS':
        return os.path.abspath(thisPath) # Just in case...
    else:
        print "ERROR myProperties.py; Cannot read property ("+propertyString+"). Please indicate ABS or REL location.<br>"
        return None
    return None

class Properties:
    def __init__(self, language=None):
        self.error = None

        # The default location for version 4.0 is cgi-bin/cpa/4/0
        rootPath    = os.path.abspath(os.path.curdir)
        # The property file is located in the folder, relative to the parent folder of the cgi-bin folder: <parent-cgi-bin>/cct40/properties
        startPropertyFile2   = os.path.abspath(os.path.join(rootPath, os.path.join('..','..','..','..', 'cct40', 'properties', 'property40.ini')))
        if not os.path.exists(startPropertyFile2):
            self.error = "ERROR myProperties.py; Could not retrieve the file ("+startPropertyFile+") for the properties. Check configuration."
            return

        # Start reading the property file
        myPropertyPath          = startPropertyFile2
        self.myPropertyFile2    = ConfigParser.ConfigParser()
        self.myPropertyFile2.read(myPropertyPath)

        # The full path to the ROOT of the repository
        self.rootPath           = os.path.abspath(os.path.join(myPropertyPath, '..','..')) # The path relative to 'cct40'
        self.ToolkitPath        = absORrel('rootFilePath', self.myPropertyFile2, self.rootPath, section="Repository")
        self.repository         = {}
        repositoryItems         = ['java', 'saxon', 'debug', 'profiles', 'createTemplateCpa.1', 'createNdd.1', 'compileNdd.1', 'postprocessCPAID.1']
        for i in repositoryItems:
            self.repository[i] = self.myPropertyFile2.get("Repository", i)
            #print i," : ",self.repository[i],"<BR>"
        
        # Information about the available languages
        self.Language           = ""
        self.Languages          = (self.myPropertyFile2.get("UI", 'languages')).split("|")

        # Information about the Apache configuration
        self.htdocs             = absORrel('htdocsDir', self.myPropertyFile2, self.rootPath, section="Apache")
        self.Url                = self.myPropertyFile2.get("Apache", 'url')
        self.DownloadUrl        = self.myPropertyFile2.get("Apache", 'downloadUrl')
        self.UploadDir          = os.path.abspath(os.path.join(self.rootPath, self.myPropertyFile2.get("Apache",'uploadDir')))
        self.DownloadDir        = absORrel('downloadDir', self.myPropertyFile2, self.rootPath, section="Apache")
        self.ResultTemplateDir  = os.path.abspath(os.path.join(self.rootPath, self.myPropertyFile2.get("Apache", 'htmlResultTemplateDir')))
        self.HTMLResultTemplate = self.ResultTemplateDir # This will change according to the selected language
        self.HTMLBaseUrl        = self.Url # This will change according to the selected language

        # Information about the MMD generation
        self.cpaAutoImport      = os.path.abspath(os.path.join(self.rootPath, self.myPropertyFile2.get("MMD", 'cpaAutoImport')))
        self.cpaAutoImportOther = os.path.abspath(os.path.join(self.rootPath, self.myPropertyFile2.get("MMD", 'cpaAutoImportOther')))
        self.fileDropPath       = self.myPropertyFile2.get("MMD", 'fileDropPath')
        self.partyId            = self.myPropertyFile2.get("MMD",  'partyId')
        self.payloadPath        = self.myPropertyFile2.get("MMD", 'payloadPath')

        # Information for the default parameter file generation
        self.defaultPartyId     = self.myPropertyFile2.get("Parameters", 'partyId')
        self.defaultPartyIdType = self.myPropertyFile2.get("Parameters", 'partyIdType')
        self.defaultIntermediary= self.myPropertyFile2.get("Parameters", 'intermediary')

        # General information for SMTP
        self.emailCpaManager    = self.myPropertyFile2.get("SMTP", 'emailCpaManager')
        self.SMTPServer         = self.myPropertyFile2.get("SMTP", 'smtpServer')

        # Default information for the URL for the SSL offloading
        self.NewEndpoint        = self.myPropertyFile2.get("SSLOffloading", 'newEndpoint')

        # Default contact information
        self.DefaultName        = self.myPropertyFile2.get("Contact", 'name')
        self.DefaultEmail       = self.myPropertyFile2.get("Contact", 'email')

        # Default HREF base URL and the UUID base URN
        self.baseHREF           = self.myPropertyFile2.get("Process", 'baseHREF')
        self.baseUUID           = self.myPropertyFile2.get("Process", 'baseUUID')
        self.httpsIncrement     = self.myPropertyFile2.get("HTTPS", 'increment')
        #
        # Set default
        if language:
            self.setLanguage(language)

    def checkLanguage(self, language):
        if not language:
            return False
        for l in self.Languages:
            if l == language[0:2]:
                return True
        return False

    def setLanguage(self, language):
        if self.checkLanguage(language):
            self.Language = language[0:2]
        else:
            self.Language = u'en' # Default language

        # Change the HTML template file path en url.
        self.HTMLResultTemplate = os.path.abspath(os.path.join(self.HTMLResultTemplate, self.Language))
        self.HTMLBaseUrl        = self.HTMLBaseUrl + '/' + self.Language
        #
        return
