"""
    ******************************************************

	European Union Public Licence - EUPL v.1.1

	Copyright 2010 Justitiele Informatiedienst, EBV, The Netherlands, http://www.justid.nl

	Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the
	European Commission - subsequent versions of the EUPL (the "Licence");
	You may not use this work except in compliance with the Licence.
	You may obtain a copy of the Licence at:

	http://ec.europa.eu/idabc/eupl

	Unless required by applicable law or agreed to in writing, software distributed
	under the Licence is distributed on an "AS IS" basis, 
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the Licence for the specific language governing permissions and limitations
	under the Licence.

    ******************************************************
"""

import os, sys, subprocess
import time
import datetime
import string
#import path
import re
import uuid
import logging

def redHtmlFormat(str):
    return "<strong><font color=\"#FF0000\">"+str+"</font></strong>"

def validEmailAddress( email_address ):
    monster = "(?:[a-z0-9!#$%&'*+/=?^_{|}~-]+(?:.[a-z0-9!#$%" + \
    "&'*+/=?^_{|}~-]+)*|\"(?:" + \
    "[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]" + \
    "|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*\")@(?:(?:[a-z0-9]" + \
    "(?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?" + \
    "|\[(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.)" + \
    "{3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?" + \
    "|[a-z0-9-]*[a-z0-9]:(?:" + \
    "[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]"  + \
    "|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])"
    evaluator = re.compile(monster)
    if evaluator.match(email_address.lower()):
        return True
    return False

def FileExists(filePath):
    filePath = filePath[0].replace('file:///','')
    if os.path.exists(filePath):
        if (os.path.getsize(filePath) > 0):
            return True
    #
    return False

class stringObject:
    def __init__(self, filename):
        self.status = False
        self.string = ""
        if os.path.exists(os.path.join(filename)):
            try:
                filePath=os.path.join(filename)
                f = open(filePath,'rb')
                self.string = f.read(-1)
                f.close()
                self.status = True
            except:
                self.status = False
        else:
            self.status = False
    def isOK(self):
        return self.status
    def replace(self, matchThis, replaceBy):
        if replaceBy:
            self.string = self.string.replace(matchThis, replaceBy)
        else:
            self.string = self.string.replace(matchThis, "")
        return
    def getString(self):
        return self.string
    
def SaveSequence(mySequence, filename):
    f = open(filename,'wb')
    for item in mySequence:
        f.write(item+'|-|')
    f.close

def GetSequence(filename):
    try:
        f = open(filename, 'rb')
        mySequenceString = f.read()
        f.close
        if mySequenceString == "":
            return None
        return string.split(mySequenceString,'|-|')
    except:
        return None
    
def filterName(myName):
    allowedCharacters = '.-_ 0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
    return ''.join([c for c in myName if c in allowedCharacters])
    
def filterKey(myName):
    allowedCharacters = '.-_0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
    return ''.join([c for c in myName if c in allowedCharacters])
    
def filterEmail(myName):
    allowedCharacters = '.-+_@0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
    email_address = ''.join([c for c in myName if c in allowedCharacters])
    if validEmailAddress(email_address):
        return email_address
    else:
        return None
    
def filterMessageId(myName):
    allowedCharacters = '.-+_@0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
    return ''.join([c for c in myName if c in allowedCharacters])

def filterDatum(myName):
    allowedCharacters = '-0123456789'
    datum = ''.join([c for c in myName if c in allowedCharacters])
    if datum != myName:
        return None
    try:
        eenDatum = datetime.datetime(*time.strptime(datum, "%Y-%m-%d")[0:6]).isoformat()+'Z'
    except:
        return None
    return datum

def getFilename(filename):
    #----------------------------------
    # MS *NIX IE en FIREFOX crazyness    
    index1 = filename.rfind('/')
    index2 = filename.rfind('\\')
    index = 0
    if index1 > -1:
        index = index1+1
    if index2 > -1:
        index = index2+1
    filename=filename[index:]
    #print filename
    #----------------------------------
    return filename

def getDirectoryList(myFilePath):
    if (not myFilePath) or (not os.path.exists(myFilePath)):
        return None
    #dirs = path.path(myFilePath)
    dirs2 = os.path.abspath(myFilePath)
    #print dirs,"<BR>", dirs2,"<BR>"
    #dirnames = [getFilename(d) for d in dirs.listdir('*')]
    #dirnames.sort()
    dirnames2 = [getFilename(d) for d in os.listdir(dirs2)]
    dirnames2.sort()
    #print dirnames,"<BR>", dirnames2,"<BR>"
    return dirnames2

def getStringPartition(aString, startString, endString):
    (pre, sep, post) = aString.partition(startString)
    (pre, sep, post) = post.partition(endString)
    return pre

def getRole(filename):
    # Only works for Python version 2.5!
    (pre, sep, post) = filename.partition('.parameters.xml')
    index = pre.rfind('.')
    if index > -1:
        return pre[index+1:]
    else:
        return None
    
def myTime():
    return str(time.asctime())

def insertDropDown(myFile, myDir, myTemplate):
    filePath=os.path.join(myFile)
    f = open(filePath,'rb')
    myString = f.read(-1)
    f.close()
    myList = makeListFromTemplate(getDirectoryList(myDir), "<OPTION VALUE=\"__PH__\">__PH__","__PH__")
    return myString.replace(myTemplate, myList)

def makeListFromTemplate(myList, myTemplate, myString):
    if not myList:
        return u""
    resultString = u""
    for elem in myList:
        newString = myTemplate.replace(myString, elem)+'\n'
        resultString = resultString+newString
    return resultString

def getFile(myFile):
    filePath=os.path.join(myFile)
    if not os.path.exists(myFile):
        return ""
    f = open(filePath,'rb')
    myString = f.read(-1)
    f.close()
    return myString

def saveFile(myString, filename):
    f = open(filename,'wb')
    f.write(myString)
    f.close

def putFile(myFile, myContents):
    try:
        filePath=os.path.join(myFile)
        f = open(filePath,'wb')
        f.write(myContents)
        f.close()
        return True
    except:
        return False

class Executer:
    def __init__(self, toolkitRoot=None):
        self.uuid       = str(uuid.uuid1())
        self.errors     = ""
        self.results    = True
        if not toolkitRoot:
            self.errors = "The root-folder of the repository is not specified."
            self.results= False
            return
        self.toolkitRoot = toolkitRoot
        logging.basicConfig(level=logging.DEBUG,
                            filename=os.path.join(self.toolkitRoot,'cpaToolkit.log'),
                            format='%(asctime)s %(message)s')
        self.logger         = logging.getLogger('')
        return
    def cmd(self, command, stylesheet):
        self.results = True
        self.errors = ""
        thisError = ""
        s = ""
        try:
            # Somehow the call function does NOT append the output, so we need a workaround...
            fileNameErr = os.path.join(self.toolkitRoot, 'tmp', self.uuid+'.stderr.tmp')
            fileNameOut = os.path.join(self.toolkitRoot, 'tmp', self.uuid+'.stdout.tmp')
            fout = open(fileNameErr, "w")   # Somehow the call function does NOT append the output...
            ferr = open(fileNameOut, "w")   # Somehow the call function does NOT append the output...
            retcode = subprocess.call(command , shell=True, stdout=fout, stderr=ferr)   # Somehow the call function does NOT append the output...
            fout.close()
            ferr.close()
            if retcode == 1:
                self.results = False
            # stderr
            fin = open(fileNameErr)
            s = fin.read()
            fin.close()
            # stdout
            fin = open(fileNameOut)
            s = s+"\n"+fin.read()
            fin.close()
            # remove garbage
            os.remove(fileNameErr)
            os.remove(fileNameOut)
            # add to standard log
            if s == "\n":
                s = "Results: OK\n"
            else:
                s = "Results: Not OK\n"+s
                self.results = False
                self.errors = "**\n"+myTime()+stylesheet+"\n"+s
            thisError = "\n"+"Applying Stylesheet: "+stylesheet+"\n"+s
        except:
            thisError = "\nError; collaboration.py/Cpatoolkit/Transform: Command Execution failed.\n"+str(sys.exc_info()[1])+"\n"
            self.errors = "\n"+thisError+"Command:\n"+command+"\n"
            self.results = False
        # Log
        # This one appends the two into an existing file...
        self.logger.warning("--------------------------------------------------------\n"+thisError+command+"\n")
        return self.results

    